@@ -40,6 +40,7 @@
 #include "usermat.h"
 #include "gretl_string_table.h"
 #include "dbread.h"
+#include "boxplots.h"
 
 #ifdef WIN32
 # include <windows.h>
@@ -114,14 +115,19 @@ static void noalloc (void)
     exit(EXIT_FAILURE);
 }
 
-static int file_get_line (char *line, CMD *cmd)
+static int file_get_line (ExecState *s)
 {
-    int len;
+    char *line = s->line;
+    int len = 0;
 
     clear(line, MAXLINE);
-    fgets(line, MAXLINE, fb);
-
-    len = strlen(line);
+    
+    if (fgets(line, MAXLINE, fb) == NULL) {
+	/* no more input from current source */
+	gretl_exec_state_uncomment(s);
+    } else {
+	len = strlen(line);
+    }
 
     if (*line == '\0') {
 	strcpy(line, "quit");
@@ -137,7 +143,7 @@ static int file_get_line (char *line, CMD *cmd)
 	set_gretl_echo(0);
     }
 
-    if (gretl_echo_on() && cmd->ci == RUN && batch && *line == '(') {
+    if (gretl_echo_on() && s->cmd->ci == RUN && batch && *line == '(') {
 	printf("%s", line);
 	*linebak = 0;
     }
@@ -243,7 +249,7 @@ static int get_interactive_line (void *p)
 #else
     printf("%s", prompt);
     fflush(stdout);
-    file_get_line(s->line, s->cmd); /* note: "file" = stdin here */
+    file_get_line(s); /* note: "file" = stdin here */
 #endif
 
     return err;
@@ -255,7 +261,7 @@ static int cli_get_input_line (ExecState *s)
 
     if (runit || batch) {
 	/* reading from script file */
-	err = file_get_line(s->line, s->cmd);
+	err = file_get_line(s);
     } else {
 	/* interactive use */
 	err = get_interactive_line(s);
@@ -617,16 +623,12 @@ static void printline (const char *s)
     }
 }
 
-static void cli_exec_callback (ExecState *s, double ***pZ,
-			       DATAINFO *pdinfo)
+static void cli_exec_callback (ExecState *s, void *ptr,
+			       GretlObjType type)
 {
     int ci = s->cmd->ci;
 
-    if (ci == VAR || ci == VECM) {
-	maybe_stack_var(s->var, s->cmd);
-    } else if (ci == END && !strcmp(s->cmd->param, "restrict")) {
-	maybe_stack_var(s->var, s->cmd);
-    } else if (ci == MODELTAB || ci == GRAPHPG) {
+    if (ci == MODELTAB || ci == GRAPHPG) {
 	pprintf(s->prn, _("%s: command not available\n"), s->cmd->word);
     } 
 
@@ -643,6 +645,7 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
     int err = 0;
 
     if (!(cmd->opt & OPT_O)) {
+	/* not using ODBC */
 	err = getopenfile(line, newfile, (cmd->opt & OPT_W)?
 			  OPT_W : OPT_NONE);
 	if (err) {
@@ -713,21 +716,12 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
     return err;
 }
 
-static gretlopt plot_opt (gretlopt opt, int batch)
-{
-    if (batch) {
-	opt |= OPT_B;
-    }
-    return opt;
-}
-
 #define ENDRUN (NC + 1)
 
 /* exec_line: this is called to execute both interactive and script
    commands.  Note that most commands get passed on to the libgretl
    function gretl_cmd_exec(), but some commands that require special
-   action are dealt with here.  All estimation commands are passed on
-   to libgretl.
+   action are dealt with here.
 
    see also gui_exec_line() in gui2/library.c
 */
@@ -761,15 +755,18 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	/* catch requests relating to saved objects, which are not
 	   really "commands" as such */
 	k = saved_object_action(line, pZ, pdinfo, models, prn);
-	if (k == 1) return 0;   /* action was OK, or ignored */
-	if (k == -1) return 1;  /* action was faulty */
+	if (k == 1) {
+	    return 0; /* action was OK, or ignored */
+	} else if (k == -1) {
+	    return 1; /* action was faulty */
+	}
     }
 
     /* tell libgretl if we're in batch mode */
     gretl_set_batch_mode(batch);
 
-    /* if we're stacking commands for a loop, parse "lightly" */
     if (gretl_compiling_loop()) {
+	/* if we're stacking commands for a loop, parse "lightly" */
 	err = get_command_index(line, cmd);
     } else {
 	err = parse_command_line(line, cmd, pZ, pdinfo);
@@ -781,8 +778,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	return err;
     }
 
-    /* are we in a multi-line comment block? */
-    s->in_comment = (cmd_ignore(cmd))? 1 : 0;
+    gretl_exec_state_transcribe_flags(s, cmd);
 
     /* if in batch mode, echo comments from input */
     if (batch && cmd->ci == CMD_COMMENT && gretl_echo_on()) {
@@ -810,23 +806,17 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
    
     if (cmd->ci == LOOP || gretl_compiling_loop()) {  
 	/* accumulating loop commands */
-	if (!ok_in_loop(cmd->ci)) {
-	    printf("> %s\n", line);
-	    pprintf(prn, _("Sorry, this command is not available in loop mode\n"));
-	    err = E_NOTIMP;
-	} else {
-	    if (gretl_echo_on() && (!gretl_compiling_loop() || batch || runit)) {
-		eflag = gretl_compiling_loop()? CMD_STACKING : CMD_BATCH_MODE;
-		/* straight visual echo */
-		echo_cmd(cmd, pdinfo, line, eflag, prn);
-	    }
-	    err = gretl_loop_append_line(s, pZ, pdinfo);
-	    if (err) {
-		errmsg(err, prn);
-	    } else if (!batch && !runit) {
-		/* echo to record */
-		echo_cmd(cmd, pdinfo, line, CMD_RECORDING, cmdprn);
-	    }
+	if (gretl_echo_on() && (!gretl_compiling_loop() || batch || runit)) {
+	    eflag = gretl_compiling_loop()? CMD_STACKING : CMD_BATCH_MODE;
+	    /* straight visual echo */
+	    echo_cmd(cmd, pdinfo, line, eflag, prn);
+	}
+	err = gretl_loop_append_line(s, pZ, pdinfo);
+	if (err) {
+	    errmsg(err, prn);
+	} else if (!batch && !runit) {
+	    /* echo to record */
+	    echo_cmd(cmd, pdinfo, line, CMD_RECORDING, cmdprn);
 	}
 	return err;
     }
@@ -839,7 +829,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     check_for_loop_only_options(cmd->ci, cmd->opt, prn);
 
-    gretl_exec_state_set_callback(s, cli_exec_callback);
+    gretl_exec_state_set_callback(s, cli_exec_callback, OPT_NONE);
 
     switch (cmd->ci) {
 
@@ -864,36 +854,6 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	}
 	break;
 
-    case GNUPLOT:
-    case SCATTERS:
-	if (cmd->ci == GNUPLOT) {
-	    if ((cmd->opt & OPT_Z) && 
-		(cmd->list[0] != 3 || 
-		 !gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[cmd->list[3]]))) { 
-		pputs(prn, _("You must supply three variables, the last of "
-			     "which is a dummy variable\n(with values 1 or 0)\n"));
-		break;
-	    }
-	    if (cmd->opt & OPT_C) {
-		err = xy_plot_with_control(cmd->list, cmd->param, 
-					   (const double **) *pZ, pdinfo, 
-					   plot_opt(cmd->opt, batch));
-	    } else {
-		err = gnuplot(cmd->list, cmd->param, (const double **) *pZ, 
-			      pdinfo, plot_opt(cmd->opt, batch));
-	    }
-	} else {
-	    err = multi_scatters(cmd->list, (const double **) *pZ, pdinfo, 
-				 plot_opt(cmd->opt, batch));
-	}
-
-	if (err) {
-	    errmsg(err, prn);
-	} else if (batch) {
-	    pprintf(prn, _("wrote %s\n"), gretl_plotfile());
-	}
-	break;
-
     case HELP:
 	cli_help(cmd->param, cmd->opt, prn);
 	break;
@@ -965,7 +925,11 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     case RUN:
     case INCLUDE:
-	err = getopenfile(line, runfile, OPT_S);
+	if (cmd->ci == INCLUDE) {
+	    err = getopenfile(line, runfile, OPT_I);
+	} else {
+	    err = getopenfile(line, runfile, OPT_S);
+	}
 	if (err) { 
 	    pputs(prn, _("Command is malformed\n"));
 	    break;
@@ -1021,17 +985,6 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	break;
     }
 
-    if (!err && s->pmod != NULL) { 
-	attach_subsample_to_model(s->pmod, pdinfo);
-	err = maybe_stack_model(s->pmod, cmd, prn);
-    }
-
-    if (system_save_flag_is_set(s->sys)) {
-	/* only warrants action in GUI program */
-	system_unset_save_flag(s->sys);
-	s->sys = NULL;
-    }
-
     if (!err && cmd->ci != QUIT && gretl_echo_on() && !batch && !old_runit) {
 	/* record a successful interactive command */
 	echo_cmd(cmd, pdinfo, line, CMD_RECORDING, cmdprn);
