@@ -454,11 +454,11 @@ static int seek_file_collections (int location)
     int i = 0, err = 0;
 
     if (location == DATA_SEARCH) {
-	tmp = g_strdup_printf("%sdata", paths.gretldir);
+	tmp = g_strdup_printf("%sdata", gretl_home());
     } else if (location == SCRIPT_SEARCH) {
-	tmp = g_strdup_printf("%sscripts", paths.gretldir);
+	tmp = g_strdup_printf("%sscripts", gretl_home());
     } else if (location == USER_SEARCH) {
-	tmp = g_strdup(paths.workdir);
+	tmp = g_strdup(gretl_workdir());
 	trim_slash(tmp);
     } else {
 	return 1;
@@ -468,6 +468,7 @@ static int seek_file_collections (int location)
 
     dir = opendir(tmp);
     if (dir == NULL) {
+	g_free(tmp);
 	return 1;
     }
 
@@ -500,7 +501,7 @@ static int seek_file_collections (int location)
     g_free(tmp);
 
     if (location == USER_SEARCH && i++ == 0) {
-	tmp = gretl_default_workdir(&paths);
+	tmp = gretl_default_workdir();
 	if (tmp != NULL) {
 	    goto user_search_2;
 	}
@@ -671,11 +672,13 @@ static void show_datafile_info (GtkWidget *w, gpointer data)
     descrip = gretl_get_gdt_description(hdrname);
 
     if (descrip != NULL) {
+	gchar *title = g_strdup_printf("gretl: %s", _("data info"));
+
 	prn = gretl_print_new_with_buffer(descrip);
-	view_buffer(prn, 80, 320, _("gretl: data header"), INFO, NULL);
+	view_buffer(prn, 80, 320, title, INFO, NULL);
+	g_free(title);
     } else {
 	errbox(_("Failed to retrieve description of data"));
-	fprintf(stderr, I_("didn't get description from %s\n"), hdrname);
     }
 }
 
@@ -733,20 +736,10 @@ static gint enter_opens_file (GtkWidget *w, GdkEventKey *key,
     }
 }
 
-enum {
-    VIEW_FN_PKG_INFO,
-    VIEW_FN_PKG_CODE,
-    LOAD_FN_PKG,
-    EDIT_FN_PKG,
-    DELETE_FN_PKG,
-    CALL_FN_PKG
-};
-
 static int gui_load_user_functions (const char *fname)
 {
-    int err;
+    int err = load_function_package_from_file(fname);
 
-    err = load_user_function_file(fname);
     if (err) {
 	gui_errmsg(err);
     } 
@@ -756,18 +749,41 @@ static int gui_load_user_functions (const char *fname)
 
 static int gui_delete_fn_pkg (const char *fname, windata_t *vwin)
 {
-    char *msg = g_strdup_printf(_("Really delete %s?"), fname);
-    int err;
-    
-    if (yes_no_dialog(_("gretl: delete"), msg, 0) != GRETL_YES) {
-        return 0;
+    char *msg = g_strdup_printf(_("Function package %s"), fname);
+    const char *opts[] = {
+	N_("Delete package file"),
+	N_("Unload member functions")
+    };
+    int active[] = {1, 1};
+    int resp, err = 0;
+
+    resp = checks_only_dialog ("gretl", msg,
+			       opts, 2, active, 0);
+    g_free(msg);
+
+    if (resp < 0 || (active[0] == 0 && active[1] == 0)) {
+	/* canceled, or equivalent */
+	return 0;
     }
 
-    err = gretl_remove(fname);
-
-    if (err) {
-	file_write_errbox(fname);
+    if (active[1]) {
+	/* unload the package and its members from memory */
+	function_package_unload_full_by_filename(fname);
     } else {
+	/* unload the package (only) from memory */
+	function_package_unload_by_filename(fname);
+    }
+
+     if (active[0]) {
+	/* scratch the package file */
+	err = gretl_remove(fname);
+	if (err) {
+	    file_write_errbox(fname);
+	}
+    }
+
+    if (!err) {
+	/* remove package from GUI listing */
 	GtkTreeModel *mod;
 	GtkTreeIter iter;
 	int i = 0;
@@ -784,41 +800,41 @@ static int gui_delete_fn_pkg (const char *fname, windata_t *vwin)
     return err;
 }
 
-windata_t *gui_show_function_info (const char *fname, int role)
+windata_t *display_function_package_data (const char *pkgname,
+					  const char *path, 
+					  int role)
 {
-    char *pkgname = NULL;
     windata_t *vwin = NULL;
-    PRN *prn;
-    int err;
+    PRN *prn = NULL;
+    int err = 0;
 
     if (bufopen(&prn)) {
 	return NULL;
     }
 
-    if (role == VIEW_FUNC_INFO) {
-	err = get_function_file_info(fname, prn, &pkgname);
+    if (role == VIEW_PKG_INFO) {
+	if (g_path_is_absolute(path) && !strstr(path, "dltmp.")) {
+	    pprintf(prn, "File: %s\n", path);
+	}
+	err = print_function_package_info(path, prn);
     } else {
-	err = get_function_file_code(fname, prn, &pkgname);
+	err = print_function_package_code(path, prn);
     }
 	
     if (err) {
 	gretl_print_destroy(prn);
 	gui_errmsg(err);
     } else {
-	gchar *title;
+	gchar *title = g_strdup_printf("gretl: %s", pkgname);
 
-	title = g_strdup_printf("gretl: %s", (pkgname)? 
-				pkgname : _("function code"));
 	vwin = view_buffer(prn, 78, 350, title, role, NULL);
-	strcpy(vwin->fname, fname);
-	if (strstr(fname, "dltmp")) {
+	strcpy(vwin->fname, path);
+	if (strstr(path, "dltmp")) {
 	    set_window_delete_filename(vwin);
 	}
 	g_free(title);
     }
 
-    free(pkgname);
-
     return vwin;
 }
 
@@ -827,30 +843,48 @@ void set_funcs_dir_callback (windata_t *vwin, char *path)
     DIR *dir = opendir(path);
 
     if (dir != NULL) {
+	const char *opts[] = {
+	    N_("Add to functions shown"),
+	    N_("Replace functions shown")
+	};
 	GtkListStore *store;
 	GtkTreeIter iter;
 	int maxlen = 0;
 	int nfn = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(vwin->listbox), "nfn"));
 	int nfn0 = nfn;	
+	int resp;
 
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
+
+	resp = radio_dialog("gretl", NULL, opts, 2, 0, 0);
+	if (resp < 0) {
+	    return;
+	} else if (resp > 0) {
+	    gtk_list_store_clear(store);
+	    nfn = nfn0 = 0;
+	}
+
 	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 	read_fn_files_in_dir(vwin->role, dir, path, store, &iter,
 			     &nfn, &maxlen);
 	if (nfn > nfn0) {
-	    infobox("Found %d additional function file(s)", nfn - nfn0);
+	    infobox(_("Found %d function file(s)"), nfn - nfn0);
 	    g_object_set_data(G_OBJECT(vwin->listbox), "nfn", GINT_TO_POINTER(nfn));
+	    if (resp > 0) {
+		g_object_set_data(G_OBJECT(vwin->listbox), "keepdir", GINT_TO_POINTER(1));
+	    }
 	} else {
-	    warnbox("No additional function files were found");
+	    warnbox(_("No function files were found"));
 	}
+	    
 	closedir(dir);
     }
 }
 
 static void browser_functions_handler (windata_t *vwin, int task)
 {
-    char fnfile[FILENAME_MAX];
-    gchar *fname;
+    char path[FILENAME_MAX];
+    gchar *pkgname = NULL;
     gchar *dir;
     int dircol = 0;
     int err = 0;
@@ -863,35 +897,39 @@ static void browser_functions_handler (windata_t *vwin, int task)
     }
 
     tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), vwin->active_var, 
-			 0, &fname);
+			 0, &pkgname);
 
     if (dircol != 0) {
 	tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), vwin->active_var, 
 			     dircol, &dir);
-	build_path(fnfile, dir, fname, ".gfn");
+	build_path(path, dir, pkgname, ".gfn");
 	g_free(dir);
     } else {
-	strcpy(fnfile, fname);
+	strcpy(path, pkgname);
     }
 
-    g_free(fname);
+#if 0
+    fprintf(stderr, "browser_functions_handler: path='%s'\n", path);
+#endif
 
     if (task == LOAD_FN_PKG) {
-	err = gui_load_user_functions(fnfile);
+	err = gui_load_user_functions(path);
     } else if (task == DELETE_FN_PKG) {
-	err = gui_delete_fn_pkg(fnfile, vwin);
+	err = gui_delete_fn_pkg(path, vwin);
     } else if (task == VIEW_FN_PKG_INFO) {
-	gui_show_function_info(fnfile, VIEW_FUNC_INFO);
+	display_function_package_data(pkgname, path, VIEW_PKG_INFO);
     } else if (task == VIEW_FN_PKG_CODE) {
-	gui_show_function_info(fnfile, VIEW_FUNC_CODE);
+	display_function_package_data(pkgname, path, VIEW_PKG_CODE);
     } else if (task == EDIT_FN_PKG) {
-	edit_function_package(fnfile, &err);
+	edit_function_package(path);
     } else if (task == CALL_FN_PKG) {
-	call_function_package(fnfile, vwin->main, &err);
+	call_function_package(path, vwin->main, &err);
     }
 
-    if (!err && (task == EDIT_FN_PKG || task == CALL_FN_PKG)) {
-	maybe_update_func_files_window(0);
+    g_free(pkgname);
+
+    if (!err && task == CALL_FN_PKG) {
+	maybe_update_func_files_window(task);
     }
 }
 
@@ -975,11 +1013,10 @@ static int browser_busy (guint code)
     int ret = 0;
 
     if (code >= TEXTBOOK_DATA && code <= REMOTE_DB) {
-	GtkWidget *w;
+	GtkWidget *w = browsers[code - TEXTBOOK_DATA];
 
-	w = browsers[code - TEXTBOOK_DATA];
 	if (w != NULL) {
-	    gdk_window_raise(w->window);
+	    gtk_window_present(GTK_WINDOW(w));
 	    ret = 1;
 	}
     }
@@ -1007,7 +1044,7 @@ windata_t *get_local_viewer (int remote_role)
 
 static void new_package_callback (GtkWidget *w , gpointer p)
 {
-    data_save_selection_wrapper(SAVE_FUNCTIONS, NULL);
+    data_save_selection_wrapper(SAVE_FUNCTIONS);
 }
 
 static void close_files_viewer (GtkWidget *w, windata_t *vwin)
@@ -1035,6 +1072,7 @@ static void build_funcfiles_popup (windata_t *vwin)
 	vwin->popup = gtk_menu_new();
 
 	if (vwin->role == FUNC_FILES) {
+	    /* local function files: full menu */
 	    add_popup_item(_("Edit"), vwin->popup, 
 			   G_CALLBACK(browser_edit_func), 
 			   vwin);
@@ -1054,15 +1092,13 @@ static void build_funcfiles_popup (windata_t *vwin)
 			   G_CALLBACK(new_package_callback), 
 			   vwin);
 	} else {
+	    /* files on server: limited menu */
 	    add_popup_item(_("Info"), vwin->popup, 
-			   G_CALLBACK(file_info_from_server), 
+			   G_CALLBACK(pkg_info_from_server), 
 			   vwin);
 	    add_popup_item(_("Install"), vwin->popup, 
 			   G_CALLBACK(install_file_from_server), 
 			   vwin);
-	    add_popup_item(_("Execute"), vwin->popup, 
-			   G_CALLBACK(browser_call_func), 
-			   vwin);
 	}
     }
 }
@@ -1130,7 +1166,13 @@ static void show_local_funcs (GtkWidget *w, gpointer p)
 
 static void alt_funcs_dir (GtkWidget *w, windata_t *vwin)
 {
-    file_selector_with_parent(NULL, SET_FDIR, FSEL_DATA_VWIN, vwin, 
+    file_selector_with_parent(SET_FDIR, FSEL_DATA_VWIN, vwin, 
+                              vwin->main);
+}
+
+static void alt_db_dir (GtkWidget *w, windata_t *vwin)
+{
+    file_selector_with_parent(SET_DBDIR, FSEL_DATA_VWIN, vwin, 
                               vwin->main);
 }
 
@@ -1152,11 +1194,11 @@ enum {
 };
 
 static GretlToolItem files_items[] = {
-    { N_("Open"),           GTK_STOCK_OK,         NULL,                          BTN_OPEN },
+    { N_("Open"),           GTK_STOCK_OK, NULL, BTN_OPEN },
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6)
-    { N_("Select directory"), GTK_STOCK_OPEN,  G_CALLBACK(alt_funcs_dir),   BTN_DIR },
+    { N_("Select directory"), GTK_STOCK_OPEN, NULL, BTN_DIR },
 #else
-    { N_("Select directory"), GTK_STOCK_DIRECTORY,  G_CALLBACK(alt_funcs_dir),   BTN_DIR },
+    { N_("Select directory"), GTK_STOCK_DIRECTORY, NULL, BTN_DIR },
 #endif
     { N_("Edit"),           GTK_STOCK_EDIT,       G_CALLBACK(browser_edit_func), BTN_EDIT },
     { N_("Info"),           GTK_STOCK_INFO,       NULL,                          BTN_INFO },
@@ -1168,7 +1210,6 @@ static GretlToolItem files_items[] = {
     { N_("Look on server"), GTK_STOCK_NETWORK,    NULL,                          BTN_WWW },
     { N_("Local machine"),  GTK_STOCK_HOME,       NULL,                          BTN_HOME },
     { N_("New"),            GTK_STOCK_NEW,        G_CALLBACK(new_package_callback), BTN_NEW },
-    { N_("Find..."),        GTK_STOCK_FIND,       G_CALLBACK(listbox_find),       BTN_FIND },
     { N_("Close"),          GTK_STOCK_CLOSE,      G_CALLBACK(close_files_viewer), BTN_CLOSE }
 };
 
@@ -1176,7 +1217,7 @@ static int n_files_items = G_N_ELEMENTS(files_items);
 
 #define common_item(f) (f == BTN_FIND || f == BTN_CLOSE)
 
-#define local_funcs_item(f) (f == BTN_DIR || f == BTN_EDIT || f == BTN_NEW || \
+#define local_funcs_item(f) (f == BTN_EDIT || f == BTN_NEW || \
 			     f == BTN_DEL || f == BTN_CODE)
 
 static int files_item_get_callback (GretlToolItem *item, int role)
@@ -1190,7 +1231,7 @@ static int files_item_get_callback (GretlToolItem *item, int role)
 		role == REMOTE_FUNC_FILES ||
 		role == REMOTE_DATA_PKGS);
     } else if (item->flag == BTN_EXEC) {
-	return (role == FUNC_FILES || role == REMOTE_FUNC_FILES);
+	return (role == FUNC_FILES);
     }
 
     item->func = NULL;
@@ -1208,7 +1249,7 @@ static int files_item_get_callback (GretlToolItem *item, int role)
 	} else if (role == FUNC_FILES) {
 	    item->func = G_CALLBACK(show_function_info);
 	} else if (role == REMOTE_FUNC_FILES) {
-	    item->func = G_CALLBACK(file_info_from_server);
+	    item->func = G_CALLBACK(pkg_info_from_server);
 	} 
     } else if (item->flag == BTN_INDX) {
 	/* index: databases only */
@@ -1232,12 +1273,18 @@ static int files_item_get_callback (GretlToolItem *item, int role)
 	} else if (role == REMOTE_DB) {
 	    item->func = G_CALLBACK(show_local_dbs);
 	} 
-    } 
+    } else if (item->flag == BTN_DIR) {
+	if (role == FUNC_FILES) {
+	    item->func = G_CALLBACK(alt_funcs_dir);
+	} else if (role == NATIVE_DB) {
+	    item->func = G_CALLBACK(alt_db_dir);
+	}
+    }
 
     return (item->func != NULL);
 }
 
-static void make_filesbar (windata_t *vwin)
+static void make_files_toolbar (windata_t *vwin)
 {
     GtkWidget *hbox;
     GretlToolItem *item;
@@ -1256,19 +1303,24 @@ static void make_filesbar (windata_t *vwin)
     }
 
     gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
+    vwin_add_finder(vwin);
     gtk_widget_show_all(hbox);
 }
 
+#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 8
+/* we'll have to fake g_get_host_name */
+static const gchar *g_get_host_name (void)
+{
+    return _("local machine");
+}
+#endif
+
 static gchar *files_title (int code)
 {
     static char hname[48];
     gchar *ret = NULL;
 
     if (*hname == '\0') {
-#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 8
-	/* no g_get_host_name() */
-	strcpy(hname, _("local machine"));
-#else
 	const gchar *s = g_get_host_name();
 
 	if (s != NULL && strlen(s) < 48) {
@@ -1276,7 +1328,6 @@ static gchar *files_title (int code)
 	} else {
 	    strcpy(hname, _("local machine"));
 	}
-#endif
     }
 
     if (code == NATIVE_DB) {
@@ -1346,6 +1397,20 @@ static void set_up_viewer_drag_target (windata_t *vwin)
 		     callback, NULL);
 }
 
+void listbox_select_first (windata_t *vwin)
+{
+    GtkTreeView *view = GTK_TREE_VIEW(vwin->listbox);
+    GtkTreeModel *model;
+    GtkTreeSelection *selection;
+    GtkTreeIter iter;
+
+    model = gtk_tree_view_get_model(view);
+    gtk_tree_model_get_iter_first(model, &iter);
+    selection = gtk_tree_view_get_selection(view);
+    gtk_tree_selection_select_iter(selection, &iter);
+    gtk_widget_grab_focus(vwin->listbox);
+}
+
 void display_files (int code, gpointer p)
 {
     GtkWidget *filebox;
@@ -1387,13 +1452,12 @@ void display_files (int code, gpointer p)
 	gtk_window_set_default_size(GTK_WINDOW(vwin->main), 640, 480);
     }
 
-    /* set up grids */
     vwin->vbox = gtk_vbox_new(FALSE, 1);
     gtk_box_set_spacing(GTK_BOX(vwin->vbox), 4);
     gtk_container_set_border_width(GTK_CONTAINER(vwin->vbox), 4);
     gtk_container_add(GTK_CONTAINER(vwin->main), vwin->vbox);
 
-    make_filesbar(vwin);
+    make_files_toolbar(vwin);
 
     if (code == TEXTBOOK_DATA || code == PS_FILES) {
 	/* we'll need more than one tab */
@@ -1470,9 +1534,17 @@ void display_files (int code, gpointer p)
 	} 
     }
 
-    if (err && code == FUNC_FILES && fresp.try_server == 1) {
-	/* no function packages on local machine */
-	display_files(REMOTE_FUNC_FILES, p);
+    if (err) {
+	if (code == FUNC_FILES && fresp.try_server == 1) {
+	    /* no function packages on local machine */
+	    display_files(REMOTE_FUNC_FILES, p);
+	} else {
+	    return;
+	}
+    }
+
+    if (code != TEXTBOOK_DATA && code != PS_FILES) {
+	listbox_select_first(vwin);
     }
 }
 
@@ -1504,14 +1576,11 @@ void show_files (GtkAction *action, gpointer p)
     display_files(code, p);
 }
 
-static int get_func_info (const char *fname, const char *fndir,
-			  char **pdesc, char **pver)
+static int get_func_info (const char *fullname, char **pdesc, 
+			  char **pver)
 {
-    char fullname[FILENAME_MAX];
-    int err = 0;
+    int err = get_function_file_header(fullname, pdesc, pver);
 
-    build_path(fullname, fndir, fname, NULL);
-    *pdesc = get_function_file_header(fullname, pver, &err);
     if (err) {
 	gui_errmsg(err);
     }
@@ -1557,6 +1626,8 @@ static int fn_file_is_duplicate (const char *fname,
     return ret;
 }
 
+#define pkg_is_loaded(n) (get_function_package_by_filename(n) != NULL)
+
 static void
 read_fn_files_in_dir (int role, DIR *dir, const char *path, 
 		      GtkListStore *store, GtkTreeIter *iter,
@@ -1564,10 +1635,7 @@ read_fn_files_in_dir (int role, DIR *dir, const char *path,
 {
     struct dirent *dirent;
     char fullname[MAXLEN];
-    gboolean loaded;
-    char *fname;
-    char *descrip;
-    char *version;
+    gchar *fname;
     int n, imax = *nfn;
     int err;
 
@@ -1577,27 +1645,26 @@ read_fn_files_in_dir (int role, DIR *dir, const char *path,
 	    continue;
 	}
 	fname = g_strdup(dirent->d_name);
-	if (fname == NULL) {
-	    break;
-	}
 	n = strlen(fname);
 	if (!g_ascii_strcasecmp(fname + n - 4, ".gfn")) {
-	    err = get_func_info(fname, path, &descrip, &version);
+	    char *descrip = NULL, *version = NULL;
+
+	    build_path(fullname, path, fname, NULL);
+	    err = get_func_info(fullname, &descrip, &version);
+
 	    if (!err) {
 		if (!fn_file_is_duplicate(fname, version, store, imax)) {
 		    gtk_list_store_append(store, iter);
-		    build_path(fullname, path, fname, NULL);
 		    n -= 4;
 		    fname[n] = '\0';
 		    if (n > *maxlen) {
 			*maxlen = n;
 		    }
-		    loaded = function_package_is_loaded(fullname);
 		    gtk_list_store_set(store, iter, 
 				       0, fname, 
 				       1, version,
 				       2, descrip, 
-				       3, loaded, 
+				       3, pkg_is_loaded(fullname), 
 				       4, path, -1);
 		    *nfn += 1;
 		}
@@ -1642,68 +1709,103 @@ static void sort_pkglist (windata_t *vwin)
 				    compare_pkgnames, NULL, NULL);
 }
 
+static int dirname_done (char **dnames, int ndirs, char *dirname)
+{
+    int i;
+
+    for (i=0; i<ndirs; i++) {
+	if (!strcmp(dnames[i], dirname)) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
 {
     GtkListStore *store;
     GtkTreeIter iter;
-    char fndir[FILENAME_MAX];
-    char *tmp;
+    char **dnames = NULL;
+    int ndirs = 0;
     DIR *dir;
-    int maxlen = 0;
-    int nfn = 0;
+    int i, nfn, maxlen = 0;
 
     store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
+    nfn = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(vwin->listbox), "nfn"));
+
+    if (nfn > 0) {
+	/* we're repopulating an existing list */
+	GtkTreeModel *model = GTK_TREE_MODEL(store);
+	gchar *dirname;
+
+	gtk_tree_model_get_iter_first(model, &iter);
+	gtk_tree_model_get(model, &iter, 4, &dirname, -1);
+	strings_array_add(&dnames, &ndirs, dirname);
+	g_free(dirname);
+
+	while (gtk_tree_model_iter_next(model, &iter)) {
+	    gtk_tree_model_get(model, &iter, 4, &dirname, -1);
+	    if (!dirname_done(dnames, ndirs, dirname)) {
+		strings_array_add(&dnames, &ndirs, dirname);
+	    }
+	    g_free(dirname);
+	}
+    }
+
+    nfn = 0;
     gtk_list_store_clear(store);
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
-    /* pick up any function files in system dir */
-    build_path(fndir, paths.gretldir, "functions", NULL);
-    dir = opendir(fndir);
-    if (dir != NULL) {
-	read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
-			     &nfn, &maxlen);
-	closedir(dir);
-    }
-
-    /* plus any function files in the user's working dir */
-    strcpy(fndir, paths.workdir);
-    dir = opendir(fndir);
-    if (dir != NULL) {
-	read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
-			     &nfn, &maxlen);
-	closedir(dir);
-    }    
-
-    /* plus any function files under "functions" in the user's working dir */
-    build_path(fndir, paths.workdir, "functions", NULL);
-    dir = opendir(fndir);
-    if (dir != NULL) {
-	read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
-			     &nfn, &maxlen);
-	closedir(dir);
-    }
-
-    /* plus any function files in the user's personal data dir */
-    build_path(fndir, paths.dotdir, "functions", NULL);
-    dir = opendir(fndir);
-    if (dir != NULL) {
-	read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
-			     &nfn, &maxlen);
-	closedir(dir);
-    } 
-
-    /* plus any in the default working dir, if not already searched */
-    tmp = gretl_default_workdir(&paths);
-    if (tmp != NULL) {
-	build_path(fndir, tmp, "functions", NULL);
-	dir = opendir(fndir);
-	if (dir != NULL) {
-	    read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
-				 &nfn, &maxlen);
-	    closedir(dir);
+    if (ndirs > 0) {
+	for (i=0; i<ndirs; i++) {
+	    dir = opendir(dnames[i]);
+	    if (dir != NULL) {
+		read_fn_files_in_dir(vwin->role, dir, dnames[i], store, &iter,
+				     &nfn, &maxlen);
+		closedir(dir);
+	    }
 	}
-	free(tmp);
-    }
+
+	free_strings_array(dnames, ndirs);
+    } else {
+	char fndir[FILENAME_MAX];
+
+	for (i=0; i<5; i++) {
+	    *fndir = '\0';
+
+	    if (i == 0) {
+		/* pick up any function files in system dir */
+		build_path(fndir, gretl_home(), "functions", NULL);
+	    } else if (i == 1) {
+		/* plus any function files in the user's working dir... */
+		strcpy(fndir, gretl_workdir());
+	    } else if (i == 2) {
+		/* and in any "functions" subdir thereof */
+		build_path(fndir, gretl_workdir(), "functions", NULL);
+	    } else if (i == 3) {
+		/* plus any in the user's dotdir */
+		build_path(fndir, gretl_dotdir(), "functions", NULL);
+	    } else if (i == 4) {
+		/* plus any in the default working dir, if not already searched */
+		char *tmp = gretl_default_workdir();
+
+		if (tmp != NULL) {
+		    build_path(fndir, tmp, "functions", NULL);
+		    g_free(tmp);
+		} 
+	    }
+
+	    if (*fndir != '\0') {
+		dir = opendir(fndir);
+		if (dir != NULL) {
+		    read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
+					 &nfn, &maxlen);
+		    closedir(dir);
+		}
+	    }
+	}
+    }	    
 
     if (nfn == 0) {
 	if (fresp->try_server == 0) {
@@ -1728,9 +1830,40 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
     return 0;
 }
 
-/* update loaded status after run, edit calls */
+static void revise_loaded_status (GtkWidget *lbox)
+{
+    char fullname[MAXLEN];
+    GtkTreeModel *model;
+    GtkListStore *store;
+    GtkTreeIter iter;
+    gchar *fname;
+    gchar *fdir;
 
-void maybe_update_func_files_window (int editing)
+    model = gtk_tree_view_get_model(GTK_TREE_VIEW(lbox));
+    store = GTK_LIST_STORE(model);
+
+    if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter)) {
+	return;
+    }
+
+    while (1) {
+	gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &fname, 
+			   4, &fdir, -1);
+	sprintf(fullname, "%s%c%s.gfn", fdir, SLASH, fname);
+	g_free(fname);
+	g_free(fdir);
+	gtk_list_store_set(store, &iter, 
+			   3, pkg_is_loaded(fullname), 
+			   -1);
+	if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter)) {
+	    break;
+	}
+    }
+}
+
+/* update function package status after run, edit calls */
+
+void maybe_update_func_files_window (int action)
 {
     GtkWidget *w = get_browser(FUNC_FILES);
     windata_t *vwin = NULL;
@@ -1744,39 +1877,11 @@ void maybe_update_func_files_window (int editing)
 	lbox = vwin->listbox;
     }
 
-    if (editing && lbox != NULL) {
-	GtkListStore *store;
-
-	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(lbox)));	
-	gtk_list_store_clear(store);
-	populate_func_list(vwin, NULL);
-	return;
-    }
-
     if (lbox != NULL) {
-	GtkListStore *store = 
-	    GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(lbox)));
-	GtkTreeIter iter;
-	char fullname[MAXLEN];
-	gchar *fname;
-	gchar *fdir;
-	gboolean loaded;
-
-	if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter)) {
-	    return;
-	}
-
-	while (1) {
-	    gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &fname, 
-			       4, &fdir, -1);
-	    sprintf(fullname, "%s%c%s.gfn", fdir, SLASH, fname);
-	    g_free(fname);
-	    g_free(fdir);
-	    loaded = function_package_is_loaded(fullname);
-	    gtk_list_store_set(store, &iter, 3, loaded, -1);
-	    if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter)) {
-		break;
-	    }
+	if (action == EDIT_FN_PKG) {
+	    populate_func_list(vwin, NULL);
+	} else {
+	    revise_loaded_status(lbox);
 	}
     }
 }
