@@ -258,7 +258,7 @@ static int dwiz_make_changes (DATAINFO *dwinfo, dw_opts *opts,
 
     /* preliminaries */
     if (time_series(dwinfo)) {
-	ntodate_full(dwinfo->stobs, dwinfo->t1, dwinfo);
+	ntodate(dwinfo->stobs, dwinfo->t1, dwinfo);
     } else if (known_panel(dwinfo)) {
 	if (!dataset_is_panel(datainfo)) {
 	    /* Turning a subset of a non-panel dataset into a panel:
@@ -557,8 +557,8 @@ static void make_confirmation_text (char *ctxt, DATAINFO *dwinfo, gretlopt *flag
 	    dwinfo->n = lastobs + 1;
 	}
 
-	ntodate_full(stobs, dwinfo->t1, dwinfo);
-	ntodate_full(endobs, lastobs, dwinfo);
+	ntodate(stobs, dwinfo->t1, dwinfo);
+	ntodate(endobs, lastobs, dwinfo);
 	sprintf(ctxt, _("%s, %s to %s"), tslabel, stobs, endobs);
     } else if (dwinfo->structure == PANEL_UNKNOWN) {
 	sprintf(ctxt, _("Panel data (%s)\n"
@@ -698,15 +698,15 @@ static void compute_default_ts_info (DATAINFO *dwinfo, int newdata)
 	dwinfo->t1 = dateton(datainfo->stobs, dwinfo);
     }
 
-    ntodate_full(dwinfo->endobs, dwinfo->n - 1, dwinfo);
+    ntodate(dwinfo->endobs, dwinfo->n - 1, dwinfo);
 
 #if DWDEBUG
-    ntodate_full(obsstr, dwinfo->t1, dwinfo);
+    ntodate(obsstr, dwinfo->t1, dwinfo);
     fprintf(stderr, "dwinfo: v=%d, pd=%d, stobs='%s', endobs='%s', sd0=%g, t1=%d (%s)\n",
 	    dwinfo->v, dwinfo->pd, dwinfo->stobs, dwinfo->endobs, dwinfo->sd0, 
 	    dwinfo->t1, obsstr);
 
-    ntodate_full(obsstr, datainfo->t1, datainfo);
+    ntodate(obsstr, datainfo->t1, datainfo);
     fprintf(stderr, "datainfo: pd=%d, stobs='%s', sd0=%g, t1=%d (%s)\n",
 	    datainfo->pd, datainfo->stobs, datainfo->sd0, datainfo->t1, obsstr);
 #endif
@@ -1336,7 +1336,7 @@ static void set_up_dw_opts (dw_opts *opts, int step,
 
 /* make two or more radio buttons based on the current setings in
    the "opts" structure
- */
+*/
 
 static void dwiz_build_radios (int step, DATAINFO *dwinfo, 
 			       dw_opts *opts, 
@@ -1578,13 +1578,16 @@ static void dwiz_button_visibility (GtkWidget *dlg, int step)
 
 static void add_editing_option (GtkWidget *vbox, gretlopt *flags)
 {
-    GtkWidget *hbox, *b;
+    if (g_object_get_data(G_OBJECT(vbox), "edbutton") == NULL) {
+	GtkWidget *hbox, *b;
 
-    hbox = gtk_hbox_new(FALSE, 5);
-    b = gretl_option_check_button(_("start entering data values"),
-				  flags, DW_SSHEET);
-    gtk_box_pack_start(GTK_BOX(hbox), b, FALSE, FALSE, 5);
-    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);    
+	hbox = gtk_hbox_new(FALSE, 5);
+	b = gretl_option_check_button(_("start entering data values"),
+				      flags, DW_SSHEET);
+	gtk_box_pack_start(GTK_BOX(hbox), b, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
+	g_object_set_data(G_OBJECT(vbox), "edbutton", b);
+    }
 }
 
 /* build the appropriate notebook page for the given step */
@@ -1738,7 +1741,7 @@ static void free_dw_opts (GtkWidget *w, dw_opts *opts)
 
 static void build_dwiz_buttons (GtkWidget *dlg, DATAINFO *dwinfo)
 {
-    GtkWidget *hbox = GTK_DIALOG(dlg)->action_area;
+    GtkWidget *hbox = gtk_dialog_get_action_area(GTK_DIALOG(dlg));
     GtkWidget *b;
 
     /* "Cancel" button */
@@ -1794,7 +1797,7 @@ static void dwiz_page_add_title (GtkWidget *vbox, int i, int smax)
 				  "weight=\"bold\" "
 				  "color=\"white\" "
 				  "background=\"#6C7B8A\" "
-				  "size=\"xx-large\"> %-*s </span>", 
+				  "size=\"x-large\"> %-*s </span>", 
 				  smax, _(wizcode_string(i)));
     gtk_label_set_markup(GTK_LABEL(label), buf);
     g_free(buf);
@@ -1855,7 +1858,7 @@ static void data_structure_wizard (int create)
 			      GRETL_DLG_QUASI_MODAL);
     g_object_set_data(G_OBJECT(dialog), "dwinfo", dwinfo);
     g_object_set_data(G_OBJECT(dialog), "opts", opts);
-    vbox = GTK_DIALOG(dialog)->vbox;
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
 
     /* notebook to hold the steps */
     nb = gtk_notebook_new();
@@ -1898,6 +1901,9 @@ static void data_structure_wizard (int create)
     dwiz_prepare_page(GTK_NOTEBOOK(nb), 0, dwinfo);
     gtk_notebook_set_current_page(GTK_NOTEBOOK(nb), 0);
 
+    /* note: we can't use gtk_widget_show_all() here
+       because component widgets may be displayed 
+       selectively */
     gtk_widget_show(dialog);
 }
 
