@@ -42,12 +42,16 @@ void infobox (const char *template, ...);
 
 void warnbox (const char *template, ...);
 
+void maybe_warn (void);
+
 void file_read_errbox (const char *fname);
 
 void file_write_errbox (const char *fname);
 
 gint yes_no_dialog (const char *title, const char *msg, int cancel);
 
+void gretl_dialog_add_message (GtkWidget *dlg, const char *msg);
+
 int make_default_storelist (void);
 
 gboolean exit_check (void);
@@ -116,6 +120,8 @@ int spin_dialog (const char *title, const char *blurb,
 		 int *spinvar, const char *spintxt, 
 		 int spinmin, int spinmax, int helpcode);
 
+int yes_no_help_dialog (const char *msg, int hcode);
+
 int get_obs_dialog (const char *title, const char *text,
 		    const char *t1str, const char *t2str,
 		    int t1min, int t1max, int *t1, 
@@ -129,7 +135,8 @@ int forecast_dialog (int t1min, int t1max, int *t1,
 		     int dyn, gretlopt *optp,
 		     double *conf, MODEL *pmod);
 
-void dialog_add_confidence_selector (GtkWidget *dlg, double *conf);
+void dialog_add_confidence_selector (GtkWidget *dlg, double *conf,
+				     gretlopt *gopt);
 
 int freq_dialog (const char *title, const char *blurb,
 		 int *nbins, int nbmax, double *f0, double *fwid,
