@@ -22,10 +22,11 @@ typedef enum {
     FSEL_DATA_PRN,
     FSEL_DATA_VWIN,
     FSEL_DATA_MISC,
-    FSEL_DATA_FNPKG
+    FSEL_DATA_FNPKG,
+    FSEL_DATA_STATUS
 } FselDataSrc;
 
-void file_selector (const char *msg, int action, FselDataSrc src, gpointer data);
+void file_selector (int action, FselDataSrc src, gpointer data);
 
-void file_selector_with_parent (const char *msg, int action, FselDataSrc src, 
+void file_selector_with_parent (int action, FselDataSrc src, 
 				gpointer data, GtkWidget *w);
