@@ -24,6 +24,7 @@
 #include "filters.h"
 #include "libset.h"
 #include "plotspec.h"
+#include "cmdstack.h"
 
 enum {
     FILTER_SMA = 1,
@@ -48,25 +49,27 @@ enum {
 typedef struct filter_info_ filter_info;
 
 struct filter_info_ {
-    int ftype;
-    int vnum;
-    const char *vname;
-    int t1;
-    int t2;
-    int nobs;
-    int center;
-    double lambda;
-    int k;
-    int bkl;
-    int bku;
+    int ftype;            /* type of filter */
+    int vnum;             /* ID number of series to filter */
+    const char *vname;    /* name of series */
+    int t1;               /* starting observation */
+    int t2;               /* ending observation */
+    int k;                /* no. of terms in MA, or no. of obs for mean */
+    int center;           /* for simple MA: center or not */
+    double lambda;        /* general purpose parameter */
+    int bkk;              /* Baxter-King parameter */
+    int bkl;              /* Baxter-King lower value */
+    int bku;              /* Baxter-King upper value */
     gretlopt graph_opt;
     gretlopt save_opt;
-    char save1[VNAMELEN];
-    char save2[VNAMELEN];
+    char save_t[VNAMELEN];
+    char save_c[VNAMELEN];
+    char label_t[MAXLABEL];
+    char label_c[MAXLABEL];
     GtkWidget *dlg;
     GtkWidget *entry1;
     GtkWidget *entry2;
-    GtkWidget *nspin;
+    GtkWidget *kspin;
 };
 
 static const char *filter_get_title (int ftype)
@@ -89,15 +92,17 @@ static const char *filter_get_title (int ftype)
 static void filter_info_init (filter_info *finfo, int ftype, int v,
 			      int t1, int t2)
 {
+    int pd = datainfo->pd;
+
     finfo->ftype = ftype;
     finfo->vnum = v;
     finfo->vname = datainfo->varname[v];
     finfo->t1 = t1;
     finfo->t2 = t2;
-    finfo->nobs = 0;
+    finfo->k = 0;
     finfo->center = 0;
     finfo->lambda = 0.0;
-    finfo->k = 0;
+    finfo->bkk = 0;
     finfo->bkl = 0;
     finfo->bku = 0;
 
@@ -109,27 +114,28 @@ static void filter_info_init (filter_info *finfo, int ftype, int v,
 
     finfo->save_opt = FILTER_SAVE_NONE;
 
-    *finfo->save1 = 0;
-    *finfo->save2 = 0;
+    *finfo->save_t = 0;
+    *finfo->save_c = 0;
+    *finfo->label_t = 0;
+    *finfo->label_c = 0;
 
     finfo->dlg = NULL;
     finfo->entry1 = NULL;
     finfo->entry2 = NULL;
-    finfo->nspin = NULL;
+    finfo->kspin = NULL;
 
     if (ftype == FILTER_SMA) {
-	finfo->center = 1;
-	finfo->nobs = (datainfo->pd == 1 || datainfo->pd == 10)? 3 :
-	    datainfo->pd;
+	finfo->center = 0;
+	finfo->k = (pd == 1 || pd == 10)? 3 : pd;
     } else if (ftype == FILTER_EMA) {
-	finfo->lambda = 0.1;
+	finfo->lambda = 0.2;
     } else if (ftype == FILTER_HP) {
 	finfo->lambda = libset_get_double(HP_LAMBDA);
 	if (na(finfo->lambda)) {
-	    finfo->lambda = 100 * datainfo->pd * datainfo->pd;
+	    finfo->lambda = 100 * pd * pd;
 	}
     } else if (ftype == FILTER_BK) {
-	finfo->k = get_bkbp_k(datainfo);
+	finfo->bkk = get_bkbp_k(datainfo);
 	get_bkbp_periods(datainfo, &finfo->bkl, &finfo->bku);
 	if (finfo->bku <= finfo->bkl) {
 	    finfo->bku = finfo->bkl + 1;
@@ -141,7 +147,7 @@ static void filter_info_init (filter_info *finfo, int ftype, int v,
 
 static void filter_make_savename (filter_info *finfo, int i)
 {
-    char *targ = (i == 0)? finfo->save1 : finfo->save2;
+    char *targ = (i == 0)? finfo->save_t : finfo->save_c;
     int len;
 
     if (finfo->ftype == FILTER_SMA) {
@@ -162,42 +168,46 @@ static void filter_make_savename (filter_info *finfo, int i)
 
 static void filter_make_varlabel (filter_info *finfo, int v, int i)
 {
-    char *targ = VARLABEL(datainfo, v);
+    char *s = (i == FILTER_SAVE_TREND)? finfo->label_t : 
+	finfo->label_c;
 
     if (finfo->ftype == FILTER_SMA) {
 	if (i == FILTER_SAVE_TREND) {
 	    if (finfo->center) {
-		sprintf(targ, _("Centered %d-period moving average of %s"), 
-			finfo->nobs, finfo->vname);
+		sprintf(s, _("Centered %d-period moving average of %s"), 
+			finfo->k, finfo->vname);
 	    } else {
-		sprintf(targ, _("%d-period moving average of %s"), 
-			finfo->nobs, finfo->vname);
+		sprintf(s, _("%d-period moving average of %s"), 
+			finfo->k, finfo->vname);
 	    }
 	} else {
-	    sprintf(targ, _("Residual from %d-period MA of %s"), 
-		    finfo->nobs, finfo->vname);
+	    sprintf(s, _("Residual from %d-period MA of %s"), 
+		    finfo->k, finfo->vname);
 	}
     } else if (finfo->ftype == FILTER_EMA) {
 	if (i == FILTER_SAVE_TREND) {
-	    sprintf(targ, _("Exponential moving average of %s (current weight %g)"),
-		    finfo->vname, 1.0 - finfo->lambda);
+	    sprintf(s, _("Exponential moving average of %s (current weight %g)"),
+		    finfo->vname, finfo->lambda);
 	} else {
-	    sprintf(targ, _("Residual from EMA of %s (current weight %g)"),
-		    finfo->vname, 1.0 - finfo->lambda);
+	    sprintf(s, _("Residual from EMA of %s (current weight %g)"),
+		    finfo->vname, finfo->lambda);
 	}	    
     } else if (finfo->ftype == FILTER_HP) {
 	if (i == FILTER_SAVE_TREND) {
-	    sprintf(targ, _("Filtered %s: Hodrick-Prescott trend (lambda = %g)"), 
+	    sprintf(s, _("Filtered %s: Hodrick-Prescott trend (lambda = %g)"), 
 		    finfo->vname, finfo->lambda);
 	} else {
-	    sprintf(targ, _("Filtered %s: Hodrick-Prescott cycle (lambda = %g)"), 
+	    sprintf(s, _("Filtered %s: Hodrick-Prescott cycle (lambda = %g)"), 
 		    finfo->vname, finfo->lambda);
 	}	    
     } else if (finfo->ftype == FILTER_BK) {
-	sprintf(targ, _("Filtered %s: Baxter-King cycle"), finfo->vname);
+	sprintf(s, _("Filtered %s: Baxter-King, frequency %d to %d"), 
+		finfo->vname, finfo->bkl, finfo->bku);
     } else if (finfo->ftype == FILTER_FD) {
-	sprintf(targ, "fracdiff(%s, %g)", finfo->vname, finfo->lambda);
+	sprintf(s, "fracdiff(%s, %g)", finfo->vname, finfo->lambda);
     }
+
+    strcpy(VARLABEL(datainfo, v), s);
 }
 
 static void check_bk_limits1 (GtkWidget *s1, GtkWidget *s2)
@@ -231,7 +241,7 @@ static void sma_center_callback (GtkWidget *w, int *c)
 
 static int varname_error (filter_info *finfo, int i) 
 {
-    const char *s = (i == 1)? finfo->save1 : finfo->save2;
+    const char *s = (i == 1)? finfo->save_t : finfo->save_c;
 
     if (*s == 0) {
 	errbox(_("Variable name is missing"));
@@ -239,7 +249,7 @@ static int varname_error (filter_info *finfo, int i)
     } else if (gui_validate_varname(s, GRETL_TYPE_SERIES)) {
 	return 1;
     } else if (i == 2 && (finfo->save_opt & FILTER_SAVE_TREND)) {
-	if (!strcmp(s, finfo->save1)) {
+	if (!strcmp(s, finfo->save_t)) {
 	    errbox(_("Conflicting variable names"));
 	    return 1;
 	}
@@ -251,28 +261,27 @@ static int varname_error (filter_info *finfo, int i)
 static void filter_dialog_ok (GtkWidget *w, filter_info *finfo)
 {
     if (finfo->save_opt & FILTER_SAVE_TREND) {
-	strcpy(finfo->save1, gtk_entry_get_text(GTK_ENTRY(finfo->entry1)));
+	strcpy(finfo->save_t, gtk_entry_get_text(GTK_ENTRY(finfo->entry1)));
 	if (varname_error(finfo, 1)) {
 	    return;
 	}
     }
 
     if (finfo->save_opt & FILTER_SAVE_CYCLE) {
-	strcpy(finfo->save2, gtk_entry_get_text(GTK_ENTRY(finfo->entry2)));
+	strcpy(finfo->save_c, gtk_entry_get_text(GTK_ENTRY(finfo->entry2)));
 	if (varname_error(finfo, 2)) {
 	    return;
 	} 
     } 
 
-    if (finfo->nspin != NULL) {
-	if (GTK_WIDGET_SENSITIVE(finfo->nspin)) {
-	    finfo->nobs = (int) 
-		gtk_spin_button_get_value(GTK_SPIN_BUTTON(finfo->nspin));
+    if (finfo->kspin != NULL) {
+	if (GTK_WIDGET_SENSITIVE(finfo->kspin)) {
+	    finfo->k = (int) 
+		gtk_spin_button_get_value(GTK_SPIN_BUTTON(finfo->kspin));
 	} else {
-	    finfo->nobs = 0;
+	    finfo->k = 0;
 	}
     }
-    
 
     gtk_widget_destroy(finfo->dlg);
 }
@@ -332,7 +341,7 @@ static void filter_save_check_buttons (GtkWidget *dlg, filter_info *finfo)
 	gtk_entry_set_max_length(GTK_ENTRY(w), VNAMELEN - 1);
 	gtk_entry_set_width_chars(GTK_ENTRY(w), VNAMELEN + 3);
 	filter_make_savename(finfo, i);
-	gtk_entry_set_text(GTK_ENTRY(w), (i == 0)? finfo->save1 : finfo->save2);
+	gtk_entry_set_text(GTK_ENTRY(w), (i == 0)? finfo->save_t : finfo->save_c);
 	gtk_entry_set_activates_default(GTK_ENTRY(w), TRUE);	
 	gtk_table_attach_defaults(GTK_TABLE(tab), w, 1, 2, i, i+1);
 	if (i == 0) {
@@ -385,11 +394,11 @@ static void bkbp_frequencies_table (GtkWidget *dlg, filter_info *finfo)
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 }
 
-static void use_submean (GtkWidget *w, GtkWidget *s)
+static void toggle_use_submean (GtkWidget *w, GtkWidget *s)
 {
     gboolean t = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
 
-    gtk_widget_set_sensitive(s, t);
+    gtk_widget_set_sensitive(s, !t);
 }
 
 static void ema_obs_radios (GtkWidget *dlg, filter_info *finfo)
@@ -409,20 +418,19 @@ static void ema_obs_radios (GtkWidget *dlg, filter_info *finfo)
     gtk_table_set_col_spacing(GTK_TABLE(tab), 0, 5);
 
     /* default */
-    r1 = gtk_radio_button_new_with_label(NULL, _("the mean of the whole series"));
+    r1 = gtk_radio_button_new_with_label(NULL, _("the mean of the first n observations"));
     gtk_table_attach_defaults(GTK_TABLE(tab), r1, 0, 1, 0, 1);
+    w = gtk_spin_button_new_with_range(1, (finfo->t2 - finfo->t1 + 1) / 4, 1);
+    gtk_table_attach_defaults(GTK_TABLE(tab), w, 1, 2, 0, 1);
+    finfo->kspin = w;
 
     /* alternate */
     group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(r1));
-    r2 = gtk_radio_button_new_with_label(group, _("the mean of the first n observations"));
+    r2 = gtk_radio_button_new_with_label(group, _("the mean of the whole series"));
     gtk_table_attach_defaults(GTK_TABLE(tab), r2, 0, 1, 1, 2);
-    w = gtk_spin_button_new_with_range(1, (finfo->t2 - finfo->t1 + 1) / 4, 1);
-    gtk_table_attach_defaults(GTK_TABLE(tab), w, 1, 2, 1, 2);
-    gtk_widget_set_sensitive(w, FALSE);
-    finfo->nspin = w;
 
     /* connect */
-    g_signal_connect(G_OBJECT(r2), "clicked", G_CALLBACK(use_submean), w);
+    g_signal_connect(G_OBJECT(r2), "clicked", G_CALLBACK(toggle_use_submean), w);
 
     /* pack everything */
     gtk_box_pack_start(GTK_BOX(hbox), tab, TRUE, TRUE, 10);  
@@ -449,7 +457,7 @@ static int filter_dialog (filter_info *finfo)
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
 
     if (finfo->ftype == FILTER_SMA) {
-	GtkWidget *nspin;
+	GtkWidget *kspin;
 	int T = finfo->t2 - finfo->t1 + 1;
 	int nmax = T / 2;
 
@@ -457,11 +465,11 @@ static int filter_dialog (filter_info *finfo)
 	hbox = gtk_hbox_new(FALSE, 5);
 	w = gtk_label_new(_("Number of observations in average:"));
 	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
-	nspin = gtk_spin_button_new_with_range(2, (gdouble) nmax, 1);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(nspin), (gdouble) finfo->nobs);
-	g_signal_connect(G_OBJECT(nspin), "value-changed",
-			 G_CALLBACK(set_int_from_spinner), &finfo->nobs);
-	gtk_box_pack_start(GTK_BOX(hbox), nspin, TRUE, TRUE, 5);    
+	kspin = gtk_spin_button_new_with_range(2, (gdouble) nmax, 1);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(kspin), (gdouble) finfo->k);
+	g_signal_connect(G_OBJECT(kspin), "value-changed",
+			 G_CALLBACK(set_int_from_spinner), &finfo->k);
+	gtk_box_pack_start(GTK_BOX(hbox), kspin, TRUE, TRUE, 5);    
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
 	/* select centered or not */
@@ -485,7 +493,6 @@ static int filter_dialog (filter_info *finfo)
 			 G_CALLBACK(set_double_from_spinner), &finfo->lambda);
 	gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);    
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-
 	/* set calculation of initial EMA value */
 	ema_obs_radios(dlg, finfo);
     } else if (finfo->ftype == FILTER_HP) {
@@ -500,17 +507,16 @@ static int filter_dialog (filter_info *finfo)
 	gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);    
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     } else if (finfo->ftype == FILTER_BK) {
-	/* set "k" */
+	/* set Baxter-King "k" */
 	hbox = gtk_hbox_new(FALSE, 5);
 	w = gtk_label_new(_("k (higher values -> better approximation):"));
 	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
-	w = gtk_spin_button_new_with_range(1.0, 4 * finfo->k, 1);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), finfo->k);
+	w = gtk_spin_button_new_with_range(1.0, 4 * finfo->bkk, 1);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), finfo->bkk);
 	g_signal_connect(G_OBJECT(w), "value-changed",
-			 G_CALLBACK(set_int_from_spinner), &finfo->k);
+			 G_CALLBACK(set_int_from_spinner), &finfo->bkk);
 	gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);    
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-
 	/* set periods */
 	bkbp_frequencies_table(dlg, finfo);
     } else if (finfo->ftype == FILTER_FD) {
@@ -534,14 +540,14 @@ static int filter_dialog (filter_info *finfo)
 				  FILTER_GRAPH_CYCLE);
 	/* save to dataset? */
 	hbox = gtk_hbox_new(FALSE, 5);
-	w = gretl_option_check_button(_("Save cyclical series as"),
+	w = gretl_option_check_button(_("Save filtered series as"),
 				      &finfo->save_opt, FILTER_SAVE_CYCLE);
 	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
 	w = gtk_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(w), VNAMELEN - 1);
 	gtk_entry_set_width_chars(GTK_ENTRY(w), VNAMELEN + 3);
 	filter_make_savename(finfo, 1);
-	gtk_entry_set_text(GTK_ENTRY(w), finfo->save2);
+	gtk_entry_set_text(GTK_ENTRY(w), finfo->save_c);
 	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
 	finfo->entry2 = w;
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
@@ -585,10 +591,13 @@ static void print_gp_data (filter_info *finfo, const double *obs,
     int t;
 
     for (t=finfo->t1; t<=finfo->t2; t++) {
+	/* same print format as in function printvars() in 
+	   lib/src/graphing.c
+	*/
 	if (na(x[t])) {
-	    fprintf(fp, "%g ?\n", obs[t]);
+	    fprintf(fp, "%.10g ?\n", obs[t]);
 	} else {
-	    fprintf(fp, "%g %g\n", obs[t], x[t]);
+	    fprintf(fp, "%.10g %.10g\n", obs[t], x[t]);
 	}
     }
 }
@@ -603,7 +612,7 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
     char xtitle[48];
     char ztitle[48];
     char title[128];
-    int v;
+    int v, err = 0;
 
     obs = gretl_plotx(datainfo);
     if (obs == NULL) {
@@ -615,8 +624,9 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
 	twoplot = 1;
     } 
 
-    if (gnuplot_init((twoplot)? PLOT_TRI_GRAPH : PLOT_REGULAR, &fp)) { 
-	return E_FOPEN;
+    fp = get_plot_input_stream((twoplot)? PLOT_TRI_GRAPH : PLOT_REGULAR, &err);
+    if (err) { 
+	return err;
     }
 
     if (!twoplot) {
@@ -704,21 +714,21 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
 
     fclose(fp);
 
-    if (gnuplot_make_graph()) {
-	errbox(_("gnuplot command failed"));
-	return 1;
-    } 
+    err = gnuplot_make_graph();
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	register_graph(NULL);
+    }
 
-    register_graph();
-
-    return 0;
+    return err;
 }
 
 static int save_filtered_var (filter_info *finfo, double *x, int i,
 			      int *saved)
 {
-    const char *vname = (i == FILTER_SAVE_TREND)? finfo->save1 :
-	finfo->save2;
+    const char *vname = (i == FILTER_SAVE_TREND)? finfo->save_t :
+	finfo->save_c;
     int v = series_index(datainfo, vname);
     int err = 0;
 
@@ -739,53 +749,70 @@ static int save_filtered_var (filter_info *finfo, double *x, int i,
     return err;
 }
 
-/* centered MA for even number of terms in average */
-
-static int sma_special (const filter_info *finfo, double *fx,
-			const double *x)
+static void record_filter_command (filter_info *finfo)
 {
-    int offset = finfo->nobs / 2;
-    double *tmp;
-    int t, t1, t2;
-    int i, k, n;
+    int trend = finfo->save_opt & FILTER_SAVE_TREND;
+    int cycle = finfo->save_opt & FILTER_SAVE_CYCLE;
+    char *s, fcmd[1024];
 
-    t1 = finfo->t1 + offset;
-    t2 = finfo->t2 - offset + 1;
-
-    n = t2 - t1 + 1;
-
-    tmp = malloc(n * sizeof *tmp);
-    if (tmp == NULL) {
-	return E_ALLOC;
+    if (finfo->ftype == FILTER_BK) {
+	sprintf(fcmd, "series %s = ", finfo->save_c);
+    } else if (finfo->ftype == FILTER_HP) {
+	if (trend) {
+	    sprintf(fcmd, "series %s = %s - ", finfo->save_t, finfo->vname);
+	} else if (cycle) {
+	    sprintf(fcmd, "series %s = ", finfo->save_c);
+	} 
+    } else if (trend) {
+	sprintf(fcmd, "series %s = ", finfo->save_t);
+    } else if (cycle) {
+	sprintf(fcmd, "series %s = %s - ", finfo->save_c, finfo->vname);
     }
 
-    for (t=t1; t<=t2; t++) {
-	fx[t] = 0.0;
-	for (i=0; i<finfo->nobs; i++) {
-	    k = i - offset + 1;
-	    fx[t] += x[t-k];
-	}
-	fx[t] /= finfo->nobs;
+    s = fcmd + strlen(fcmd);
+
+    gretl_push_c_numeric_locale();
+
+    if (finfo->ftype == FILTER_SMA) {
+	sprintf(s, "movavg(%s, %d, %d)\n", finfo->vname,
+		finfo->k, finfo->center);
+    } else if (finfo->ftype == FILTER_EMA) {
+	sprintf(s, "movavg(%s, %g, %d)\n", finfo->vname,
+		finfo->lambda, finfo->k);
+    } else if (finfo->ftype == FILTER_HP) {
+	sprintf(s, "hpfilt(%s, %g)\n", finfo->vname, finfo->lambda);
+    } else if (finfo->ftype == FILTER_BK) {
+	sprintf(s, "bkfilt(%s, %d, %d, %d)\n", finfo->vname,
+		finfo->bkl, finfo->bku, finfo->bkk);
+    } else if (finfo->ftype == FILTER_FD) {
+	sprintf(s, "fracdiff(%s, %g)\n", finfo->vname, finfo->lambda);
     }
 
-    for (t=0; t<n; t++) {
-	tmp[t] = fx[t+t1];
+    gretl_pop_c_numeric_locale();
+
+    if (trend) {
+	s = fcmd + strlen(fcmd);
+	sprintf(s, "setinfo %s -d \"%s\"\n", finfo->save_t, finfo->label_t);
+    } else if (cycle) {
+	s = fcmd + strlen(fcmd);
+	sprintf(s, "setinfo %s -d \"%s\"\n", finfo->save_c, finfo->label_c);
+    }	
+
+    if (trend && cycle) {
+	s = fcmd + strlen(fcmd);
+	sprintf(s, "series %s = %s - %s\n", finfo->save_c, finfo->vname, 
+		finfo->save_t);
+	s = fcmd + strlen(fcmd);
+	sprintf(s, "setinfo %s -d \"%s\"\n", finfo->save_c, finfo->label_c);
     }
 
-    for (t=t1; t<t2; t++) {
-	k = t - t1;
-	fx[t] = (tmp[k] + tmp[k+1]) / 2.0;
-    }
-
-    fx[t2] = NADBL;
-
-    free(tmp);
-
-    return 0;
+    add_command_to_stack(fcmd);
 }
 
 static int calculate_filter (filter_info *finfo)
 {
+    int save_t1 = datainfo->t1;
+    int save_t2 = datainfo->t2;
     const double *x = Z[finfo->vnum];
     double *fx = NULL;
     double *u = NULL;
@@ -811,59 +838,28 @@ static int calculate_filter (filter_info *finfo)
 	}
     }
 
+    datainfo->t1 = finfo->t1;
+    datainfo->t2 = finfo->t2;
+
     if (finfo->ftype == FILTER_SMA) {
-	if (finfo->center && finfo->nobs % 2 == 0) {
-	    err = sma_special(finfo, fx, x);
-	} else {
-	    int offset = finfo->center ? finfo->nobs / 2 : 0;
-	    int i, k, t1, t2;
-
-	    t1 = finfo->center ? finfo->t1 + offset : finfo->t1 + finfo->nobs - 1;
-	    t2 = finfo->center ? finfo->t2 - offset : finfo->t2;
-
-	    for (t=t1; t<=t2; t++) {
-		fx[t] = 0.0;
-		for (i=0; i<finfo->nobs; i++) {
-		    k = i - offset;
-		    fx[t] += x[t-k];
-		}
-		fx[t] /= finfo->nobs;
-	    }
-	}
+	/* simple moving average */
+	movavg_series(x, fx, datainfo, finfo->k, finfo->center);
     } else if (finfo->ftype == FILTER_EMA) {
-	int t1 = finfo->t1 + 1;
-
-	if (finfo->nobs == 0) {
-	    fx[finfo->t1] = gretl_mean(finfo->t1, finfo->t2, x);
-	} else {
-	    t1 = finfo->t1 + finfo->nobs - 1;
-	    fx[t1] = gretl_mean(finfo->t1, t1, x);
-	    t1++;
-	}
-	for (t=t1; t<=finfo->t2; t++) {
-	    fx[t] = finfo->lambda * x[t] + (1.0 - finfo->lambda) * fx[t-1];
-	}
-	for (t=finfo->t2; t>=t1; t--) {
-	    fx[t] = fx[t-1];
-	}
-	for (t=t1-1; t>=0; t--) {
-	    fx[t] = NADBL;
-	}
+	/* exponential moving average */
+	exponential_movavg_series(x, fx, datainfo, finfo->lambda, finfo->k);
     } else if (finfo->ftype == FILTER_HP) {
-	double l0 = libset_get_double(HP_LAMBDA);
-
-	libset_set_double(HP_LAMBDA, finfo->lambda);
-	err = hp_filter(x, fx, datainfo, OPT_T);
-	libset_set_double(HP_LAMBDA, l0);
+	/* Hodrick-Prescott */
+	err = hp_filter(x, fx, datainfo, finfo->lambda, OPT_T);
     } else if (finfo->ftype == FILTER_BK) {
-	set_bkbp_k(finfo->k);
-	set_bkbp_periods(finfo->bkl, finfo->bku);
-	err = bkbp_filter(x, u, datainfo);
-	unset_bkbp_k();
-	unset_bkbp_periods();
+	/* Baxter and King bandpass */
+	err = bkbp_filter(x, u, datainfo, finfo->bkl, finfo->bku, finfo->bkk);
     } else if (finfo->ftype == FILTER_FD) {
+	/* fractional differencing */
 	err = fracdiff_series(x, fx, finfo->lambda, 1, -1, datainfo);
     }
+
+    datainfo->t1 = save_t1;
+    datainfo->t2 = save_t2;
 	
     if (!err && fx != NULL && u != NULL) {
 	for (t=0; t<datainfo->n; t++) {
@@ -897,6 +893,10 @@ static int calculate_filter (filter_info *finfo)
 	mark_dataset_as_modified();
     }
 
+    if (!err && finfo->save_opt) {
+	record_filter_command(finfo);
+    }
+
     return err;
 }
 
