@@ -74,6 +74,7 @@ struct plot_editor_ {
     GtkWidget *notebook;
     GPT_SPEC *spec;
     gchar *fontname;
+    gchar *barsfile;
 
     GtkWidget **linetitle;
     GtkWidget **lineformula;
@@ -81,6 +82,7 @@ struct plot_editor_ {
     GtkWidget **yaxiscombo;
     GtkWidget **linescale;
     GtkWidget **linewidth;
+    GtkWidget **colorsel;
 
     GtkWidget *fitformula;
     GtkWidget *fitlegend;
@@ -92,10 +94,13 @@ struct plot_editor_ {
     GtkWidget *keycombo;
     GtkWidget *fitcombo;
     GtkWidget *border_check;
+    GtkWidget *grid_check;
     GtkWidget *y2_check;
+    GtkWidget *bars_check;
     GtkWidget *ttfcombo;
     GtkWidget *ttfspin;
     GtkWidget *fontcheck;
+    GtkWidget *barscombo;
 
     int gui_nlines;
     int gui_nlabels;
@@ -347,21 +352,31 @@ static int style_from_line_number (GPT_SPEC *spec, int i,
 	j = LT_AUTO;
     }
 
+    if (j >= spec->n_lines) {
+	/* don't go out of bounds */
+	j = -1;
+    }
+
     return j;
 }
 
 static GtkWidget *line_color_button (GPT_SPEC *spec, int i)
 {
     GtkWidget *image, *button;
-    int dotted = 0;
-    int j = style_from_line_number(spec, i, &dotted);
+    int j = 0, dotted = 0, shade = 0;
 
-    if (j < 0) {
-	/* shouldn't happen */
-	return NULL;
+    if (spec->lines[i].style == GP_STYLE_FILLEDCURVE) {
+	shade = 1;
+    } else {
+	j = style_from_line_number(spec, i, &dotted);
+	if (j < 0) {
+	    return NULL;
+	}
     }
 
-    if (spec->lines[j].rgb[0] != '\0') {
+    if (shade) {
+	image = get_image_for_color(get_graph_color(SHADECOLOR));
+    } else if (spec->lines[j].rgb[0] != '\0') {
 	gretlRGB color;
 
 	gretl_rgb_get(&color, spec->lines[j].rgb); 
@@ -390,22 +405,25 @@ static GtkWidget *line_color_button (GPT_SPEC *spec, int i)
     return button;
 }
 
-static void maybe_apply_line_color (GtkWidget *w, GPT_SPEC *spec,
-				    int i)
+static void apply_line_color (GtkWidget *cb, GPT_SPEC *spec,
+			      int i)
 {
     gretlRGB *rgb = NULL;
-    GtkWidget *cb;
-    int j;
-
-    cb = g_object_get_data(G_OBJECT(w), "colorsel");
-
+ 
     if (cb != NULL && GTK_WIDGET_SENSITIVE(cb)) {
 	rgb = g_object_get_data(G_OBJECT(cb), "rgb");
     }
 
     if (rgb != NULL) {
-	j = style_from_line_number(spec, i, NULL);
-	print_rgb_hash(spec->lines[j].rgb, rgb);
+	if (spec->lines[i].style == GP_STYLE_FILLEDCURVE) {
+	    set_graph_palette(SHADECOLOR, *rgb);
+	} else {
+	    int j = style_from_line_number(spec, i, NULL);
+
+	    if (j >= 0) {
+		print_rgb_hash(spec->lines[j].rgb, rgb);
+	    }
+	}
     }
 }
 
@@ -476,7 +494,7 @@ static void combo_to_gp_style (GtkWidget *w, int *sty)
     s = gtk_combo_box_get_active_text(GTK_COMBO_BOX(w));
 
     if (s != NULL && *s != '\0') {
-	*sty = gp_style_from_translation(s);
+	*sty = gp_style_index_from_display_name(s);
     }
 
     g_free(s);
@@ -575,9 +593,11 @@ static void dot_callback (GtkComboBox *box, GPT_SPEC *spec)
 
     /* the color selector is effective only for solid lines */
     w = g_object_get_data(G_OBJECT(box), "colorsel");
-    gtk_widget_set_sensitive(w, solid);
-    w = g_object_get_data(G_OBJECT(box), "color-label");
-    gtk_widget_set_sensitive(w, solid);
+    if (w != NULL) {
+	gtk_widget_set_sensitive(w, solid);
+	w = g_object_get_data(G_OBJECT(box), "color-label");
+	gtk_widget_set_sensitive(w, solid);
+    }
 }
 
 /* take a double (which might be NA) and format it for
@@ -738,6 +758,54 @@ static void set_graph_font_from_widgets (plot_editor *ed)
     g_free(tmp);
 }
 
+static gchar *default_bars_file (void)
+{
+    return g_strdup_printf("%sdata%cplotbars%cnber.txt",
+			   gretl_home(), SLASH, SLASH);
+}
+
+static void try_adding_plotbars (GPT_SPEC *spec, plot_editor *ed)
+{
+    png_plot *plot = (png_plot *) spec->ptr;
+    double xmin = -1, xmax = -1;
+    double ymin = -1, ymax = -1;
+    gchar *fname;
+    int err;
+
+    err = plot_get_coordinates(plot, &xmin, &xmax, &ymin, &ymax);
+
+    if (!err) {
+	if (!button_is_active(ed->axis_range[1].isauto)) {
+	    /* we have a manual y-axis setting that may override the
+	       current (ymin, ymax) as read from the plot */
+	    ymin = spec->range[1][0];
+	    ymax = spec->range[1][1];
+	} else {
+	    /* freeze the current (ymin, ymax) so that the addition
+	       of bars doesn't disturb the range */
+	    spec->range[1][0] = ymin;
+	    spec->range[1][1] = ymax;
+	}
+
+	if (gtk_combo_box_get_active(GTK_COMBO_BOX(ed->barscombo)) == 1
+	    && ed->barsfile != NULL) {
+	    err = plotspec_add_bars_info(spec, xmin, xmax,
+					 ymin, ymax, ed->barsfile);
+	} else {
+	    fname = default_bars_file();
+	    err = plotspec_add_bars_info(spec, xmin, xmax,
+					 ymin, ymax, fname);
+	    g_free(fname);
+	}
+    }
+
+    if (err) {
+	gui_errmsg(err);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ed->bars_check),
+				     FALSE);
+    }
+}
+
 /* Respond to "OK" or "Apply", or to hitting the Enter key in
    some parts of the plot editor dialog */
 
@@ -758,7 +826,7 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 
     if (ed->keycombo != NULL) {
         s = gtk_combo_box_get_active_text(GTK_COMBO_BOX(ed->keycombo));
-	spec->keyspec = gp_keypos_from_translation(s);
+	spec->keyspec = gp_keypos_from_display_name(s);
 	g_free(s);
     }
 
@@ -807,15 +875,17 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
     }
 
     for (i=0; i<ed->gui_nlines && !err; i++) {
+	GtkWidget *combo = ed->stylecombo[i];
+
 	line = &spec->lines[i];
-	if (ed->stylecombo[i] != NULL) {
+	if (combo != NULL && GTK_WIDGET_IS_SENSITIVE(combo)) {
 	    int oldalt = 0;
 
 	    if (line->style == GP_STYLE_FILLEDCURVE ||
 		line->style == GP_STYLE_ERRORBARS) {
 		oldalt = line->style;
 	    }
-	    combo_to_gp_style(ed->stylecombo[i], &line->style);
+	    combo_to_gp_style(combo, &line->style);
 	    if (oldalt == GP_STYLE_FILLEDCURVE &&
 		line->style == GP_STYLE_ERRORBARS) {
 		spec->flags |= GPT_ERR_SWITCH;
@@ -825,7 +895,7 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 		spec->flags |= GPT_FILL_SWITCH;
 		spec->flags &= ~GPT_ERR_SWITCH;
 	    }
-	    maybe_set_point_type(line, ed->stylecombo[i], i);
+	    maybe_set_point_type(line, combo, i);
 	}
 	if (ed->linetitle[i] != NULL) {
 	    entry_to_gp_string(ed->linetitle[i], line->title, 
@@ -842,7 +912,9 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 	if (ed->linewidth[i] != NULL) {
 	    line->width = 
 		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ed->linewidth[i]));
-	    maybe_apply_line_color(ed->linewidth[i], spec, i);
+	}
+	if (ed->colorsel[i] != NULL) {
+	    apply_line_color(ed->colorsel[i], spec, i);
 	}
     }
 
@@ -869,6 +941,14 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 	}
     } 
 
+    if (!err && ed->grid_check != NULL) {
+	if (button_is_active(ed->grid_check)) {
+	    spec->flags |= GPT_GRID;
+	} else {
+	    spec->flags &= ~GPT_GRID;
+	}
+    }
+
     if (!err) {
 	if (ed->fontname != NULL) {
 	    set_gretl_png_font(ed->fontname);
@@ -884,6 +964,14 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 	fittype_from_combo(GTK_COMBO_BOX(ed->fitcombo), spec);
     }
 
+    if (!err && ed->bars_check != NULL) {
+	if (button_is_active(ed->bars_check)) {
+	    try_adding_plotbars(spec, ed);
+	} else {
+	    plotspec_remove_bars(spec);
+	}
+    }
+
     if (!err) {
 	png_plot *plot = (png_plot *) spec->ptr;
 
@@ -1268,6 +1356,99 @@ static void add_color_selector (int i, GtkWidget *tbl, int cols,
     }
 }
 
+static void combo_set_ignore_changed (GtkComboBox *box, gboolean s)
+{
+    if (s) {
+	g_object_set_data(G_OBJECT(box), "ignore-changed", 
+			  GINT_TO_POINTER(1));
+    } else {
+	g_object_set_data(G_OBJECT(box), "ignore-changed", 
+			  NULL);
+    }
+}
+
+void set_plotbars_filename (const char *fname, gpointer data) 
+{
+    plot_editor *ed = (plot_editor *) data;
+    GtkComboBox *box = GTK_COMBO_BOX(ed->barscombo);
+
+    /* we should respond to the changed signal only when it
+       has been emitted by the user making a new selection,
+       otherwise we get into a loop
+    */
+    combo_set_ignore_changed(box, TRUE);
+
+    if (fname == NULL) {
+	/* canceled or error */
+	if (ed->barsfile == NULL) {
+	    gtk_combo_box_set_active(box, 0);
+	}
+    } else {
+	/* completed selection */
+	const char *p = strrchr(fname, SLASH);
+	const char *show;
+
+	show = (p != NULL)? (p+1) : fname;
+	g_free(ed->barsfile);
+	ed->barsfile = g_strdup(fname);
+	gtk_combo_box_remove_text(box, 1);
+	gtk_combo_box_append_text(box, show);
+	gtk_combo_box_append_text(box, _("other..."));
+	gtk_combo_box_set_active(box, 1);
+    }
+}
+
+static void plot_bars_changed (GtkComboBox *box, plot_editor *ed)
+{
+    int i = gtk_combo_box_get_active(box);
+
+    if (g_object_get_data(G_OBJECT(box), "ignore-changed")) {
+	combo_set_ignore_changed(box, FALSE);
+	return;
+    }
+
+    if (i == 0) {
+	; /* the default: no-op */
+    } else if (i == 1 && ed->barsfile != NULL) {
+	; /* selecting previously defined own file: no-op */
+    } else {
+	/* exploring... */
+	const char *msg = N_("To add your own \"bars\" to a plot, you must supply the\n"
+			     "name of a plain text file containing pairs of dates.");
+	GtkWidget *dlg;
+	gint ret;
+
+	dlg = gtk_dialog_new_with_buttons("gretl",
+					  GTK_WINDOW(ed->dialog),
+					  GTK_DIALOG_MODAL | 
+					  GTK_DIALOG_DESTROY_WITH_PARENT,
+					  GTK_STOCK_OPEN, 1,
+					  _("See example"), 2,
+					  GTK_STOCK_CANCEL, 3,
+					  NULL);
+
+	gretl_dialog_add_message(dlg, msg);
+	gtk_dialog_set_has_separator(GTK_DIALOG(dlg), FALSE);
+	gtk_window_set_keep_above(GTK_WINDOW(dlg), TRUE);  
+	ret = gtk_dialog_run(GTK_DIALOG(dlg));
+	gtk_widget_destroy(dlg);
+
+	if (ret != 1) {
+	    gtk_combo_box_set_active(box, 0);
+	}
+
+	if (ret == 1) {
+	    file_selector_with_parent(OPEN_BARS, FSEL_DATA_MISC, 
+				      ed, ed->dialog);
+	} else if (ret == 2) {
+	    gchar *fname = default_bars_file();
+
+	    view_file(fname, 0, 0, 60, 420, VIEW_FILE);
+	    g_free(fname);
+	}
+    }
+} 
+
 /* PNG anti-aliasing switch */
 
 static void set_aa_status (GtkWidget *w, int *ok)
@@ -1370,6 +1551,17 @@ static void add_gd_font_selector (plot_editor *ed, GtkWidget *tbl, int *rows)
     gtk_widget_show(ed->ttfspin);
 }
 
+static int show_bars_check (GPT_SPEC *spec)
+{
+    if (!(spec->flags & GPT_TS)) {
+	return 0;
+    }
+
+    return (spec->pd == 1 || 
+	    spec->pd == 4 || 
+	    spec->pd == 12);
+}
+
 static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec) 
 {
     static int aa_ok = 1;
@@ -1430,21 +1622,21 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
     gtk_table_attach_defaults(GTK_TABLE(tbl), 
 			      ed->keycombo, 1, TAB_MAIN_COLS, rows-1, rows);
     for (i=0; ; i++) {
-	gp_style_spec *kp = get_keypos_spec(i);
+	gp_key_spec *kp = get_keypos_spec(i);
 
 	if (kp == NULL) {
 	    break;
 	}
 	gtk_combo_box_append_text(GTK_COMBO_BOX(ed->keycombo), _(kp->str));
-	if (kp->sty == spec->keyspec) {
+	if (kp->id == spec->keyspec) {
 	    kactive = i;
 	}
     }
     gtk_combo_box_set_active(GTK_COMBO_BOX(ed->keycombo), kactive);
     gtk_widget_show(ed->keycombo);
 
-    /* choice of fitted line type, if appropriate */
     if (spec->fit != PLOT_FIT_NA) {
+	/* give choice of fitted line type, if applicable */
 	table_add_row(tbl, &rows, TAB_MAIN_COLS);
 
 	label = gtk_label_new(_("fitted line"));
@@ -1481,19 +1673,8 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 	gtk_widget_show(ed->fitcombo);
     } 
 
-    /* give option of removing/adding top & right border? */
-    if (!(spec->flags & GPT_Y2AXIS)) { 
-	if (spec->border == GP_BORDER_DEFAULT || spec->border == 3) {
-	    table_add_row(tbl, &rows, TAB_MAIN_COLS);
-	    ed->border_check = gtk_check_button_new_with_label(_("Show full border"));
-	    gtk_table_attach_defaults(GTK_TABLE(tbl), 
-				      ed->border_check, 0, TAB_MAIN_COLS, 
-				      rows-1, rows);
-	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ed->border_check),
-					 spec->border == GP_BORDER_DEFAULT);
-	    gtk_widget_show(ed->border_check);
-	}
-    } else {
+    if (spec->flags & GPT_Y2AXIS) { 
+	/* give option of forcing a single y-axis */
 	table_add_row(tbl, &rows, TAB_MAIN_COLS);
 	ed->y2_check = gtk_check_button_new_with_label(_("Use only one y axis"));
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
@@ -1504,14 +1685,38 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 	g_signal_connect(G_OBJECT(ed->y2_check), "clicked", 
 			 G_CALLBACK(toggle_axis_selection), ed);
 	gtk_widget_show(ed->y2_check);
+    } else {	
+	/* give option of removing/adding top & right border */
+	if (spec->border == GP_BORDER_DEFAULT || spec->border == 3) {
+	    table_add_row(tbl, &rows, TAB_MAIN_COLS);
+	    ed->border_check = gtk_check_button_new_with_label(_("Show full border"));
+	    gtk_table_attach_defaults(GTK_TABLE(tbl), 
+				      ed->border_check, 0, TAB_MAIN_COLS, 
+				      rows-1, rows);
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ed->border_check),
+					 spec->border == GP_BORDER_DEFAULT);
+	    gtk_widget_show(ed->border_check);
+	}
+    } 
+
+    if (1) {
+	/* FIXME does this need to be conditional? */
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);
+	ed->grid_check = gtk_check_button_new_with_label(_("Show grid"));
+	gtk_table_attach_defaults(GTK_TABLE(tbl), 
+				  ed->grid_check, 0, TAB_MAIN_COLS, 
+				  rows-1, rows);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ed->grid_check),
+				     spec->flags & GPT_GRID);
+	gtk_widget_show(ed->grid_check);
     }
 
     if (show_aa_check < 0) {
 	show_aa_check = (gnuplot_png_terminal() == GP_PNG_GD2);
     }
 
-    /* give option of suppressing anti-aliasing for PNGs */
     if (show_aa_check) {
+	/* give option of suppressing anti-aliasing for PNGs */
 	GtkWidget *aa_check;
 
 	table_add_row(tbl, &rows, TAB_MAIN_COLS);
@@ -1526,8 +1731,37 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 	gtk_widget_show(aa_check);
     }
 
-    /* setting of graph font, if gnuplot uses freetype */
+    if (show_bars_check(spec)) {
+	/* option to display NBER "recession bars" or similar */
+	GtkWidget *combo;
+
+	/* check button */
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);
+	ed->bars_check = gtk_check_button_new_with_label(_("Show bars"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ed->bars_check),
+				     spec->nbars > 0);
+	gtk_table_attach_defaults(GTK_TABLE(tbl), ed->bars_check,
+				  0, 1, rows-1, rows);
+	gtk_widget_show(ed->bars_check);
+
+	/* plus combo selector */
+	ed->barscombo = combo = gtk_combo_box_new_text();
+	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("NBER recessions"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("other..."));
+	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
+	gtk_widget_set_sensitive(combo, spec->nbars > 0);
+	g_signal_connect(G_OBJECT(combo), "changed", 
+			 G_CALLBACK(plot_bars_changed),
+			 ed);	
+	gtk_table_attach(GTK_TABLE(tbl), combo,
+			 1, 2, rows-1, rows,
+			 GTK_FILL, 0, 0, 0);
+	gtk_widget_show(combo);
+	sensitize_conditional_on(combo, ed->bars_check);
+    }
+
     if (pngcairo || gnuplot_has_ttf(0)) {
+	/* setting of graph font, if gnuplot supports freetype */
 	GtkWidget *hsep, *b;
 
 	/* first a separator */
@@ -1563,6 +1797,7 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
     } 
 
     if (gnuplot_png_terminal() != GP_PNG_OLD && frequency_plot_code(spec->code)) {
+	/* give option of setting fill color */
 	GtkWidget *hsep = gtk_hseparator_new();
 
 	table_add_row(tbl, &rows, TAB_MAIN_COLS);
@@ -2020,7 +2255,7 @@ static void flip_pointsel (GtkWidget *box, GtkWidget *targ)
 {
     GtkWidget *label = g_object_get_data(G_OBJECT(targ), "label");
     gchar *s = gtk_combo_box_get_active_text(GTK_COMBO_BOX(box));
-    int hp = has_point(gp_style_from_translation(s));
+    int hp = has_point(gp_style_index_from_display_name(s));
 
     gtk_widget_set_sensitive(targ, hp);
     gtk_widget_set_sensitive(label, hp);
@@ -2066,7 +2301,7 @@ static int gp_style_index (int sty, GList *list)
 
     while (mylist != NULL) {
 	spec = (gp_style_spec *) mylist->data;
-	if (sty == spec->sty) {
+	if (sty == spec->id) {
 	    return i;
 	}
 	i++;
@@ -2083,11 +2318,13 @@ void set_combo_box_strings_from_stylist (GtkComboBox *box, GList *list)
 
     while (mylist != NULL) {
 	spec = (gp_style_spec *) mylist->data;
-	gtk_combo_box_append_text(box, _(spec->str));
+	gtk_combo_box_append_text(box, _(spec->trname));
 	mylist = mylist->next;
     }
 }
 
+#define IRF_plot(c) (c == PLOT_IRFBOOT || c == PLOT_MULTI_IRF)
+
 static GList *add_style_spec (GList *list, int t)
 {
     return g_list_append(list, get_style_spec(t));
@@ -2098,9 +2335,10 @@ static GList *add_style_spec (GList *list, int t)
 static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
 {
     GtkWidget *notebook = ed->notebook;
+    GtkWidget *color_label = NULL;
     GtkWidget *label, *tbl;
     GtkWidget *vbox, *hbox, *sep;
-    GtkWidget *button, *page;
+    GtkWidget *page;
     int i, tbl_len, tbl_num, tbl_col;
     GList *stylist = NULL;
     int axis_chooser;
@@ -2159,6 +2397,8 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
 	GPT_LINE *line = &spec->lines[i];
 	int label_done = 0;
 
+	hbox = NULL;
+
 	if (line_is_formula(line)) {
 	    tbl_len++;
 	    gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
@@ -2235,12 +2475,8 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
 
 	/* the errorbars and filledcurves styles are not exchangeable
 	   with the others */
-	if (line->style == GP_STYLE_ERRORBARS && !gnuplot_has_style_fill()) {
-	    set_combo_box_default_text(GTK_COMBO_BOX(ed->stylecombo[i]), 
-				       _(gp_line_style_string(line->style)));
-	    gtk_widget_set_sensitive(ed->stylecombo[i], FALSE);
-	} else if (line->style == GP_STYLE_ERRORBARS ||
-		   line->style == GP_STYLE_FILLEDCURVE) {
+	if (line->style == GP_STYLE_ERRORBARS || 
+	    line->style == GP_STYLE_FILLEDCURVE) {
 	    GList *altsty = NULL;
 
 	    altsty = add_style_spec(altsty, GP_STYLE_ERRORBARS); 
@@ -2271,6 +2507,12 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
 			     G_CALLBACK(flip_pointsel), ptsel);
 	} 
 
+	if (IRF_plot(spec->code) && (line->style == GP_STYLE_FILLEDCURVE ||
+				     line->style == GP_STYLE_ERRORBARS)) {
+	    /* styles not exchangeable in this context */
+	    gtk_widget_set_sensitive(ed->stylecombo[i], FALSE);
+	}
+
 	gtk_table_attach_defaults(GTK_TABLE(tbl), hbox, 2, 3, tbl_len-1, tbl_len);
 	gtk_widget_show_all(hbox);	
 
@@ -2319,32 +2561,36 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
 	    gtk_spin_button_set_value(GTK_SPIN_BUTTON(ed->linewidth[i]),
 				      line->width);
 	}
-	g_signal_connect(G_OBJECT(ed->linewidth[i]), "activate", 
-			 G_CALLBACK(apply_gpt_changes), 
-			 ed);
 	gtk_box_pack_start(GTK_BOX(hbox), ed->linewidth[i], FALSE, FALSE, 0);
 	gtk_widget_show(ed->linewidth[i]);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), hbox, 2, 3, 
 				  tbl_len-1, tbl_len);
+	if (spec->lines[i].style != GP_STYLE_FILLEDCURVE) {
+	    g_signal_connect(G_OBJECT(ed->linewidth[i]), "activate", 
+			     G_CALLBACK(apply_gpt_changes), 
+			     ed);
+	} else {
+	    gtk_widget_set_sensitive(ed->linewidth[i], FALSE);
+	}
+
 	/* line color adjustment */
 	if (i < 6 && !frequency_plot_code(spec->code)) {
-	    button = line_color_button(spec, i);
-	    if (button != NULL) {
-		label = gtk_label_new(_("color"));
-		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-		gtk_widget_show(label);
-		gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-		g_object_set_data(G_OBJECT(ed->linewidth[i]), "colorsel",
-				  button);
-		gtk_widget_show_all(button);
+	    ed->colorsel[i] = line_color_button(spec, i);
+	    if (ed->colorsel[i] != NULL) {
+		color_label = gtk_label_new(_("color"));
+		gtk_box_pack_start(GTK_BOX(hbox), color_label, FALSE, FALSE, 0);
+		gtk_widget_show(color_label);
+		gtk_box_pack_start(GTK_BOX(hbox), ed->colorsel[i], 
+				   FALSE, FALSE, 0);
+		gtk_widget_show_all(ed->colorsel[i]);
 	    }
 	} else {
-	    label = NULL;
-	    button = NULL;
+	    ed->colorsel[i] = NULL;
+	    color_label = NULL;
 	}
 
 	if (line->flags & GP_LINE_USER) {
-	    /* dotted option */
+	    /* offer dotted option */
 	    GtkWidget *dotcombo = dash_types_combo();
 	    int dotted = (line->type == 0);
 
@@ -2352,14 +2598,18 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
 				     dotted ? 1 : 0);
 	    gtk_widget_show(dotcombo);
 	    widget_set_int(dotcombo, "linenum", i);
-	    g_object_set_data(G_OBJECT(dotcombo), "colorsel", button);
-	    g_object_set_data(G_OBJECT(dotcombo), "color-label", label);
-	    g_signal_connect(G_OBJECT(dotcombo), "changed", 
-			     G_CALLBACK(dot_callback), spec);
+	    if (ed->colorsel[i] != NULL) {
+		g_object_set_data(G_OBJECT(dotcombo), "colorsel", ed->colorsel[i]);
+		g_object_set_data(G_OBJECT(dotcombo), "color-label", color_label);
+		g_signal_connect(G_OBJECT(dotcombo), "changed", 
+				 G_CALLBACK(dot_callback), spec);
+	    }
 	    gtk_box_pack_start(GTK_BOX(hbox), dotcombo, FALSE, FALSE, 5);
 	}
 
-	gtk_widget_show(hbox);
+	if (hbox != NULL) {
+	    gtk_widget_show(hbox);
+	}
 
 	/* separator */
 	tbl_len++;
@@ -2373,15 +2623,17 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
     if ((spec->code == PLOT_REGULAR || spec->code == PLOT_CURVE)
 	&& spec->n_lines < 8) {
 	/* button for adding a line (formula) */
+	GtkWidget *add_button;
+
 	tbl_len++;
 	gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
-	button = gtk_button_new_with_label(_("Add line..."));
-	g_signal_connect(G_OBJECT(button), "clicked", 
+	add_button = gtk_button_new_with_label(_("Add line..."));
+	g_signal_connect(G_OBJECT(add_button), "clicked", 
 			 G_CALLBACK(add_line_callback), 
 			 ed);
-	gtk_widget_show(button);
-	gtk_table_attach_defaults(GTK_TABLE(tbl), button, 0, 1, 
-				  tbl_len-1, tbl_len);
+	gtk_widget_show(add_button);
+	gtk_table_attach_defaults(GTK_TABLE(tbl), add_button, 
+				  0, 1, tbl_len-1, tbl_len);
     }
 
     g_list_free(stylist);
@@ -2761,7 +3013,8 @@ static void plot_editor_destroy (plot_editor *ed)
 	ed->spec->n_labels = ed->old_n_labels;
     }  
 
-    free(ed->fontname);
+    g_free(ed->fontname);
+    g_free(ed->barsfile);
 
     free(ed->linetitle);
     free(ed->lineformula);
@@ -2769,6 +3022,7 @@ static void plot_editor_destroy (plot_editor *ed)
     free(ed->yaxiscombo);
     free(ed->linescale);
     free(ed->linewidth);
+    free(ed->colorsel);
 
     free(ed->labeltext);
     free(ed->labelpos);
@@ -2823,6 +3077,7 @@ static int add_line_widget (plot_editor *ed)
     ed->yaxiscombo  = widget_array_expand(&ed->yaxiscombo, n, &err);
     ed->linescale   = widget_array_expand(&ed->linescale, n, &err);
     ed->linewidth   = widget_array_expand(&ed->linewidth, n, &err);
+    ed->colorsel    = widget_array_expand(&ed->colorsel, n, &err);
     
     if (!err) {
 	ed->gui_nlines = n;
@@ -2842,6 +3097,7 @@ static int allocate_line_widgets (plot_editor *ed, int n)
 	ed->yaxiscombo  = widget_array_new(n, &err);
 	ed->linescale   = widget_array_new(n, &err);
 	ed->linewidth   = widget_array_new(n, &err);
+	ed->colorsel    = widget_array_new(n, &err);
 	if (!err) {
 	    ed->gui_nlines = n;
 	}
@@ -2894,6 +3150,7 @@ static plot_editor *plot_editor_new (GPT_SPEC *spec)
 
     ed->spec = spec;
     ed->fontname = NULL;
+    ed->barsfile = NULL;
 
     ed->dialog = NULL;
 
@@ -2903,6 +3160,7 @@ static plot_editor *plot_editor_new (GPT_SPEC *spec)
     ed->yaxiscombo = NULL;
     ed->linescale = NULL;
     ed->linewidth = NULL;
+    ed->colorsel = NULL;
 
     ed->labeltext = NULL;
     ed->labelpos = NULL;
@@ -2917,10 +3175,13 @@ static plot_editor *plot_editor_new (GPT_SPEC *spec)
     ed->keycombo = NULL;
     ed->fitcombo = NULL;
     ed->border_check = NULL;
+    ed->grid_check = NULL;
     ed->y2_check = NULL;
+    ed->bars_check = NULL;
     ed->ttfcombo = NULL;
     ed->ttfspin = NULL;
     ed->fontcheck = NULL;
+    ed->barscombo = NULL;
 
     ed->gui_nlines = ed->gui_nlabels = 0;
 
