@@ -258,7 +258,7 @@ static void spreadsheet_edit (void)
 
 static void varinfo_callback (void)
 {
-    varinfo_dialog(mdata->active_var, 1);
+    varinfo_dialog(mdata->active_var);
 }
 
 static void options_dialog_callback (void)
@@ -336,6 +336,8 @@ static int script_type (const char *fname)
 	return EDIT_GP;
     } else if (ox_support && has_suffix(fname, ".ox")) {
 	return EDIT_OX;
+    } else if (has_suffix(fname, ".m")) {
+	return EDIT_OCTAVE;
     } else {
 	return 0;
     }
@@ -827,7 +829,7 @@ static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 	if (selcount == 1 && vnum != 0) {
 	    mdata->active_var = vnum;
 	    if (k == GDK_e || k == GDK_F2) {
-		varinfo_dialog(mdata->active_var, 1);
+		varinfo_dialog(mdata->active_var);
 	    } else if (k == GDK_t) {
 		do_graph_var(mdata->active_var);
 	    } else if (k == GDK_Return) {
@@ -922,6 +924,8 @@ void populate_varlist (void)
 {
     static gint check_connected;
     static gint click_connected;
+    GtkTreeView *view;
+    GtkTreeModel *model;
     GtkTreeStore *store;
     GtkTreeSelection *select;
     GtkTreeIter iter;    
@@ -929,16 +933,18 @@ void populate_varlist (void)
     int pos = 0;
     int i;
 
-    store = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(mdata->listbox)));
+    view = GTK_TREE_VIEW(mdata->listbox);
+    model = gtk_tree_view_get_model(view);
+    store = GTK_TREE_STORE(model);
 
     if (store != NULL) {
 	/* record line position? */
-	pos = get_line_pos(GTK_TREE_MODEL(store));
+	pos = get_line_pos(model);
     }
     
     gtk_tree_store_clear(store);
 
-    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+    gtk_tree_model_get_iter_first(model, &iter);
 
     for (i=0; i<datainfo->v; i++) {
 	int pv = 0;
@@ -975,12 +981,12 @@ void populate_varlist (void)
 	}
     } 
 
-    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+    gtk_tree_model_get_iter_first(model, &iter);
 
     if (pos == 0) {
 	/* no saved position */
 	pos = 1;
-	gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+	gtk_tree_model_iter_next(model, &iter);
     } else {	
 	/* try to return to previous position */
 	GtkTreeIter last;
@@ -988,7 +994,7 @@ void populate_varlist (void)
 	i = 1;
 	while (1) {
 	    last = iter;
-	    if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter)) {
+	    if (!gtk_tree_model_iter_next(model, &iter)) {
 		/* reached the end! */
 		iter = last;
 		break;
@@ -1002,7 +1008,7 @@ void populate_varlist (void)
     } 
 
     mdata->active_var = pos;
-    select = gtk_tree_view_get_selection(GTK_TREE_VIEW(mdata->listbox));
+    select = gtk_tree_view_get_selection(view);
     gtk_tree_selection_select_iter(select, &iter);
 
     if (datainfo->v > 1) {
@@ -1010,7 +1016,7 @@ void populate_varlist (void)
 
 	sprintf(id, "%d", pos);
 	path = gtk_tree_path_new_from_string(id);
-	gtk_tree_view_set_cursor(GTK_TREE_VIEW(mdata->listbox), path, NULL, FALSE);
+	gtk_tree_view_set_cursor(view, path, NULL, FALSE);
 	gtk_tree_path_free(path);
     }
 
@@ -1040,21 +1046,26 @@ void populate_varlist (void)
 void mdata_select_last_var (void)
 {
     GtkTreeIter iter, last;
+    GtkTreeView *view;
+    GtkTreeModel *model;
     GtkTreeStore *store;
     GtkTreeSelection *select;
 
-    store = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(mdata->listbox)));
-    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+    view = GTK_TREE_VIEW(mdata->listbox);
+    model = gtk_tree_view_get_model(view);
+    store = GTK_TREE_STORE(model);
+
+    gtk_tree_model_get_iter_first(model, &iter);
 
     while (1) {
 	last = iter;
-	if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter)) {
+	if (!gtk_tree_model_iter_next(model, &iter)) {
 	    iter = last;
 	    break;
 	}
     }
 
-    select = gtk_tree_view_get_selection(GTK_TREE_VIEW(mdata->listbox));
+    select = gtk_tree_view_get_selection(view);
     gtk_tree_selection_unselect_all(select);
     gtk_tree_selection_select_iter(select, &iter);
 }
@@ -1312,9 +1323,8 @@ GtkActionEntry main_entries[] = {
     { "GretlScript", NULL, N_("gretl script"), NULL, NULL, G_CALLBACK(new_script_callback) },
     { "GnuplotScript", NULL, N_("gnuplot script"), NULL, NULL, G_CALLBACK(new_script_callback) },
     { "RScript", NULL, N_("R script"), NULL, NULL, G_CALLBACK(new_script_callback) },
-#ifdef USE_OX
+    { "OctaveScript", NULL, N_("Octave script"), NULL, NULL, G_CALLBACK(new_script_callback) },
     { "OxScript", NULL, N_("Ox program"), NULL, NULL, G_CALLBACK(new_script_callback) },
-#endif
 
     { "SessionFiles", NULL, N_("_Session files"), NULL, NULL, NULL },
     { "OpenSession", GTK_STOCK_OPEN, N_("_Open session..."), "", NULL, G_CALLBACK(open_code_callback) },
@@ -1370,6 +1380,7 @@ GtkActionEntry main_entries[] = {
     { "RemoveObs", NULL, N_("_Remove extra observations"), NULL, NULL, G_CALLBACK(do_remove_obs) },
     { "DataInfo", NULL, N_("_Dataset info"), NULL, NULL, G_CALLBACK(dataset_info) },
     { "DataMarkers", NULL, N_("_Observation markers..."), NULL, NULL, G_CALLBACK(markers_callback) },
+    { "VarLabels", NULL, N_("_Variable labels..."), NULL, NULL, G_CALLBACK(labels_callback) },
     { "DataStructure", NULL, N_("Dataset _structure..."), NULL, NULL, G_CALLBACK(data_structure_dialog) },
     { "DataCompact", NULL, N_("_Compact data..."), NULL, NULL, G_CALLBACK(do_compact_data_set) },
     { "DataExpand", NULL, N_("_Expand data..."), NULL, NULL, G_CALLBACK(do_expand_data_set) },
@@ -1518,7 +1529,8 @@ GtkActionEntry main_entries[] = {
     { "oprobit", NULL, N_("_Ordered..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "tobit", NULL, N_("To_bit..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "heckit", NULL, N_("_Heckit..."), NULL, NULL, G_CALLBACK(model_callback) }, 
-    { "poisson", NULL, N_("Poi_sson..."), NULL, NULL, G_CALLBACK(model_callback) }, 
+    { "countmod", NULL, N_("_Count data..."), NULL, NULL, G_CALLBACK(model_callback) }, 
+    { "duration", NULL, N_("_Duration data..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "logistic", NULL, N_("Lo_gistic..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "nls", NULL, N_("_Nonlinear Least Squares..."), NULL, NULL, G_CALLBACK(gretl_callback) }, 
     { "intreg", NULL, N_("_Interval regression..."), NULL, NULL, G_CALLBACK(model_callback) }, 
@@ -1577,7 +1589,6 @@ static void add_conditional_items (GtkUIManager *ui)
 			  FALSE);
 #endif
 
-#ifdef USE_OX
     if (ox_support) {
 	gtk_ui_manager_add_ui(ui, gtk_ui_manager_new_merge_id(ui),
 			      "/menubar/File/ScriptFiles/NewScript/OxScript",
@@ -1586,7 +1597,6 @@ static void add_conditional_items (GtkUIManager *ui)
 			      GTK_UI_MANAGER_MENUITEM, 
 			      FALSE);
     }
-#endif
 }
 
 /* retrieve the XML description of the main window menus */
