@@ -26,7 +26,6 @@ enum extra_cmds {
     GR_XYZ,
     GR_BOX,
     GR_NBOX,
-    GR_QQ,
     GR_3D,
     COMPACT,
     EXPAND, 
@@ -57,6 +56,7 @@ enum extra_cmds {
     EDIT_GP,
     EDIT_R,
     EDIT_OX,
+    EDIT_OCTAVE,
     CLI_HELP,
     GUI_HELP,
     CLI_HELP_EN,
@@ -104,6 +104,7 @@ enum extra_cmds {
     IV_LIML,
     IV_GMM,
     BFGS_CONFIG,
+    COUNTMOD,
     GUI_CMD_MAX
 };
 
@@ -137,6 +138,8 @@ enum file_ops {
     APPEND_JMULTI,   /* end of "data append" entries */
     OPEN_SESSION,
     OPEN_MARKERS,
+    OPEN_LABELS,
+    OPEN_BARS,
     OPEN_GFN,
     END_OPEN,        /* marker for end of file open section */
     SAVE_DATA,
@@ -160,12 +163,14 @@ enum file_ops {
     SAVE_GP_CMDS,
     SAVE_R_CMDS,
     SAVE_OX_CMDS,
+    SAVE_OCTAVE_CMDS,
     SAVE_CONSOLE,
     SAVE_CMD_LOG,
     SAVE_FUNCTIONS,
     SAVE_FUNCTIONS_AS,
     SAVE_BOOT_DATA,
     SAVE_MARKERS,
+    SAVE_LABELS,
     EDIT_FUNCTIONS,
     SET_PROG,
     SET_DIR,
