@@ -900,7 +900,7 @@ obs_button_motion_notify (GtkWidget      *widget,
     if (event->window == spin->panel) {
 	gint y;
       
-	gdk_window_get_pointer (spin->panel, NULL, &y, NULL);
+	gdk_window_get_pointer(spin->panel, NULL, &y, NULL);
   
 	if (y <= widget->requisition.height / 2 && 
 	    spin->in_child == GTK_ARROW_DOWN) {
@@ -1098,7 +1098,7 @@ obs_button_default_output (ObsButton *obs_button)
     gchar buf[OBSLEN];
     gpointer data;
 
-    ntodate_full(buf, ival, obs_button->pdinfo);
+    ntodate(buf, ival, obs_button->pdinfo);
 
     if (strcmp(buf, gtk_entry_get_text(GTK_ENTRY(obs_button))))
 	gtk_entry_set_text(GTK_ENTRY(obs_button), buf);
