@@ -106,7 +106,7 @@ struct SESSION_TEXT_ {
 };
 
 struct SESSION_MODEL_ {
-    char *name;
+    char name[MAXSAVENAME];
     void *ptr;
     GretlObjType type;
 };
@@ -318,7 +318,7 @@ static SESSION_MODEL *session_model_new (void *ptr, const char *name,
 	mod->ptr = ptr;
 	mod->type = type;
 	gretl_stack_object_as(ptr, type, name);
-	mod->name = gretl_object_get_name(ptr, type);
+	strcpy(mod->name, gretl_object_get_name(ptr, type));
 	/* note: we don't add a reference here: that's handled by the
 	   mechanism in objstack.c 
 	*/
@@ -699,22 +699,22 @@ void save_output_as_text_icon (windata_t *vwin)
     }
 }
 
-static int real_add_model_to_session (void *ptr, const char *name,
-				      GretlObjType type)
+static int add_model_to_session (void *ptr, const char *name,
+				 GretlObjType type)
 {
     SESSION_MODEL *mod;
+    int err = 0;
 
 #if SESSION_DEBUG
-    fprintf(stderr, "real_add_model_to_session: doing session_model_new\n"
+    fprintf(stderr, "add_model_to_session: doing session_model_new\n"
 	    " with ptr = %p\n", ptr);
 #endif
 
     mod = session_model_new(ptr, name, type);
-    if (mod == NULL || session_append_model(mod)) {
-	return 1;
-    } 
 
-    if (iconlist != NULL) {
+    if (mod == NULL || session_append_model(mod)) {
+	err = E_ALLOC;
+    } else if (iconlist != NULL) {
 	session_add_icon(mod, type, ICON_ADD_SINGLE);
     } else if (autoicon_on()) {
 	view_session(NULL);
@@ -856,6 +856,8 @@ int cli_add_graph_to_session (const char *fname, const char *gname,
 	return ADD_OBJECT_FAIL;
     } 
 
+    /* we copied the plot commands file, @fname, into the
+       session directory; now delete the original */
     gretl_remove(fname);
 
     return real_add_graph_to_session(name, gname, type);
@@ -912,30 +914,37 @@ static void delete_icon_for_data (void *data)
     }
 }
 
-int maybe_add_model_to_session (void *ptr, GretlObjType type,
-				const char *name)
+/* Callback (indirect) from libgretl for a model created via 
+   script command. When this function is called, the model
+   in question has already been stacked; it is just a matter
+   of syncing the GUI session with the model stack state.
+*/
+
+int add_model_to_session_callback (void *ptr, GretlObjType type)
 {
-    SESSION_MODEL *oldmod;
+    SESSION_MODEL *targ = NULL;
+    char *name = gretl_object_get_name(ptr, type);
+    int ret = 0;
 
-    if (get_session_model_by_data(ptr)) {
-	/* model already present */
-	return 0;
+    /* are we replacing a session model's content? */
+    targ = get_session_model_by_name(name); 
+
+    if (targ != NULL) {
+	GtkWidget *w = match_window_by_data(targ->ptr);
+
+	if (w != NULL) {
+	    /* current model window is "orphaned" */
+	    gtk_widget_destroy(w);
+	}
+
+	targ->ptr = ptr;
+	targ->type = type;
+	mark_session_changed();
+    } else {
+	ret = add_model_to_session(ptr, name, type);
     }
 
-    /* check to see if there's already a (different) model with the
-       same name: if so, delete it
-    */
-    if (name == NULL) {
-	name = gretl_object_get_name(ptr, type);
-    }
-    oldmod = get_session_model_by_name(name); 
-
-    if (oldmod != NULL) {
-	delete_icon_for_data(oldmod);
-	real_delete_model_from_session(oldmod);
-    }
-
-    return real_add_model_to_session(ptr, name, type);
+    return ret;
 }
 
 static int model_type_from_vwin (windata_t *vwin)
@@ -995,7 +1004,7 @@ void model_add_as_icon (GtkAction *action, gpointer p)
 	return;
     }    
 
-    err = real_add_model_to_session(ptr, name, type);
+    err = add_model_to_session(ptr, name, type);
 
     if (!err) {
 	mark_session_changed();
@@ -2143,7 +2152,7 @@ static void maybe_delete_session_object (gui_obj *obj)
 
 static gui_obj *get_gui_obj_by_data (void *targ)
 {
-    GList *mylist = iconlist;
+    GList *mylist = g_list_first(iconlist);
     gui_obj *obj = NULL;
 
     while (mylist != NULL) {
@@ -2218,6 +2227,8 @@ static int rename_session_object (gui_obj *obj, const char *newname)
 	} else {
 	    sm = obj->data;
 	    gretl_object_rename(sm->ptr, sm->type, newname);
+	    *sm->name = '\0';
+	    strncat(sm->name, newname, MAXSAVENAME - 1);
 	    maybe_sync_model_window_name(sm);
 	}
     } else if (obj->sort == GRETL_OBJ_GRAPH || obj->sort == GRETL_OBJ_PLOT) {
@@ -2818,7 +2829,7 @@ static void object_popup_callback (GtkWidget *widget, gpointer data)
 
 	    gretl_chdir(gretl_dotdir());
 	    session_file_make_path(fullname, graph->fname);
-	    remove_png_term_from_plotfile_by_name(fullname);
+	    remove_png_term_from_plot_by_name(fullname);
 	    vwin = view_file(fullname, 1, 0, 78, 400, EDIT_GP);
 	    object_set_window_title(vwin, obj);
 	    /* add flag so we can mark the session as modified
