@@ -167,10 +167,8 @@ RCVAR rc_vars[] = {
       BOOLSET, 0, TAB_MAIN, NULL },
     { "session_prompt", N_("Prompt to save session"), NULL, &session_prompt, 
       BOOLSET, 0, TAB_MAIN, NULL },
-#ifdef USE_OX
     { "oxsupport", N_("Enable Ox support"), NULL, &ox_support, 
       BOOLSET | RESTART, 0, TAB_MAIN, NULL },
-#endif
     { "usecwd", N_("Set working directory from shell"), NULL, &usecwd, 
       INVISET | BOOLSET | RESTART, 0, TAB_NONE, NULL },
     { "keepfolder", N_("File selector remembers folder"), NULL, &keep_folder, 
@@ -217,10 +215,10 @@ RCVAR rc_vars[] = {
     { "Rlib", N_("Path to R library"), NULL, paths.rlibpath, 
       MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL},
 #endif
-#ifdef USE_OX
     { "ox", N_("Path to oxl executable"), NULL, paths.oxlpath, 
       MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL},
-#endif
+    { "octave", N_("Path to octave executable"), NULL, paths.octpath, 
+      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL},
     { "ratsbase", N_("RATS data directory"), NULL, paths.ratsbase, 
       USERSET | BROWSER, MAXLEN, TAB_DBS, NULL },
     { "dbhost", N_("Database server name"), NULL, paths.dbhost, 
@@ -1167,12 +1165,10 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    continue;
 	}
 
-#ifdef USE_OX
 	if (!ox_support && rc->var == paths.oxlpath) {
 	    /* don't show ox path entry if support is not enabled */
 	    continue;
 	}
-#endif
 
 	if ((rc->flags & BOOLSET) && rc->link == NULL) { 
 	    /* simple boolean variable (check box) */
@@ -1910,7 +1906,7 @@ static int get_network_settings (void)
    except that we'll respect the registry entry for gretldir.
 */
 
-static int win32_read_gretlrc (void) 
+static void win32_read_gretlrc (void) 
 {
     char line[MAXLEN], key[32], linevar[MAXLEN];
     int got_recent = 0;
@@ -1918,14 +1914,14 @@ static int win32_read_gretlrc (void)
 
     if (*rcfile == '\0') {
 	/* shouldn't happen */
-	return 1;
+	return;
     }
 
     fp = gretl_fopen(rcfile, "r");
 
     if (fp == NULL) {
 	/* not necessarily an error: may be starting from scratch */
-	return 1;
+	return;
     }
 
     while (fgets(line, sizeof line, fp) != NULL) {
