@@ -20,9 +20,13 @@ void tree_view_get_int (GtkTreeView *view, int row, int col, int *val);
 
 int tree_path_get_row_number (GtkTreePath *path); 
 
+void tree_model_get_iter_last (GtkTreeModel *mod, GtkTreeIter *iter);
+
 gboolean main_varclick (GtkWidget *widget, GdkEventButton *event,
 			windata_t *win);
 
 int *main_window_selection_as_list (void); 
 
+void set_main_colheads_clickable (gboolean s);
+
 #endif /* TREEUTILS_H */
