@@ -566,23 +566,30 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int flag,
     return cmp;
 }
 
-/* reconstitute full varlist for WLS, POISSON, AR and models */
+static int get_extra_var (const MODEL *pmod)
+{
+    if (COUNT_MODEL(pmod->ci)) {
+	return gretl_model_get_int(pmod, "offset_var");
+    } else if (pmod->ci == DURATION) {
+	return gretl_model_get_int(pmod, "cens_var");
+    } else {
+	return 0;
+    }
+}
+
+/* reconstitute full varlist for WLS, AR and count models */
 
 static int *
 full_model_list (const MODEL *pmod, const int *inlist)
 {
-    int i, len, pos = 0;
+    int i, len = 0, pos = 0;
     int *flist = NULL;
 
-    if (pmod->ci != WLS && pmod->ci != POISSON && pmod->ci != AR) {
-	return NULL;
-    }
-
     if (pmod->ci == WLS) { 
 	len = inlist[0] + 2;
-    } else if (pmod->ci == POISSON) {
+    } else if (COUNT_MODEL(pmod->ci)) {
 	len = inlist[0] + 3;
-    } else {
+    } else if (pmod->ci == AR) {
 	pos = pmod->arinfo->arlist[0] + 1;
 	len = pos + inlist[0] + 2;
     }
@@ -598,15 +605,15 @@ full_model_list (const MODEL *pmod, const int *inlist)
 	for (i=1; i<=inlist[0]; i++) {
 	    flist[i+1] = inlist[i];
 	}
-    } else if (pmod->ci == POISSON) {
-	int offvar = gretl_model_get_int(pmod, "offset_var");
+    } else if (COUNT_MODEL(pmod->ci) || pmod->ci == DURATION) {
+	int extra = get_extra_var(pmod);
 
 	flist[0] = len - 1;
 	for (i=1; i<=inlist[0]; i++) {
 	    flist[i] = inlist[i];
 	}
 	flist[flist[0] - 1] = LISTSEP;
-	flist[flist[0]] = offvar;
+	flist[flist[0]] = extra;
     } else if (pmod->ci == AR) {
 	flist[0] = len - 2;
 	for (i=1; i<pos; i++) {
@@ -693,8 +700,7 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 	    myopt |= OPT_P;
 	}
 	rho = estimate_rho(list, pZ, pdinfo, myopt, prn, &rep.errcode);
-    } else if (orig->ci == WLS || orig->ci == AR || 
-	       (orig->ci == POISSON && gretl_model_get_int(orig, "offset_var"))) {
+    } else if (orig->ci == WLS || orig->ci == AR || get_extra_var(orig)) {
 	int *full_list = full_model_list(orig, list);
 
 	free(list);
@@ -718,8 +724,10 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 	if (gretl_model_get_int(orig, "pooled")) {
 	    myopt |= OPT_P;
 	} else if (orig->opt & OPT_U) {
+	    /* random effects */
 	    myopt |= OPT_U;
 	} else if (orig->opt & OPT_W) {
+	    /* unit weights */
 	    myopt |= OPT_W;
 	    if (gretl_model_get_int(orig, "iters")) {
 		myopt |= OPT_I;
@@ -765,7 +773,7 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 	rep = logit_probit(list, pZ, pdinfo, orig->ci, myopt, NULL);
 	break;
     case TOBIT:
-	rep = tobit_model(list, pZ, pdinfo, NULL);
+	rep = tobit_model(list, pZ, pdinfo, OPT_NONE, NULL);
 	break;
     case LAD:
 	if (gretl_model_get_int(orig, "rq")) {
@@ -775,7 +783,11 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 	}
 	break;
     case POISSON:
-	rep = poisson_model(list, pZ, pdinfo, NULL);
+    case NEGBIN:
+	rep = count_model(list, orig->ci, pZ, pdinfo, myopt, NULL);
+	break;
+    case DURATION:
+	rep = duration_model(list, pZ, pdinfo, myopt, NULL);
 	break;
     case HECKIT:
 	rep = heckit_model(list, pZ, pdinfo, myopt, NULL);
@@ -2264,11 +2276,12 @@ static int QLR_graph (const double *Ft, int t1, int t2,
 		      int tmax, int dfn, const DATAINFO *pdinfo)
 {
     const double *x = gretl_plotx(pdinfo);
-    FILE *fp = NULL;
-    int t;
+    FILE *fp;
+    int t, err = 0;
 
-    if (gnuplot_init(PLOT_REGULAR, &fp)) {
-	return E_FOPEN;
+    fp = get_plot_input_stream(PLOT_REGULAR, &err);
+    if (err) {
+	return err;
     }
 
     print_keypos_string(GP_KEY_LEFT_TOP, fp);
@@ -2704,14 +2717,13 @@ static int cusum_do_graph (double a, double b, const double *W,
 			   int t1, int k, int m, 
 			   DATAINFO *pdinfo, gretlopt opt)
 {
-    FILE *fq = NULL;
+    FILE *fp = NULL;
     const double *obs = NULL;
-    double x0 = 0.0;
-    int j, t;
-    int err = 0;
     double frac = 1.0;
+    double x0 = 0.0;
+    int j, t, err = 0;
 
-    err = gnuplot_init(PLOT_CUSUM, &fq);
+    fp = get_plot_input_stream(PLOT_CUSUM, &err);
     if (err) {
 	return err;
     }
@@ -2727,41 +2739,41 @@ static int cusum_do_graph (double a, double b, const double *W,
 
     gretl_push_c_numeric_locale();
 
-    fprintf(fq, "set xlabel '%s'\n", _("Observation"));
-    fputs("set nokey\n", fq);
+    fprintf(fp, "set xlabel '%s'\n", _("Observation"));
+    fputs("set nokey\n", fp);
 
     if (opt & OPT_R) {
-	fprintf(fq, "set title '%s'\n",
+	fprintf(fp, "set title '%s'\n",
 		/* xgettext:no-c-format */
 		_("CUSUMSQ plot with 95% confidence band"));
-	fprintf(fq, "plot \\\n%g*(x-%g) title '' w dots lt 2, \\\n", b, x0 - frac);
-	fprintf(fq, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", -a, b, x0 - frac);
-	fprintf(fq, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, b, x0 - frac);
+	fprintf(fp, "plot \\\n%g*(x-%g) title '' w dots lt 2, \\\n", b, x0 - frac);
+	fprintf(fp, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", -a, b, x0 - frac);
+	fprintf(fp, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, b, x0 - frac);
     } else {
-	fputs("set xzeroaxis\n", fq);
-	fprintf(fq, "set title '%s'\n",
+	fputs("set xzeroaxis\n", fp);
+	fprintf(fp, "set title '%s'\n",
 		/* xgettext:no-c-format */
 		_("CUSUM plot with 95% confidence band"));
-	fprintf(fq, "plot \\\n%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, b, x0);
-	fprintf(fq, "%g-%g*(x-%g) title '' w lines lt 2, \\\n", -a, b, x0);
+	fprintf(fp, "plot \\\n%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, b, x0);
+	fprintf(fp, "%g-%g*(x-%g) title '' w lines lt 2, \\\n", -a, b, x0);
     }	
 
-    fputs("'-' using 1:2 w linespoints lt 1\n", fq);
+    fputs("'-' using 1:2 w linespoints lt 1\n", fp);
 
     for (j=0; j<m; j++) { 
 	t = t1 + k + j;
 	if (obs != NULL) {
-	    fprintf(fq, "%g %g\n", obs[t], W[j]);
+	    fprintf(fp, "%g %g\n", obs[t], W[j]);
 	} else {
-	    fprintf(fq, "%d %g\n", t, W[j]);
+	    fprintf(fp, "%d %g\n", t, W[j]);
 	}
     }
 
-    fputs("e\n", fq);
+    fputs("e\n", fp);
 
     gretl_pop_c_numeric_locale();
 
-    fclose(fq);
+    fclose(fp);
 
     err = gnuplot_make_graph();
 
