@@ -55,18 +55,21 @@ typedef enum {
     AUX_GROUPWISE, /* testing for groupwise heteroskedasticity */
     AUX_HET_1, /* aux. regression for Pesaran-Taylor HET_1 test */
     AUX_BP,    /* aux. regression for Breusch-Pagan heterosked. test */
-    AUX_AUX    /* auxiliary regression not otherwise specified */
+    AUX_AUX,   /* auxiliary regression not otherwise specified */
+    AUX_COMFAC /* aux. regression for common factor test */
 } ModelAuxCode;
 
 /* functions follow */
  
-double robust_omit_F (const int *list, MODEL *pmod);
+double wald_omit_F (const int *list, MODEL *pmod);
 
-int add_test (const int *addvars,  MODEL *orig, MODEL *new,
+double wald_omit_chisq (const int *list, MODEL *pmod);
+
+int add_test (const int *addvars,  MODEL *orig, MODEL *pmod,
 	      double ***pZ, DATAINFO *pdinfo, 
 	      gretlopt opt, PRN *prn);
 
-int omit_test (const int *omitvars, MODEL *orig, MODEL *new, 
+int omit_test (const int *omitvars, MODEL *orig, MODEL *pmod, 
 	       double ***pZ, DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn);
 
@@ -80,7 +83,11 @@ int autocorr_test (MODEL *pmod, int order,
 		   double ***pZ, DATAINFO *pdinfo, 
 		   gretlopt opt, PRN *prn);
 
-double ljung_box (int m, int t1, int t2, const double *y, int *err);
+int comfac_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
+		 gretlopt opt, PRN *prn);
+
+double get_dw_pvalue (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
+		      int *err);
 
 int chow_test (const char *line, MODEL *pmod, 
 	       double ***pZ, DATAINFO *pdinfo, 
@@ -105,8 +112,8 @@ int leverage_test (MODEL *pmod,
 int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
 				    gretl_matrix *m, unsigned char flags);
 
-int lmtest_driver (const char *param,
-		   double ***pZ, DATAINFO *pdinfo, 
-		   gretlopt opt, PRN *prn);
+int model_test_driver (const char *param,
+		       double ***pZ, DATAINFO *pdinfo, 
+		       gretlopt opt, PRN *prn);
 
 #endif /* COMPARE_H */
