@@ -1069,6 +1069,31 @@ int test_markers_for_dates (double ***pZ, DATAINFO *pdinfo,
     return pd;
 }
 
+static int utf8_ok (FILE *fp, int pos)
+{
+    long mark = ftell(fp);
+    int len = pos + 9;
+    char *test = malloc(len + 1);
+    int i, ret = 0;
+
+    fseek(fp, mark - pos - 1, SEEK_SET);
+
+    for (i=0; i<len; i++) {
+	test[i] = fgetc(fp);
+    }
+    test[i] = '\0';
+
+    if (g_utf8_validate(test, -1, NULL)) {
+	ret = 1;
+    } 
+
+    free(test);
+
+    fseek(fp, mark, SEEK_SET);
+
+    return ret;
+}
+
 /* The function below checks for the maximum line length in the given
    file.  It also checks for extraneous binary data (the file is 
    supposed to be plain text), and checks whether the 'delim'
@@ -1083,6 +1108,7 @@ static int csv_max_line_length (FILE *fp, csvdata *cdata, PRN *prn)
 {
     int c, c1, cbak = 0, cc = 0;
     int comment = 0, maxlen = 0;
+    int lines = 0;
 
     csv_set_trailing_comma(cdata);
 
@@ -1109,13 +1135,15 @@ static int csv_max_line_length (FILE *fp, csvdata *cdata, PRN *prn)
 	    if (cbak != 0 && cbak != ',') {
 		csv_unset_trailing_comma(cdata);
 	    }
+	    lines++;
 	    continue;
 	}
 	cbak = c;
 	if (!isspace((unsigned char) c) && !isprint((unsigned char) c) &&
-	    !(c == CTRLZ)) {
-	    pprintf(prn, M_("Binary data (%d) encountered: this is not a valid "
-			   "text file\n"), c);
+	    !(c == CTRLZ) && !utf8_ok(fp, cc)) {
+	    pprintf(prn, M_("Binary data (%d) encountered (line %d:%d): "
+			    "this is not a valid text file\n"), 
+		    c, lines, cc);
 	    return -1;
 	}
 	if (cc == 0) {
@@ -1788,6 +1816,34 @@ static int fixed_format_read (csvdata *c, FILE *fp, PRN *prn)
     return err;
 }
 
+#define XML1_OK(u) ((u>=0x0020 && u<=0xD7FF) || \
+		    (u>=0xE000 && u<=0xFFFD))
+
+/* check that observation label contains only 
+   valid UTF-8, and moreover that every character
+   is valid in XML 1.0
+*/
+
+static int label_is_valid (gchar *s)
+{
+    if (!g_utf8_validate(s, -1, NULL)) {
+	return 0;
+    } else {
+	int i, n = g_utf8_strlen(s, -1);
+	gunichar u;
+
+	for (i=0; i<n; i++) {
+	    u = g_utf8_get_char(s);
+	    if (!XML1_OK(u)) {
+		return 0;
+	    }
+	    s = g_utf8_next_char(s);
+	}
+    }
+
+    return 1;
+}
+
 static int 
 real_read_labels_and_data (csvdata *c, FILE *fp, PRN *prn)
 {
@@ -1835,7 +1891,9 @@ real_read_labels_and_data (csvdata *c, FILE *fp, PRN *prn)
 		    S++;
 		}
 		strncat(c->dinfo->S[t], S, OBSLEN - 1);
-		iso_to_ascii(c->dinfo->S[t]);
+		if (!label_is_valid((gchar *) c->dinfo->S[t])) {
+		    iso_to_ascii(c->dinfo->S[t]);
+		} 
 	    } else {
 		nv = (csv_skip_column(c))? k : k + 1;
 		err = process_csv_obs(c, nv, t, prn);
