@@ -674,7 +674,7 @@ real_dateton (const char *date, const DATAINFO *pdinfo, int nolimit)
 #if DATES_DEBUG
 	fprintf(stderr, "dateton: treating as calendar data\n");
 #endif
-	if (pdinfo->markers && pdinfo->S != NULL) {
+	if (dataset_has_markers(pdinfo)) {
 	    /* "hard-wired" calendar dates as strings */
 	    for (t=0; t<pdinfo->n; t++) {
 		if (!strcmp(date, pdinfo->S[t])) {
@@ -716,7 +716,7 @@ real_dateton (const char *date, const DATAINFO *pdinfo, int nolimit)
 	    n = (t - pdinfo->sd0) / 10;
 	    handled = 1;
 	}	
-    } else if (pdinfo->markers && pdinfo->S != NULL) {
+    } else if (dataset_has_markers(pdinfo)) {
 	t = match_obs_marker(date, pdinfo);
 	if (t >= 0) {
 	    return t;
@@ -841,7 +841,7 @@ char *ntodate (char *datestr, int t, const DATAINFO *pdinfo)
 
     if (calendar_data(pdinfo)) {
 	/* handles both daily and dated weekly data */
-	if (pdinfo->markers && pdinfo->S != NULL) {
+	if (dataset_has_markers(pdinfo)) {
 	    strcpy(datestr, pdinfo->S[t]);
 	} else {
 	    calendar_date_string(datestr, t, pdinfo);
@@ -1345,7 +1345,7 @@ int write_data (const char *fname, int *list,
     if (fmt == GRETL_FMT_TRAD) { 
 	/* plain ASCII */
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	    if (pdinfo->markers && pdinfo->S != NULL) {
+	    if (dataset_has_markers(pdinfo)) {
 		fprintf(fp, "%s ", pdinfo->S[t]);
 	    }
 	    for (i=1; i<=l0; i++) {
@@ -1866,7 +1866,7 @@ int gretl_get_data (char *datfile, double ***pZ, DATAINFO *pdinfo,
     if (err) goto bailout;
 
     /* print out basic info from the files read */
-    pprintf(prn, I_("periodicity: %d, maxobs: %d,\n"
+    pprintf(prn, I_("periodicity: %d, maxobs: %d\n"
 	   "observations range: %s-%s\n"), tmpdinfo->pd, tmpdinfo->n,
 	   tmpdinfo->stobs, tmpdinfo->endobs);
 
@@ -1945,7 +1945,7 @@ int open_nulldata (double ***pZ, DATAINFO *pdinfo,
     }
 
     /* print out basic info */
-    pprintf(prn, M_("periodicity: %d, maxobs: %d,\n"
+    pprintf(prn, M_("periodicity: %d, maxobs: %d\n"
 	   "observations range: %s-%s\n"), pdinfo->pd, pdinfo->n,
 	   pdinfo->stobs, pdinfo->endobs);
 
@@ -2399,7 +2399,7 @@ int merge_or_replace_data (double ***pZ0, DATAINFO *pdinfo0,
     return err;
 }
 
-static int check_marker (char *src, int i)
+static int check_imported_string (char *src, int i, size_t len)
 {
     int err = 0;
 
@@ -2410,11 +2410,11 @@ static int check_marker (char *src, int i)
 	trstr = g_locale_to_utf8(src, -1, NULL, &bytes, NULL);
 
 	if (trstr == NULL) {
-	    gretl_errmsg_sprintf("Invalid characters in marker, line %d", i);
+	    gretl_errmsg_sprintf("Invalid characters in imported string, line %d", i);
 	    err = E_DATA;
 	} else {
 	    *src = '\0';
-	    strncat(src, trstr, OBSLEN - 1);
+	    strncat(src, trstr, len - 1);
 	    g_free(trstr);
 	}
     }
@@ -2465,7 +2465,7 @@ int add_obs_markers_from_file (DATAINFO *pdinfo, const char *fname)
 	} else {
 	    g_strstrip(marker);
 	    strncat(S[t], marker, OBSLEN - 1);
-	    err = check_marker(S[t], t+1);
+	    err = check_imported_string(S[t], t+1, OBSLEN);
 	}
     }
 
@@ -2482,6 +2482,157 @@ int add_obs_markers_from_file (DATAINFO *pdinfo, const char *fname)
     return err;
 }
 
+/**
+ * dataset_has_var_labels:
+ * @pdinfo: data information struct.
+ * 
+ * Returns: 1 if at least one variable in the current dataset
+ * has a descriptive label, otherwise 0.
+ */
+
+int dataset_has_var_labels (const DATAINFO *pdinfo)
+{
+    const char *label;
+    int i, imin = 1;
+
+    if (pdinfo->v > 1) {
+	if (!strcmp(pdinfo->varname[1], "index") &&
+	    !strcmp(VARLABEL(pdinfo, 1), _("index variable"))) {
+	    imin = 2;
+	}
+    }
+
+    for (i=imin; i<pdinfo->v; i++) {
+	label = VARLABEL(pdinfo, i);
+	if (*label != '\0') {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+/**
+ * save_var_labels_to_file:
+ * @pdinfo: data information struct.
+ * @fname: name of file containing labels.
+ * 
+ * Writes to @fname the descriptive labels for the series in
+ * the current dataset.
+ * 
+ * Returns: 0 on successful completion, non-zero otherwise.
+ */
+
+int save_var_labels_to_file (const DATAINFO *pdinfo, const char *fname)
+{
+    FILE *fp;
+    int i, err = 0;
+
+    fp = gretl_fopen(fname, "w");
+
+    if (fp == NULL) {
+	err = E_FOPEN;
+    } else {
+	for (i=1; i<pdinfo->v; i++) {
+	    fprintf(fp, "%s\n", VARLABEL(pdinfo, i));
+	}
+	fclose(fp);
+    }
+
+    return err;
+}
+
+/**
+ * add_var_labels_from_file:
+ * @pdinfo: data information struct.
+ * @fname: name of file containing labels.
+ * 
+ * Read descriptive variables for labels (strings of %MAXLABEL - 1 
+ * characters or less) from a file, and associate them with the 
+ * current data set.  The file should contain one label per line,
+ * with a number of lines equal to the number of variables in
+ * the current data set, excluding the constant.
+ * 
+ * Returns: 0 on successful completion, non-zero otherwise.
+ */
+
+int add_var_labels_from_file (DATAINFO *pdinfo, const char *fname)
+{
+    FILE *fp;
+    char line[256], label[MAXLABEL];
+    char *targ;
+    int nlabels = 0;
+    int i, err = 0;
+
+    fp = gretl_fopen(fname, "r");
+    if (fp == NULL) {
+	return E_FOPEN;
+    }
+
+    for (i=1; i<pdinfo->v && !err; i++) {
+	if (fgets(line, sizeof line, fp) == NULL) {
+	    break;
+	} else if (sscanf(line, "%127[^\n\r]", label) != 1) {
+	    continue;
+	} else {
+	    targ = VARLABEL(pdinfo, i);
+	    g_strstrip(label);
+	    *targ = '\0';
+	    strncat(targ, label, MAXLABEL - 1);
+	    err = check_imported_string(targ, i+1, MAXLABEL);
+	    if (err) {
+		*targ = '\0';
+	    } else {
+		nlabels++;
+	    }
+	}
+    }
+
+    if (!err && nlabels == 0) {
+	gretl_errmsg_set("No labels found");
+	err = E_DATA;
+    }
+
+    return err;
+}
+
+int read_or_write_var_labels (gretlopt opt, DATAINFO *pdinfo, PRN *prn)
+{
+    const char *fname;
+    int err;
+
+    err = incompatible_options(opt, OPT_T | OPT_F); 
+    if (err) {
+	return err;
+    }
+
+    fname = get_optval_string(LABELS, opt);
+    if (fname == NULL) {
+	return E_BADOPT;
+    }
+
+    if (opt & OPT_T) {
+	/* to-file */
+	if (!dataset_has_var_labels(pdinfo)) {
+	    pprintf(prn, "No labels are available for writing\n");
+	    err = E_DATA;
+	} else {
+	    err = save_var_labels_to_file(pdinfo, fname);
+	    if (!err && gretl_messages_on() && !gretl_looping_quietly()) {
+		pprintf(prn, "Labels written OK\n");
+	    }
+	}
+    } else if (opt & OPT_F) {
+	/* from-file */
+	err = add_var_labels_from_file(pdinfo, fname);
+	if (!err && gretl_messages_on() && !gretl_looping_quietly()) {
+	    pprintf(prn, "Labels loaded OK\n");
+	}	
+    }
+
+    return err;
+}
+
 static void 
 octave_varname (char *name, const char *s, int nnum, int v)
 {
