@@ -332,8 +332,8 @@ int dataset_allocate_panel_info (DATAINFO *pdinfo)
 static int resize_panel_info (DATAINFO *pdinfo, int n)
 {
     PANINFO *pan = pdinfo->paninfo;
-    int *unit;
-    int *period;
+    int *unit, *period;
+    int err = 0;
 
     unit = realloc(pan->unit, n * sizeof *unit);
     if (unit == NULL) {
@@ -352,11 +352,16 @@ static int resize_panel_info (DATAINFO *pdinfo, int n)
 	int i, t, j = pdinfo->paninfo->nunits;
 	int s = pdinfo->n;
 
-	for (i=0; i<uadd; i++) {
-	    for (t=0; t<pdinfo->pd; t++) {
-		pan->unit[s + i] = j;
-		pan->period[s + i] = t;
-		s++;
+	for (i=0; i<uadd && !err; i++) {
+	    for (t=0; t<pdinfo->pd && !err; t++) {
+		if (s+i >= n) {
+		    err = E_DATA;
+		    fprintf(stderr, "panel info out of bounds\n");
+		} else {
+		    pan->unit[s+i] = j;
+		    pan->period[s+i] = t;
+		    s++;
+		}
 	    }
 	    j++;
 	}
@@ -364,7 +369,7 @@ static int resize_panel_info (DATAINFO *pdinfo, int n)
 
     pdinfo->paninfo->nunits = n / pdinfo->pd;
 
-    return 0;
+    return err;
 }
 
 /**
@@ -1037,6 +1042,7 @@ int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo,
     double *x;
     int oldn = pdinfo->n;
     int i, t, bign;
+    int err = 0;
 
     if (newobs <= 0) {
 	return 0;
@@ -1059,7 +1065,7 @@ int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo,
 	}	    
     }
     
-    if (pdinfo->markers && pdinfo->S != NULL) {
+    if (dataset_has_markers(pdinfo)) {
 	if (opt & OPT_D) {
 	    dataset_destroy_obs_markers(pdinfo);
 	} else {
@@ -1073,8 +1079,9 @@ int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (pdinfo->paninfo != NULL) {
-	if (resize_panel_info(pdinfo, bign)) {
-	    return E_ALLOC;
+	err = resize_panel_info(pdinfo, bign);
+	if (err) {
+	    return err;
 	}
     }
     
@@ -1092,7 +1099,7 @@ int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo,
     /* does daily data need special handling? */
     ntodate(pdinfo->endobs, bign - 1, pdinfo);
 
-    return 0;
+    return err;
 }
 
 /**
@@ -1130,7 +1137,7 @@ int dataset_drop_observations (int n, double ***pZ, DATAINFO *pdinfo)
 	(*pZ)[i] = x;
     }
     
-    if (pdinfo->markers && pdinfo->S != NULL) {
+    if (dataset_has_markers(pdinfo)) {
 	if (reallocate_markers(pdinfo, newn)) {
 	    return E_ALLOC;
 	}
@@ -1181,7 +1188,7 @@ int dataset_shrink_obs_range (double ***pZ, DATAINFO *pdinfo)
 	    memmove(Z[i], Z[i] + head, mvsize);
 	}
 
-	if (pdinfo->markers && pdinfo->S != NULL) {
+	if (dataset_has_markers(pdinfo)) {
 	    for (i=0; i<head; i++) {
 		free(pdinfo->S[i]);
 	    }
@@ -1544,7 +1551,7 @@ int series_is_parent (const DATAINFO *pdinfo, int v)
  * @v: ID number of the variable to be renamed.
  * @name: new name to give the variable.
  * 
- * Returns: 0 on sucess, non-zero on error.
+ * Returns: 0 on success, non-zero on error.
  */
 
 int dataset_rename_series (DATAINFO *pdinfo, int v, const char *name)
@@ -1566,6 +1573,53 @@ int dataset_rename_series (DATAINFO *pdinfo, int v, const char *name)
     return 0;
 }
 
+/**
+ * dataset_replace_series:
+ * @Z: data array.
+ * @pdinfo: dataset information.
+ * @v: ID number of the series to be replaced.
+ * @x: replacement values.
+ * @descrip: replacement description.
+ *
+ * Replaces the description and numerical content of
+ * series @v with the information provided. 
+ * 
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int dataset_replace_series (double **Z, DATAINFO *pdinfo, int v,
+			    double *x, const char *descrip,
+			    DataCopyFlag flag)
+{
+    if (v < 0 || v >= pdinfo->v) {
+	/* out of bounds */
+	return E_DATA;
+    }
+
+    if (object_is_const(pdinfo->varname[v]) ||
+	series_is_parent(pdinfo, v)) {
+	return overwrite_err(pdinfo->varname[v]);
+    }
+
+    gretl_varinfo_init(pdinfo->varinfo[v]);
+    strcpy(VARLABEL(pdinfo, v), descrip);
+
+    if (flag == DS_GRAB_VALUES) {
+	free(Z[v]);
+	Z[v] = x;
+    } else {
+	int t;
+
+	for (t=0; t<pdinfo->n; t++) {
+	    Z[v][t] = x[t];
+	}
+    }
+
+    set_dataset_is_changed();
+
+    return 0;
+}
+
 static int real_drop_listed_vars (int *list, double ***pZ, 
 				  DATAINFO *pdinfo, int *renumber,
 				  int drop, PRN *prn)
@@ -1968,45 +2022,88 @@ typedef struct spoint_t_ spoint_t;
 
 struct spoint_t_ {
     int obsnum;
-    double val;
+    int nvals;
+    double *vals;
 };
 
+static void free_spoints (spoint_t *sv, int n, int v)
+{
+    int i;
+
+    for (i=0; i<n; i++) {
+	free(sv[i].vals);
+    }
+
+    free(sv);
+}
+
+static spoint_t *allocate_spoints (int n, int v)
+{
+    spoint_t *sv;
+    int i;
+
+    sv = malloc(n * sizeof *sv);
+
+    if (sv != NULL) {
+	for (i=0; i<n; i++) {
+	    sv[i].vals = NULL;
+	}
+	for (i=0; i<n; i++) {
+	    sv[i].vals = malloc(v * sizeof(double));
+	    if (sv[i].vals == NULL) {
+		free_spoints(sv, n, v);
+		sv = NULL;
+		break;
+	    }
+	}
+    }
+
+    return sv;
+}
+
 static int compare_vals_up (const void *a, const void *b)
 {
     const spoint_t *pa = (const spoint_t *) a;
     const spoint_t *pb = (const spoint_t *) b;
-     
-    return (pa->val > pb->val) - (pa->val < pb->val);
+    int i, ret = 0;
+
+    for (i=0; i<pa->nvals && !ret; i++) {
+	ret = (pa->vals[i] > pb->vals[i]) - (pa->vals[i] < pb->vals[i]);
+    }
+
+    return ret;
 }
 
 static int compare_vals_down (const void *a, const void *b)
 {
     const spoint_t *pa = (const spoint_t *) a;
     const spoint_t *pb = (const spoint_t *) b;
-     
-    return (pa->val < pb->val) - (pa->val > pb->val);
+    int i, ret = 0;
+
+    for (i=0; i<pa->nvals && !ret; i++) {
+	ret = (pa->vals[i] < pb->vals[i]) - (pa->vals[i] > pb->vals[i]);
+    }
+
+    return ret;
 }
 
-int dataset_sort_by (int v, double **Z, DATAINFO *pdinfo, gretlopt opt)
+int dataset_sort_by (const int *list, double **Z, DATAINFO *pdinfo, gretlopt opt)
 {
     spoint_t *sv = NULL;
     double *x = NULL;
     char **S = NULL;
-    int i, t;
+    int ns = list[1];
+    int i, t, v;
     int err = 0;
 
-    if (v < 1 || v >= pdinfo->v) {
-	return E_DATA;
-    }
-
-    sv = malloc(pdinfo->n * sizeof *sv);
+    sv = allocate_spoints(pdinfo->n, ns);
     if (sv == NULL) {
 	return E_ALLOC;
     }
 
     x = malloc(pdinfo->n * sizeof *x);
     if (x == NULL) {
-	free(sv);
+	free_spoints(sv, pdinfo->n, ns);
 	return E_ALLOC;
     }    
 
@@ -2020,7 +2117,11 @@ int dataset_sort_by (int v, double **Z, DATAINFO *pdinfo, gretlopt opt)
 
     for (t=0; t<pdinfo->n; t++) {
 	sv[t].obsnum = t;
-	sv[t].val = Z[v][t];
+	for (i=0; i<ns; i++) {
+	    v = list[i+1];
+	    sv[t].vals[i] = Z[v][t];
+	    sv[t].nvals = ns;
+	}
     }
 
     if (opt & OPT_D) {
@@ -2049,43 +2150,26 @@ int dataset_sort_by (int v, double **Z, DATAINFO *pdinfo, gretlopt opt)
 
  bailout:
 
-    free(sv);
+    free_spoints(sv, pdinfo->n, ns);
     free(x);
 
     return err;
 }
 
-static int dataset_sort (const char *s, double **Z, DATAINFO *pdinfo,
+static int dataset_sort (const int *list, double **Z, DATAINFO *pdinfo,
 			 gretlopt opt)
 {
-    char fmt[10];
-    char vname[VNAMELEN];
-    int v;
-
     if (dataset_is_time_series(pdinfo) ||
 	dataset_is_panel(pdinfo)) {
 	gretl_errmsg_set("You can only do this with undated data");
 	return E_DATA;
     }
 
-    s += strspn(s, " \t");
-
-    sprintf(fmt, "%%%ds", VNAMELEN - 1);
-
-    if (sscanf(s, fmt, vname) != 1) {
+    if (list == NULL || list[0] < 1) {
 	return E_DATA;
     }
 
-    v = series_index(pdinfo, vname);
-    if (v == pdinfo->v) {
-	return E_UNKVAR;
-    }
-
-    if (v < 1) {
-	return E_DATA;
-    }
-    
-    return dataset_sort_by(v, Z, pdinfo, opt);
+    return dataset_sort_by(list, Z, pdinfo, opt);
 }
 
 /**
@@ -2329,8 +2413,10 @@ int dataset_stack_variables (const char *vname, const char *line,
     char format[16];
     char *p, *s = NULL, *scpy = NULL;
     int *vnum = NULL;
+    int *stacklist = NULL;
     double *bigx = NULL;
     int i, v1 = 0, v2 = 0, nv = 0;
+    int done = 0;
     int maxok, offset;
     int oldn, bign, genv;
     int err = 0;
@@ -2365,23 +2451,47 @@ int dataset_stack_variables (const char *vname, const char *line,
 
     genv = series_index(pdinfo, vname);
 
-    /* do we have a range of vars? */
-    sprintf(format, "%%%d[^.]..%%%ds", VNAMELEN-1, VNAMELEN-1);
-    if (sscanf(s, format, vn1, vn2) == 2) {
-	if (isdigit(*vn1) && isdigit(*vn2)) {
-	    v1 = atoi(vn1);
-	    v2 = atoi(vn2);
-	} else {
-	    v1 = series_index(pdinfo, vn1);
-	    v2 = series_index(pdinfo, vn2);
-	}
-	if (v1 >= 0 && v2 > v1 && v2 < pdinfo->v) {
-	    nv = v2 - v1 + 1;
-	} else {
-	    fputs("stack vars: range is invalid\n", stderr);
+    /* do we have a named list? */
+    stacklist = get_list_by_name(s);
+    if (stacklist != NULL) {
+	nv = stacklist[0];
+	if (nv == 0) {
 	    err = E_DATA;
+	    goto bailout;
 	}
-    } else {
+	vnum = malloc(nv * sizeof *vnum);
+	if (vnum == NULL) {
+	    err = E_ALLOC;
+	    goto bailout;
+	}
+	for (i=0; i<nv; i++) {
+	    vnum[i] = stacklist[i+1];
+	}
+	done = 1;
+    }
+
+    if (!done) {
+	/* do we have a range of vars? */
+	sprintf(format, "%%%d[^.]..%%%ds", VNAMELEN-1, VNAMELEN-1);
+	if (sscanf(s, format, vn1, vn2) == 2) {
+	    if (isdigit(*vn1) && isdigit(*vn2)) {
+		v1 = atoi(vn1);
+		v2 = atoi(vn2);
+	    } else {
+		v1 = series_index(pdinfo, vn1);
+		v2 = series_index(pdinfo, vn2);
+	    }
+	    if (v1 >= 0 && v2 > v1 && v2 < pdinfo->v) {
+		nv = v2 - v1 + 1;
+	    } else {
+		fputs("stack vars: range is invalid\n", stderr);
+		err = E_DATA;
+	    }
+	    done = 1;
+	}
+    }
+
+    if (!done) {
 	/* or a comma-separated list of vars? */
 	char *p = s;
 
@@ -2416,30 +2526,27 @@ int dataset_stack_variables (const char *vname, const char *line,
 	}
     }
 
-    if (err) {
-	goto bailout;
+    if (!err) {
+	/* get offset specified by user? */
+	offset = get_optional_offset(scpy, (const double **) *pZ, 
+				     pdinfo, &err);
     }
 
-    /* get offset specified by user? */
-    offset = get_optional_offset(scpy, (const double **) *pZ, 
-				 pdinfo, &err);
-    if (err) {
-	goto bailout;
-    }
-
-    /* get length specified by user? */
-    maxok = get_optional_length(scpy, (const double **) *pZ, 
-				pdinfo, &err);
-    if (err) {
-	goto bailout;
+    if (!err) {
+	/* get length specified by user? */
+	maxok = get_optional_length(scpy, (const double **) *pZ, 
+				    pdinfo, &err);
     }
 
 #if PDEBUG
     fprintf(stderr, "offset = %d, maxok = %d\n", offset, maxok);
 #endif
 
-    if (offset + maxok > pdinfo->n) {
+    if (!err && offset + maxok > pdinfo->n) {
 	err = E_DATA;
+    }
+
+    if (err) {
 	goto bailout;
     }
 
@@ -2452,11 +2559,8 @@ int dataset_stack_variables (const char *vname, const char *line,
 	/* calculate required series length */	
 	maxok = 0;
 	for (i=0; i<nv; i++) {
-	    int j, ok;
-
-	    j = (vnum == NULL)? i + v1 : vnum[i];
-
-	    ok = pdinfo->n - missing_tail((*pZ)[j], pdinfo->n);
+	    int j = (vnum == NULL)? i + v1 : vnum[i];
+	    int ok = pdinfo->n - missing_tail((*pZ)[j], pdinfo->n);
 
 	    if (ok > maxok) {
 		maxok = ok;
@@ -2497,9 +2601,8 @@ int dataset_stack_variables (const char *vname, const char *line,
 
     /* construct stacked series */
     for (i=0; i<nv; i++) {
-	int j, t, bigt, tmax;
-
-	j = (vnum == NULL)? i + v1 : vnum[i];
+	int j = (vnum == NULL)? i + v1 : vnum[i];
+	int t, bigt, tmax;
 
 	if (maxok > 0) {
 	    bigt = maxok * i;
@@ -2983,6 +3086,20 @@ int dataset_resample (int n, unsigned int seed,
     return err;
 }
 
+/* alternate forms:
+
+           op         list   s
+   dataset addobs            24
+   dataset compact           1
+   dataset compact           4 last
+   dataset expand            12
+   dataset transpose
+   dataset sortby     x1
+   dataset resample          500
+   dataset clear   
+
+*/
+
 int modify_dataset (int op, const int *list, const char *s, 
 		    double ***pZ, DATAINFO *pdinfo, 
 		    PRN *prn)
@@ -2994,6 +3111,11 @@ int modify_dataset (int op, const int *list, const char *s,
 	return E_NODATA;
     }
 
+#if 0
+    fprintf(stderr, "modify_dataset: op=%d, s='%s'\n", op, s);
+    printlist(list, "list");
+#endif
+
     if (op == DS_CLEAR) {
 	/* must be handled by the calling program */
 	return E_NOTIMP;
@@ -3041,9 +3163,9 @@ int modify_dataset (int op, const int *list, const char *s,
     } else if (op == DS_TRANSPOSE) {
 	err = transpose_data(pZ, pdinfo);
     } else if (op == DS_SORTBY) {
-	err = dataset_sort(s, *pZ, pdinfo, OPT_NONE);
+	err = dataset_sort(list, *pZ, pdinfo, OPT_NONE);
     } else if (op == DS_DSORTBY) {
-	err = dataset_sort(s, *pZ, pdinfo, OPT_D);
+	err = dataset_sort(list, *pZ, pdinfo, OPT_D);
     } else if (op == DS_RESAMPLE) {
 	err = dataset_resample(k, 0, pZ, pdinfo);
 	if (!err) {
