@@ -55,6 +55,11 @@ typedef enum {
     DS_CLEAR
 } DatasetOp;
 
+typedef enum {
+    DS_COPY_VALUES,
+    DS_GRAB_VALUES
+} DataCopyFlag;
+
 /**
  * dataset_is_cross_section:
  * @p: pointer to data information struct.
@@ -197,6 +202,15 @@ typedef enum {
  */
 #define dataset_is_panel(p) (p != NULL && p->structure == STACKED_TIME_SERIES)
 
+/**
+ * dataset_has_markers:
+ * @p: pointer to data information struct.
+ *
+ * Determine whether a data set has observation marker strings (1)
+ * or not (0).
+ */
+#define dataset_has_markers(p) (p != NULL && p->markers && p->S != NULL)
+
 /**
  * var_is_discrete:
  * @p: pointer to data information struct.
@@ -337,6 +351,10 @@ int overwrite_err (const char *name);
 
 int series_is_parent (const DATAINFO *pdinfo, int v);
 
+int dataset_replace_series (double **Z, DATAINFO *pdinfo, int v,
+			    double *x, const char *descrip,
+			    DataCopyFlag flag);
+
 int dataset_rename_series (DATAINFO *pdinfo, int v, const char *name);
 
 int dataset_drop_listed_variables (int *list, double ***pZ, 
@@ -356,7 +374,7 @@ int dataset_stack_variables (const char *vname, const char *line,
 			     double ***pZ, DATAINFO *pdinfo, 
 			     PRN *prn);
 
-int dataset_sort_by (int v, double **Z, DATAINFO *pdinfo, gretlopt opt);
+int dataset_sort_by (const int *list, double **Z, DATAINFO *pdinfo, gretlopt opt);
 
 int is_log_variable (int i, const DATAINFO *pdinfo, char *parent);
 
