@@ -26,6 +26,7 @@ typedef struct Xtab_ Xtab;
 typedef struct MahalDist_ MahalDist;
 
 struct Summary_ {
+    gretlopt opt;
     int n;
     int missing;
     int *list;
@@ -88,7 +89,8 @@ double gretl_restricted_mean (int t1, int t2, const double *x,
 			      const double *y, GretlOp yop, 
 			      double yval);
 
-double gretl_quantile (int t1, int t2, const double *x, double p);
+double gretl_quantile (int t1, int t2, const double *x, 
+		       double p, int *err);
 
 int gretl_array_quantiles (double *a, int n, double *p, int k);
 
@@ -124,6 +126,11 @@ int gretl_moments (int t1, int t2, const double *x,
 		   double *xbar, double *sd, 
 		   double *skew, double *kurt, int k);
 
+double *gretl_sorted_series (int v, const double **Z, 
+			     const DATAINFO *pdinfo,
+			     gretlopt opt, int *n, 
+			     int *err);
+
 void free_freq (FreqDist *freq);
 
 int freq_setup (int v, const double **Z, const DATAINFO *pdinfo,
@@ -145,6 +152,8 @@ int crosstab (const int *list, const double **Z,
 	      const DATAINFO *pdinfo, gretlopt opt,
 	      PRN *prn);
 
+int crosstab_from_matrix (gretlopt opt, PRN *prn);
+
 int compare_xtab_rows (const void *a, const void *b);
 
 Xtab *single_crosstab (const int *list, const double **Z, 
@@ -175,12 +184,18 @@ int periodogram (int varno, int width,
 		 const double **Z, const DATAINFO *pdinfo, 
 		 gretlopt opt, PRN *prn);
 
+gretl_matrix *periodogram_func (const double *x, 
+				const DATAINFO *pdinfo,
+				int width, int *err);
+
 Summary *get_summary (const int *list, const double **Z, 
 		      const DATAINFO *pdinfo,
-		      PRN *prn, int *err);
+		      gretlopt opt, PRN *prn, 
+		      int *err);
 
 int list_summary (const int *list, const double **Z, 
-		  const DATAINFO *pdinfo, PRN *prn);
+		  const DATAINFO *pdinfo, 
+		  gretlopt opt, PRN *prn);
 
 void print_summary (const Summary *summ,
 		    const DATAINFO *pdinfo,
