@@ -798,8 +798,8 @@ static int do_ordered (int ci, int ndum,
 
     err = BFGS_max(theta, npar, maxit, toler, 
 		   &fncount, &grcount, op_loglik, C_LOGLIK,
-		   op_score, OC, (prn != NULL)? OPT_V : OPT_NONE,
-		   prn);
+		   op_score, OC, NULL, 
+		   (prn != NULL)? OPT_V : OPT_NONE, prn);
 
     if (err) {
 	goto bailout;
@@ -1143,6 +1143,14 @@ static void Lr_chisq (MODEL *pmod, const double **Z)
 {
     int t, zeros, ones = 0, m = pmod->nobs;
     double Lr, chisq;
+
+    if (pmod->ncoeff == 1 && pmod->ifc) {
+	/* constant-only model */
+	pmod->chisq = NADBL;
+	pmod->rsq = 0;
+	pmod->adjrsq = NADBL;
+	return;
+    }
     
     for (t=pmod->t1; t<=pmod->t2; t++) {
 	if (!model_missing(pmod, t)) {
@@ -1154,18 +1162,20 @@ static void Lr_chisq (MODEL *pmod, const double **Z)
 
     zeros = m - ones;
 
-    Lr = (double) ones * log((double) ones / (double) m);
-    Lr += (double) zeros * log((double) zeros /(double) m);
+    Lr = ones * log(ones / (double) m);
+    Lr += zeros * log(zeros /(double) m);
 
     chisq = 2.0 * (pmod->lnL - Lr);
 
     if (chisq < 0) {
 	pmod->rsq = pmod->adjrsq = pmod->chisq = NADBL;
     } else {
+	int K = pmod->ncoeff;
+
 	pmod->chisq = chisq;
 	/* McFadden pseudo-R^2 */
 	pmod->rsq = 1.0 - pmod->lnL / Lr;
-	pmod->adjrsq = 1.0 - (pmod->lnL - pmod->ncoeff) / Lr;
+	pmod->adjrsq = 1.0 - (pmod->lnL - K) / Lr;
     }
 }
 
@@ -2610,7 +2620,7 @@ static MODEL mnl_model (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     mod.errcode = BFGS_max(mnl->theta, mnl->npar, maxit, 0.0, 
 			   &fncount, &grcount, mn_logit_loglik, C_LOGLIK,
-			   mn_logit_score, mnl, 
+			   mn_logit_score, mnl, NULL,
 			   (prn != NULL)? OPT_V : OPT_NONE, prn);
 
     if (!mod.errcode) {
