@@ -708,6 +708,140 @@ int filter_series (const double *x, double *y, const DATAINFO *pdinfo,
     return err;
 }
 
+/**
+ * exponential_movavg_series:
+ * @x: array of original data.
+ * @y: array into which to write the result.
+ * @pdinfo: data set information.
+ * @d: coefficient on lagged @x.
+ * @n: number of @x observations to average to give the
+ * initial @y value.
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+int exponential_movavg_series (const double *x, double *y, 
+			       const DATAINFO *pdinfo,
+			       double d, int n)
+{
+    int t1 = pdinfo->t1;
+    int t2 = pdinfo->t2;
+    int t, T;
+
+    if (n < 0) {
+	return E_INVARG;
+    } else if (array_adjust_t1t2(x, &t1, &t2) != 0) {
+	return E_MISSDATA;
+    }
+
+    T = t2 - t1 + 1;
+    if (T < n) {
+	return E_TOOFEW;
+    }
+
+    if (n == 0) {
+	/* signal to use full sample mean */
+	n = T;
+    }
+    
+    if (n == 1) {
+	/* initialize on first observation */
+	y[t1] = x[t1];
+    } else {
+	/* initialize on mean of first n obs */
+	y[t1] = 0.0;
+	for (t=t1; t<t1+n; t++) {
+	    y[t1] += x[t];
+	}
+	y[t1] /= n;
+    }
+
+    for (t=t1+1; t<=t2; t++) {
+	y[t] = d * x[t] + (1-d) * y[t-1];
+    }
+
+    return 0;
+}
+
+/**
+ * movavg_series:
+ * @x: array of original data.
+ * @y: array into which to write the result.
+ * @pdinfo: data set information.
+ * @k: number of terms in MA.
+ * @center: if non-zero, produce centered MA.
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+int movavg_series (const double *x, double *y, const DATAINFO *pdinfo,
+		   int k, int center)
+{
+    int t1 = pdinfo->t1;
+    int t2 = pdinfo->t2;
+    int k1 = k-1, k2 = 0;
+    int i, s, t, T;
+
+    array_adjust_t1t2(x, &t1, &t2);
+    T = t2 - t1 + 1;
+    if (T < k) {
+	return E_TOOFEW;
+    }
+
+    if (center) {
+	k1 = k / 2;
+	k2 = (k % 2 == 0)? (k1 - 1) : k1;
+    }
+
+    t1 += k1;
+    t2 -= k2;
+
+    for (t=t1; t<=t2; t++) {
+	double xs, msum = 0.0;
+
+	for (i=-k1; i<=k2; i++) {
+	    s = t + i;
+	    if (pdinfo->structure == STACKED_TIME_SERIES) {
+		if (pdinfo->paninfo->unit[s] != 
+		    pdinfo->paninfo->unit[t]) {
+		    s = -1;
+		}
+	    }
+
+	    if (s >= 0) {
+		xs = x[s];
+	    } else {
+		xs = NADBL;
+	    }
+
+	    if (na(xs)) {
+		msum = NADBL;
+		break;
+	    } else {
+		msum += x[s];
+	    }
+	}
+
+	if (!na(msum)) {
+	    y[t] = (k > 0)? (msum / k) : msum;
+	}
+    }
+
+    if (center && k % 2 == 0) {
+	/* centered, with even number of terms */
+	for (t=t1; t<t2; t++) {
+	    if (na(y[t]) || na(y[t+1])) {
+		y[t] = NADBL;
+	    } else {
+		y[t] = (y[t] + y[t+1]) / 2.0;
+	    }
+	}
+	y[t2] = NADBL;
+    }
+
+    return 0;
+}
+
 int seasonally_adjust_series (const double *x, double *y, 
 			      DATAINFO *pdinfo, int tramo)
 {
@@ -938,23 +1072,24 @@ static double hp_lambda (const DATAINFO *pdinfo)
  * @x: array of original data.
  * @hp: array in which filtered series is computed.
  * @pdinfo: data set information.
+ * @lambda: smoothing parameter (or #NADBL to use the default
+ * value).
  * @opt: if %OPT_T, return the trend rather than the cycle.
  *
  * Calculates the "cycle" component of the time series in
  * array @x, using the Hodrick-Prescott filter.  Adapted from the 
- * original FORTRAN code by E. Prescott. Very few changes.
+ * original FORTRAN code by E. Prescott. 
  *
  * Returns: 0 on success, non-zero error code on failure.
  */
 
 int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo,
-	       gretlopt opt)
+	       double lambda, gretlopt opt)
 {
     int t1 = pdinfo->t1, t2 = pdinfo->t2;
     double v00 = 1.0, v11 = 1.0, v01 = 0.0;
     double det, tmp0, tmp1;
     double e0, e1, b00, b01, b11;
-    double lambda;
     double **V = NULL;
     double m[2], tmp[2];
     int i, t, T, tb;
@@ -976,7 +1111,9 @@ int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo,
 	goto bailout;
     }
 
-    lambda = hp_lambda(pdinfo);
+    if (na(lambda)) {
+	lambda = hp_lambda(pdinfo);
+    }
 
     V = doubles_array_new(4, T);
     if (V == NULL) {
@@ -1008,7 +1145,6 @@ int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo,
 	v00 -= v00 * v00 / tmp0;
 	v11 -= v01 * v01 / tmp0;
 	v01 -= (tmp1 / tmp0) * v01;
-
     }
 
     m[0] = x[0];
@@ -1096,35 +1232,35 @@ int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo,
 
 /**
  * bkbp_filter:
- * @y: array of original data.
+ * @x: array of original data.
  * @bk: array into which to write the filtered series.
  * @pdinfo: data set information.
+ * @bkl: lower frequency bound (or 0 for automatic).
+ * @bku: upper frequency bound (or 0 for automatic).
+ * @k: approximation order (or 0 for automatic).
  *
- * Calculates the Baxter & King bandpass filter.
+ * Calculates the Baxter and King bandpass filter.
  *
  * Returns: 0 on success, non-zero error code on failure.
  */
 
-int bkbp_filter (const double *y, double *bk, const DATAINFO *pdinfo)
+int bkbp_filter (const double *x, double *bk, const DATAINFO *pdinfo,
+		 int bkl, int bku, int k)
 {
     int t1 = pdinfo->t1, t2 = pdinfo->t2;
-    int bkl, bku;
-
     double omubar, omlbar;
     double avg_a;
     double *a;
+    int i, t, err = 0;
 
-    int i, k, t;
-    int err = 0;
+    if (bkl <= 0 || bku <= 0) {
+	/* get user settings if available (or the defaults) */
+	get_bkbp_periods(pdinfo, &bkl, &bku);
+    }
 
-    /*
-      periods[0] and periods[1]: threshold periodicities for business cycle
-      k: order of the approximation
-    */
-
-    /* get user settings if available (or the defaults) */
-    get_bkbp_periods(pdinfo, &bkl, &bku);
-    k = get_bkbp_k(pdinfo);
+    if (k <= 0) {
+	k = get_bkbp_k(pdinfo);
+    }
 
 #if BK_DEBUG
     fprintf(stderr, "lower limit = %d, upper limit = %d, \n", 
@@ -1136,7 +1272,7 @@ int bkbp_filter (const double *y, double *bk, const DATAINFO *pdinfo)
 	return 1;
     }
 
-    err = array_adjust_t1t2(y, &t1, &t2);
+    err = array_adjust_t1t2(x, &t1, &t2);
     if (err) {
 	return err;
     } 
@@ -1179,9 +1315,9 @@ int bkbp_filter (const double *y, double *bk, const DATAINFO *pdinfo)
 	if (t < t1 + k || t > t2 - k) {
 	    bk[t] = NADBL;
 	} else {
-	    bk[t] = a[0] * y[t];
+	    bk[t] = a[0] * x[t];
 	    for (i=1; i<=k; i++) {
-		bk[t] += a[i] * (y[t-i] + y[t+i]);
+		bk[t] += a[i] * (x[t-i] + x[t+i]);
 	    }
 	}
     }
@@ -1521,8 +1657,7 @@ int panel_unit_first_obs (int t, const DATAINFO *pdinfo)
 
 /* make special time variable for panel data */
 
-static void 
-make_panel_time_var (double *x, const DATAINFO *pdinfo)
+static void make_panel_time_var (double *x, const DATAINFO *pdinfo)
 {
     int t, xt = 0;
 
@@ -1773,13 +1908,101 @@ const double *gretl_plotx (const DATAINFO *pdinfo)
 }
 
 /**
- * genr_fit_resid:
- * @pmod: pointer to model to be tested.
- * @pZ: pointer to data matrix.
+ * get_fit_or_resid:
+ * @pmod: pointer to source model.
  * @pdinfo: information on the data set.
- * @code: GENR_RESID or GENR_FITTED or GENR_RESID2.
- * @undo: if non-zero, don't bother labeling the variables
+ * @code: M_UHAT, M_UHAT2, M_YHAT, M_AHAT or M_H.
+ * @vname: location to write series name (length #VNAMELEN)
+ * @vlabel: location to write series description (length should
+ * be #MAXLABEL).
+ * @err: location to receive error code.
  *
+ * Creates a full-length array holding the specified model
+ * data, and writes name and description into the @vname and
+ * @vlabel.
+ * 
+ * Returns: allocated array on success or %NULL on failure.
+ */
+
+double *get_fit_or_resid (const MODEL *pmod, DATAINFO *pdinfo, int code,
+			  char *vname, char *vlabel, int *err)
+{
+    const double *src = NULL;
+    double *ret = NULL;
+    int t;
+
+    if (code == M_H) {
+	src = gretl_model_get_data(pmod, "garch_h");
+    } else if (code == M_AHAT) {
+	src = gretl_model_get_data(pmod, "ahat");
+    } else if (code == M_UHAT || code == M_UHAT2) {
+	src = pmod->uhat;
+    } else if (code == M_YHAT) {
+	src = pmod->yhat;
+    }
+
+    if (src == NULL) {
+	*err = E_BADSTAT;
+	return NULL;
+    }
+
+    ret = malloc(pdinfo->n * sizeof *ret);
+    if (ret == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    for (t=0; t<pdinfo->n; t++) {
+	if (t >= pmod->t1 && t <= pmod->t2) {
+	    if (code == M_UHAT2) {
+		ret[t] = na(src[t]) ? NADBL : (src[t] * src[t]);
+	    } else {
+		ret[t] = src[t];
+	    }
+	} else {
+	    ret[t] = NADBL;
+	}
+    }
+
+    if (code == M_UHAT) {
+	sprintf(vname, "uhat%d", pmod->ID);
+	if (pmod->ci == GARCH && (pmod->opt & OPT_Z)) {
+	    sprintf(vlabel, _("standardized residual from model %d"), pmod->ID);
+	} else {
+	    sprintf(vlabel, _("residual from model %d"), pmod->ID);
+	}
+    } else if (code == M_YHAT) {
+	sprintf(vname, "yhat%d", pmod->ID);
+	sprintf(vlabel, _("fitted value from model %d"), pmod->ID);
+    } else if (code == M_UHAT2) { 
+	/* squared residuals */
+	sprintf(vname, "usq%d", pmod->ID);
+	if (pmod->ci == GARCH && (pmod->opt & OPT_Z)) {
+	    sprintf(vlabel, _("squared standardized residual from model %d"), pmod->ID);
+	} else {
+	    sprintf(vlabel, _("squared residual from model %d"), pmod->ID);
+	}
+    } else if (code == M_H) { 
+	/* garch variance */
+	sprintf(vname, "h%d", pmod->ID);
+	sprintf(vlabel, _("fitted variance from model %d"), pmod->ID);
+    } else if (code == M_AHAT) { 
+	/* fixed-effects constants */
+	sprintf(vname, "ahat%d", pmod->ID);
+	sprintf(vlabel, _("per-unit constants from model %d"), pmod->ID);
+    }	
+
+    return ret;
+}
+
+/**
+ * genr_fit_resid:
+ * @pmod: pointer to source model.
+ * @pZ: pointer to data array.
+ * @pdinfo: information on the data set.
+ * @code: M_UHAT, M_UHAT2, M_YHAT, M_AHAT or M_H.
+ * @undo: if non-zero, don't bother labeling the series.
+ * 
  * Adds residuals or fitted values or squared residuals from a
  * given model to the data set.
  * 
@@ -1790,85 +2013,27 @@ int genr_fit_resid (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		    int code, int undo)
 {
     char vname[VNAMELEN], vlabel[MAXLABEL];
-    const double *x = NULL;
-    int i, t;
+    double *x;
+    int err = 0;
 
-    if (code == M_H) {
-	x = gretl_model_get_data(pmod, "garch_h");
-	if (x == NULL) return E_BADSTAT;
-    } else if (code == M_AHAT) {
-	x = gretl_model_get_data(pmod, "ahat");
-	if (x == NULL) return E_BADSTAT;
-    } else if (code == M_UHAT || code == M_UHAT2) {
-	if (pmod->uhat == NULL) return E_BADSTAT;
-    } else if (code == M_YHAT) {
-	if (pmod->yhat == NULL) return E_BADSTAT;
+    x = get_fit_or_resid(pmod, pdinfo, code, vname, vlabel, &err);
+
+    if (!err) {
+	err = dataset_add_allocated_series(x, pZ, pdinfo);
     }
 
-    if (dataset_add_series(1, pZ, pdinfo)) {
-	return E_ALLOC;
+    if (err) {
+	free(x);
+    } else {
+	int v = pdinfo->v - 1;
+
+	strcpy(pdinfo->varname[v], vname);
+	if (!undo) {
+	    strcpy(VARLABEL(pdinfo, v), vlabel);
+	}
     }
 
-    i = pdinfo->v - 1;
-
-    for (t=0; t<pdinfo->n; t++) {
-	(*pZ)[i][t] = NADBL;
-    }
-
-    if (code == M_UHAT) {
-	sprintf(vname, "uhat%d", pmod->ID);
-	if (pmod->ci == GARCH && (pmod->opt & OPT_U)) {
-	    sprintf(vlabel, _("standardized residual from model %d"), pmod->ID);
-	} else {
-	    sprintf(vlabel, _("residual from model %d"), pmod->ID);
-	}
-	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    (*pZ)[i][t] = pmod->uhat[t];
-	}
-    } else if (code == M_YHAT) {
-	sprintf(vname, "yhat%d", pmod->ID);
-	sprintf(vlabel, _("fitted value from model %d"), pmod->ID);
-	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    (*pZ)[i][t] = pmod->yhat[t];
-	}
-    } else if (code == M_UHAT2) { 
-	/* squared residuals */
-	sprintf(vname, "usq%d", pmod->ID);
-	if (pmod->ci == GARCH && (pmod->opt & OPT_U)) {
-	    sprintf(vlabel, _("squared standardized residual from model %d"), pmod->ID);
-	} else {
-	    sprintf(vlabel, _("squared residual from model %d"), pmod->ID);
-	}
-	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    if (na(pmod->uhat[t])) {
-		(*pZ)[i][t] = NADBL;
-	    } else {
-		(*pZ)[i][t] = pmod->uhat[t] * pmod->uhat[t];
-	    }
-	}
-    } else if (code == M_H) { 
-	/* garch variance */
-	sprintf(vname, "h%d", pmod->ID);
-	sprintf(vlabel, _("fitted variance from model %d"), pmod->ID);
-	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    (*pZ)[i][t] = x[t];
-	}
-    } else if (code == M_AHAT) { 
-	/* fixed-effects constants */
-	sprintf(vname, "ahat%d", pmod->ID);
-	sprintf(vlabel, _("per-unit constants from model %d"), pmod->ID);
-	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    (*pZ)[i][t] = x[t];
-	}
-    }	
-
-    strcpy(pdinfo->varname[i], vname);
-
-    if (!undo) {
-	strcpy(VARLABEL(pdinfo, i), vlabel);
-    }
-
-    return 0;
+    return err;
 }
 
 int get_observation_number (const char *s, const DATAINFO *pdinfo)
@@ -1885,7 +2050,7 @@ int get_observation_number (const char *s, const DATAINFO *pdinfo)
 	test[n-1] = '\0';
     }
 
-    if (pdinfo->markers && pdinfo->S != NULL) {
+    if (dataset_has_markers(pdinfo)) {
 	for (t=0; t<pdinfo->n; t++) {
 	    if (!strcmp(test, pdinfo->S[t])) {
 		return t + 1;
@@ -1974,6 +2139,7 @@ int get_t_from_obs_string (const char *s, const double **Z,
 	    } 
 
 	    if (t > pdinfo->n) {
+		/* e.g. annual dates */
 		char try[16];
 
 		sprintf(try, "%d", t);
@@ -2275,6 +2441,30 @@ static int x_sectional_extremum (int f, double *x, const int *list,
     return err;
 }
 
+static int x_sectional_sum (double *x, const int *list, 
+			    const double **Z, 
+			    const DATAINFO *pdinfo)
+{
+    double xit, xx;
+    int i, t, err = 0;
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	xx = 0.0;
+	for (i=1; i<=list[0]; i++) {
+	    xit = Z[list[i]][t];
+	    if (na(xit)) { 
+		xx = NADBL;
+		break;
+	    } else {
+		xx += xit;
+	    }
+	}
+	x[t] = xx;
+    }
+
+    return err;
+}
+
 int cross_sectional_stat (double *x, const int *list, 
 			  const double **Z, 
 			  const DATAINFO *pdinfo,
@@ -2288,6 +2478,8 @@ int cross_sectional_stat (double *x, const int *list,
 	return x_sectional_wtd_stddev(x, list, NULL, Z, pdinfo);
     } else if (f == F_MIN || f == F_MAX) {
 	return x_sectional_extremum(f, x, list, NULL, Z, pdinfo);
+    } else if (f == F_SUM) {
+	return x_sectional_sum(x, list, Z, pdinfo);
     } else {
 	return E_DATA;
     }
