@@ -52,6 +52,13 @@ int boxcox_series (const double *x, double *y, double d,
 int filter_series (const double *x, double *y, const DATAINFO *pdinfo, 
 		   gretl_matrix *A, gretl_matrix *C, double y0);
 
+int exponential_movavg_series (const double *x, double *y, 
+			       const DATAINFO *pdinfo,
+			       double d, int n);
+
+int movavg_series (const double *x, double *y, const DATAINFO *pdinfo,
+		   int k, int center);
+
 int seasonally_adjust_series (const double *x, double *y, 
 			      DATAINFO *pdinfo, int tramo);
 
@@ -59,10 +66,10 @@ int panel_statistic (const double *x, double *y, const DATAINFO *pdinfo,
 		     int k);
 
 int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo,
-	       gretlopt opt);
+	       double lambda, gretlopt opt);
 
-int bkbp_filter (const double *y, double *bk, 
-		 const DATAINFO *pdinfo);
+int bkbp_filter (const double *y, double *bk, const DATAINFO *pdinfo, 
+		 int bkl, int bku, int k);
 
 int dummy (double ***pZ, DATAINFO *pdinfo, int center);
 
@@ -80,6 +87,9 @@ int plotvar_code (const DATAINFO *pdinfo);
 
 const double *gretl_plotx (const DATAINFO *pdinfo);
 
+double *get_fit_or_resid (const MODEL *pmod, DATAINFO *pdinfo, int code,
+			  char *vname, char *vlabel, int *err);
+
 int genr_fit_resid (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		    int code, int undo);
 
