@@ -56,6 +56,7 @@ struct str_table consts[] = {
     { CONST_PI,    "pi" },
     { CONST_NA,    "NA" },
     { CONST_WIN32, "WIN32" },
+    { CONST_EPS,   "macheps" },
     { 0,        NULL }
 };
 
@@ -199,6 +200,7 @@ struct str_table funcs[] = {
     { F_QNORM,    "qnorm" },
     { F_GAMMA,    "gammafun" },
     { F_LNGAMMA,  "lngamma" },
+    { F_DIGAMMA,  "digamma" },
     { F_RESAMPLE, "resample" },
     { F_PNOBS,    "pnobs" },     /* per-unit nobs in panels */
     { F_PMIN,     "pmin" },      /* panel min */
@@ -338,6 +340,9 @@ struct str_table funcs[] = {
     { F_NPV,      "npv" },
     { F_LOGISTIC, "logistic" },
     { F_WEEKDAY,  "weekday" },
+    { F_KDENSITY, "kdensity" },
+    { F_MONTHLEN, "monthlen" },
+    { F_EPOCHDAY, "epochday" },
     { 0,          NULL }
 };
 
@@ -874,7 +879,7 @@ static void look_up_word (const char *s, parser *p)
 		    p->idstr = gretl_strdup(s);
 		} else if (!strcmp(s, "t")) {
 		    /* if "t" has not been otherwise defined, treat it
-		       as a synonym for the internal variable "obs" 
+		       as an alias for "obs"
 		    */
 		    p->sym = DVAR;
 		    p->idnum = R_INDEX;
