@@ -129,8 +129,6 @@ int genr_get_output_varnum (const GENERATOR *genr);
 
 double genr_get_output_scalar (const GENERATOR *genr);
 
-int genr_get_warning (const GENERATOR *genr);
-
 int genr_get_last_output_type (void);
 
 gretl_matrix *genr_get_output_matrix (const GENERATOR *genr);
@@ -191,6 +189,8 @@ int function_lookup (const char *s);
 
 int const_lookup (const char *s);
 
+double get_const_by_name (const char *name);
+
 const char *gretl_function_complete (const char *s);
 
 void gretl_function_hash_cleanup (void);
