@@ -171,8 +171,7 @@ enum {
     F_QNORM,
     F_GAMMA,	
     F_LNGAMMA,
-    F_HPFILT,
-    F_BKFILT,
+    F_DIGAMMA,
     F_PNOBS,
     F_PMIN,
     F_PMAX,
@@ -259,6 +258,7 @@ enum {
     F_SORTBY,
     F_RUNIFORM,
     F_RNORMAL,
+    F_HPFILT,
     F_FRACDIFF,
     F_BOXCOX,
     F_ZEROS,
@@ -278,7 +278,6 @@ enum {
     F_CDIV,	  /* complex division */
     F_MXTAB,
     F_MWRITE,
-    F_MOVAVG,
     F_MRSEL,
     F_MCSEL,
     F_LLAG,
@@ -312,7 +311,12 @@ enum {
     F_STRNCMP,
     F_BESSEL,
     F_WEEKDAY,
+    F_MONTHLEN,
+    F_EPOCHDAY,
+    F_MOVAVG,
+    F_KDENSITY,
     F3_MAX,       /* SEPARATOR: end of three-arg functions */
+    F_BKFILT,
     F_MOLS,
     F_MPOLS,
     F_FILTER,
@@ -326,7 +330,8 @@ enum {
 enum {
     CONST_PI = 1,
     CONST_NA,
-    CONST_WIN32
+    CONST_WIN32,
+    CONST_EPS
 };
 
 enum {
@@ -348,7 +353,8 @@ enum {
 #define string_arg_func(s) (s == F_ISSERIES || s == F_ISNULL || \
 			    s == F_ISLIST   || s == F_LISTLEN || \
 			    s == F_ISSTRING || s == F_OBSNUM || \
-			    s == F_BACKTICK || s == F_VARNUM)
+			    s == F_BACKTICK || s == F_VARNUM || \
+			    s == F_ARGNAME)
 
 /* function with multiple args, character code for first arg */
 #define char0_func(s) (s == F_PVAL || s == F_CDF || s == F_INVCDF || \
@@ -514,7 +520,6 @@ struct parser_ {
     NODE **aux;        /* auxiliary nodes used in evaluation */
     int n_aux;         /* the number of the above */
     int aux_i;         /* the current ID of the above */
-    char warning[64];  /* to hold a warning, if needed */
     int loopline;      /* line number within loop, if applicable */
     /* below: parser state variables */
     int obs;
@@ -524,7 +529,6 @@ struct parser_ {
     int idnum;
     char *idstr;
     int err;
-    int warn;
 };
 
 #define starting(p) (p->flags & P_START)
