@@ -49,7 +49,6 @@ static struct gretl_cmd gretl_cmds[] = {
     { COINT2,   "coint2" },
     { CORR,     "corr" },     
     { CORRGM,   "corrgm" },   
-    { CRITERIA, "criteria" },
     { CUSUM,    "cusum" },
     { DATA,     "data" },
     { DATAMOD,  "dataset" },
@@ -58,6 +57,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { DIFFTEST, "difftest" },
     { DISCRETE, "discrete" },
     { DUMMIFY,  "dummify" },
+    { DURATION, "duration" },
     { ELIF,     "elif" },
     { ELSE,     "else" },
     { END,      "end" },
@@ -75,7 +75,6 @@ static struct gretl_cmd gretl_cmds[] = {
     { GENR,     "genr" },  
     { GMM,      "gmm" },
     { GNUPLOT,  "gnuplot" },  
-    { GRAPH,    "graph" }, 
     { GRAPHPG,  "graphpg" }, 
     { HAUSMAN,  "hausman" },
     { HECKIT,   "heckit" },
@@ -104,6 +103,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { MODPRINT, "modprint" },
     { MODTEST,  "modtest" },
     { MPOLS,    "mpols" },
+    { NEGBIN,   "negbin" },
     { NLS,      "nls" },
     { NORMTEST, "normtest" },
     { NULLDATA, "nulldata" },
@@ -115,7 +115,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { PANEL,    "panel" },
     { PCA,      "pca" },
     { PERGM,    "pergm" },
-    { PLOT,     "plot" },    
+    { PLOT,     "textplot" },    
     { POISSON,  "poisson" },
     { PRINT,    "print" }, 
     { PRINTF,   "printf" },
@@ -159,6 +159,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { XTAB,     "xtab" },
     { FUNDEBUG, "debug" },
     { FUNCRET,  "return" },
+    { CATCH,    "catch" },
     { NC,       NULL}
 }; 
 
@@ -168,9 +169,6 @@ static struct gretl_cmd gretl_cmd_aliases[] = {
     { GENR, "scalar" },  
     { GENR, "matrix" },  
     { GENR, "string" },
-    { LOOP, "for" },   
-    { LOOP, "foreach" },
-    { LOOP, "while" },
     { ARMA, "arma" },
     { NC,   NULL }
 };
