@@ -39,7 +39,6 @@ typedef enum {
     COINT2,
     CORR,     
     CORRGM,   
-    CRITERIA,
     CUSUM,
     DATA,
     DATAMOD,
@@ -48,6 +47,7 @@ typedef enum {
     DIFFTEST,
     DISCRETE,
     DUMMIFY,
+    DURATION,
     ELIF,
     ELSE,
     END,
@@ -65,7 +65,6 @@ typedef enum {
     GENR,  
     GMM,
     GNUPLOT, 
-    GRAPH,
     GRAPHPG,
     HAUSMAN,
     HECKIT,
@@ -94,6 +93,7 @@ typedef enum {
     MODPRINT,
     MODTEST,
     MPOLS,
+    NEGBIN,
     NLS,
     NORMTEST,
     NULLDATA,
@@ -149,6 +149,7 @@ typedef enum {
     XTAB,
     FUNDEBUG,
     FUNCRET,
+    CATCH,
     NC
 } GretlCmdIndex;
 
@@ -156,7 +157,7 @@ typedef enum {
                             (c) == AR1 || (c) == CORR || (c) == HSK || \
                             (c) == LOGIT || (c) == PROBIT || \
                             (c) == SPEARMAN || (c) == OLS || \
-                            (c) == IVREG || (c) == VAR || (c) == WLS || \
+                            (c) == IVREG || (c) == WLS || \
 			    (c) == XTAB)
 
 #define TEXTSAVE_OK(c) (c == ADD || \
@@ -167,11 +168,9 @@ typedef enum {
                         c == COINT2 || \
                         c == CORR || \
                         c == CORRGM || \
-                        c == CRITERIA || \
                         c == CUSUM || \
                         c == FCAST || \
                         c == FREQ || \
-                        c == GRAPH || \
                         c == HAUSMAN || \
                         c == KPSS || \
                         c == LEVERAGE || \
