@@ -197,6 +197,9 @@ gretl_matrix *gretl_matrix_reuse (gretl_matrix *m, int rows, int cols);
 
 int gretl_matrix_realloc (gretl_matrix *m, int rows, int cols);
 
+gretl_matrix *gretl_matrix_replace (gretl_matrix **pa, 
+				    gretl_matrix *b);
+
 void gretl_matrix_block_destroy (gretl_matrix_block *B);
 
 gretl_matrix_block *gretl_matrix_block_new (gretl_matrix **pm, ...);
@@ -253,6 +256,8 @@ int gretl_matrix_zero_upper (gretl_matrix *m);
 
 int gretl_matrix_zero_lower (gretl_matrix *m);
 
+int gretl_matrix_mirror (gretl_matrix *m, char uplo);
+
 void gretl_matrix_fill (gretl_matrix *m, double x);
 
 void gretl_matrix_multiply_by_scalar (gretl_matrix *m, double x);
@@ -484,6 +489,10 @@ gretl_gensymm_eigenvals (const gretl_matrix *A,
 			 gretl_matrix *V, 
 			 int *err);
 
+double gretl_symm_matrix_lambda_min (const gretl_matrix *m);
+
+double gretl_symm_matrix_lambda_max (const gretl_matrix *m);
+
 gretl_matrix *gretl_matrix_right_nullspace (const gretl_matrix *M, 
 					    int *err);
 
@@ -607,6 +616,8 @@ int gretl_matrices_are_equal (const gretl_matrix *a, const gretl_matrix *b,
 gretl_matrix *gretl_covariance_matrix (const gretl_matrix *m, int corr,
 				       int *errp);
 
+gretl_matrix *gretl_matrix_GG_inverse (const gretl_matrix *G, int *err);
+
 gretl_matrix **gretl_matrix_array_new (int n);
 
 gretl_matrix **
