@@ -102,6 +102,7 @@ struct VCVInfo_ {
                           c == ARBOND || \
                           c == ARCH || \
                           c == ARMA || \
+			  c == DURATION || \
                           c == GARCH || \
                           c == GMM || \
 		          c == HECKIT || \
@@ -113,6 +114,7 @@ struct VCVInfo_ {
                           c == LOGIT || \
                           c == MLE || \
                           c == MPOLS || \
+			  c == NEGBIN || \
                           c == NLS || \
                           c == OLS || \
                           c == PANEL || \
@@ -130,10 +132,12 @@ struct VCVInfo_ {
 #define SIMPLE_AR_MODEL(c) (c == AR || c == AR1)
 
 #define ML_ESTIMATOR(c) (c == ARMA || \
+			 c == DURATION || \
                          c == GARCH || \
                          c == HECKIT || \
                          c == LOGIT || \
                          c == MLE || \
+			 c == NEGBIN ||	\
                          c == POISSON || \
                          c == PROBIT || \
                          c == TOBIT)
@@ -142,6 +146,8 @@ struct VCVInfo_ {
                    c == PROBIT || \
                    c == TOBIT)
 
+#define COUNT_MODEL(c) (c == POISSON || c == NEGBIN)
+
 #define LSQ_MODEL(c) (c == AR1 || \
                       c == HSK || \
                       c == OLS || \
@@ -149,6 +155,7 @@ struct VCVInfo_ {
 
 #define ASYMPTOTIC_MODEL(c) (c == ARBOND || \
                              c == ARMA || \
+			     c == DURATION || \
                              c == GARCH || \
                              c == GMM || \
                              c == HECKIT || \
@@ -156,6 +163,7 @@ struct VCVInfo_ {
                              c == IVREG || \
                              c == LOGIT || \
                              c == MLE || \
+			     c == NEGBIN || \
                              c == POISSON || \
                              c == PROBIT || \
                              c == TOBIT)
