@@ -64,6 +64,7 @@ struct INTERNAL_PATHS {
     char rbinpath[MAXLEN];
     char rlibpath[MAXLEN];
     char oxlpath[MAXLEN];
+    char octpath[MAXLEN];
     char dbhost[32];
     char pngfont[128];
     unsigned char status;
@@ -844,7 +845,7 @@ static void make_findname (char *targ, const char *src)
 {
     strcpy(targ, src);
 
-    if (string_is_utf8(targ)) {
+    if (string_is_utf8((const unsigned char *) targ)) {
 	gchar *tmp;
 	gsize sz;
 	
@@ -1305,7 +1306,8 @@ static int substitute_homedir (char *fname)
  * @line: command line (e.g. "open foo").
  * @fname: filename to be filled out.
  * @opt: if includes %OPT_W, treat as web filename and don't
- * try to add path, if %OPT_S, treat as a script.
+ * try to add path; if %OPT_S, treat as a script; if %OPT_I
+ * we're responding to the "include" command.
  * 
  * Elementary path-searching: try adding various paths to the given
  * @fname and see if it can be opened.
@@ -1315,7 +1317,7 @@ static int substitute_homedir (char *fname)
 
 int getopenfile (const char *line, char *fname, gretlopt opt)
 {
-    int script = (opt & OPT_S)? 1 : 0;
+    int script = (opt & (OPT_S | OPT_I))? 1 : 0;
     char *fullname;
 
     /* skip past command word */
@@ -1347,7 +1349,7 @@ int getopenfile (const char *line, char *fname, gretlopt opt)
     /* try a basic path search on this filename */
     fullname = addpath(fname, script);
 
-    if (fullname != NULL && script) {
+    if (fullname != NULL && (opt & OPT_S)) {
 	int spos = slashpos(fname);
 
 	if (spos) {
@@ -1698,6 +1700,13 @@ char *set_gretl_plotfile (const char *fname)
     return paths.plotfile;
 }
 
+void report_plot_written (PRN *prn)
+{
+    if (prn != NULL) {
+	pprintf(prn, _("wrote %s\n"), paths.plotfile);
+    }
+}
+
 const char *gretl_binbase (void)
 {
     return paths.binbase;
@@ -1738,17 +1747,15 @@ const char *gretl_rlib_path (void)
     return paths.rlibpath;
 }
 
-#ifdef USE_OX
 const char *gretl_oxl_path (void)
 {
     return paths.oxlpath;
 }
-#else
-const char *gretl_oxl_path (void)
+
+const char *gretl_octave_path (void)
 {
-    return "unsupported";
+    return paths.octpath;
 }
-#endif
 
 const char *gretl_current_dir (void)
 {
@@ -2039,7 +2046,7 @@ static int maybe_transcribe_path (char *targ, char *src,
 
    gretldir
    gnuplot (but not on MS Windows)
-   tramo, x12a, rbinpath, rlibpath, oxlpath, 
+   tramo, x12a, rbinpath, rlibpath, oxlpath, octpath,
    ratsbase, dbhost
 
    * paths.workdir is updated via the separate working directory
@@ -2073,6 +2080,7 @@ int gretl_update_paths (ConfigPaths *cpaths, gretlopt opt)
     ndelta += maybe_transcribe_path(paths.tramo, cpaths->tramo, 0);
     ndelta += maybe_transcribe_path(paths.rbinpath, cpaths->rbinpath, 0);
     ndelta += maybe_transcribe_path(paths.oxlpath, cpaths->oxlpath, 0);
+    ndelta += maybe_transcribe_path(paths.octpath, cpaths->octpath, 0);
 
 #ifdef USE_RLIB
     if (maybe_transcribe_path(paths.rlibpath, cpaths->rlibpath, 0)) {
@@ -2132,11 +2140,9 @@ static void load_default_path (char *targ)
     } else if (targ == paths.rlibpath) {
 	R_path_from_registry(targ, RLIB);
     } else if (targ == paths.oxlpath) {
-#ifdef USE_OX
 	sprintf(targ, "%s\\OxMetrics5\\Ox\\bin\\oxl.exe", progfiles);
-#else
-	*paths.oxlpath = '\0';
-#endif
+    } else if (targ == paths.octpath) {
+	strcpy(targ, "C:\\Octave\\3.2.3_gcc-4.0.0\\bin\\octave.exe");
     } else if (targ == paths.pngfont) {
 	strcpy(targ, "verdana 8");
     }
@@ -2185,13 +2191,13 @@ static void load_default_path (char *targ)
 	strcpy(targ, "x12a");
 #else
 	*targ = '\0';
-#endif
+#endif /* HAVE_X12A */
     } else if (targ == paths.tramo) {
 #ifdef HAVE_TRAMO
 	strcpy(targ, "tramo");
 #else
 	*targ = '\0';
-#endif
+#endif /* HAVE_TRAMO */
     } else if (targ == paths.rbinpath) {
 	strcpy(paths.rbinpath, "R");
     } else if (targ == paths.rlibpath) {
@@ -2199,23 +2205,26 @@ static void load_default_path (char *targ)
 	strcpy(paths.rlibpath, RLIBPATH);
 #else
 	*paths.rlibpath = '\0';
-#endif
+#endif /* RLIBPATH */
     } else if (targ == paths.oxlpath) {
-#ifdef USE_OX
-# ifdef OSX_BUILD
+#ifdef OSX_BUILD
 	strcpy(paths.oxlpath, "/Applications/OxMetrics5/ox/bin/oxl");
-# else
+#else
 	strcpy(paths.oxlpath, "oxl");
-# endif
-#else /* USE_OX */
-	*paths.oxlpath = '\0';
-#endif
+#endif /* OSX_BUILD */
+    } else if (targ == paths.octpath) {
+#ifdef OSX_BUILD
+	strcpy(paths.octpath, 
+	       "/Applications/Octave.app/Contents/Resources/bin/octave");
+#else
+	strcpy(paths.octpath, "octave");
+#endif /* OSX_BUILD */
     } else if (targ == paths.pngfont) {
 #ifdef OSX_BUILD
 	strcpy(targ, "Sans 9");
 #else
 	strcpy(targ, "Vera 9");
-#endif	
+#endif /* OSX_BUILD */	
     }
 }
 
@@ -2274,6 +2283,7 @@ static void copy_paths_with_fallback (ConfigPaths *cpaths)
     path_init(paths.rbinpath, cpaths->rbinpath, 0);
     path_init(paths.rlibpath, cpaths->rlibpath, 0);
     path_init(paths.oxlpath, cpaths->oxlpath, 0);
+    path_init(paths.octpath, cpaths->octpath, 0);
 
     /* graphing font */
     path_init(paths.pngfont, cpaths->pngfont, 0);
@@ -2378,7 +2388,20 @@ const char *gretl_maybe_switch_dir (const char *fname)
     return fname;
 }
 
-/* argument should be of length FILENAME_MAX */
+/**
+ * gretl_maybe_prepend_dir:
+ * @fname: the original filename, which should be in a
+ * location of length %FILENAME_MAX.
+ *
+ * If @fname starts with the construction "~/" to indicate
+ * the user's %HOME, replace this with the full path to that
+ * directory.  Otherwise, if @fname is not already an
+ * absolute path, prepend either the gretl "shelldir" or the
+ * user's gretl working directory, depending on whether or
+ * %USE_CWD is set. Otherwise do nothing.
+ * 
+ * Returns: the possibly modified filename.
+ */
 
 char *gretl_maybe_prepend_dir (char *fname)
 {
@@ -2411,6 +2434,35 @@ char *gretl_maybe_prepend_dir (char *fname)
     return fname;
 }
 
+/**
+ * gretl_read_user_file:
+ * @fname: name of file to open.
+ *
+ * Attempts to open @fname in read-only mode.  If the file
+ * is not found when the name is used "as is", we use
+ * gretl_maybe_prepend_dir() to prepend the user's gretl 
+ * working directory and try again.
+ *
+ * Returns: file pointer, or %NULL on failure.
+ */
+
+FILE *gretl_read_user_file (const char *fname)
+{
+    FILE *fp = gretl_fopen(fname, "r");
+
+    if (fp == NULL) {
+	char fullname[FILENAME_MAX];
+
+	strcpy(fullname, fname);
+	gretl_maybe_prepend_dir(fullname);
+	if (*fullname != '\0') {
+	    fp = gretl_fopen(fullname, "r");
+	}
+    }    
+
+    return fp;
+}
+
 /* remove '.' and '..' from @path */
 
 int gretl_normalize_path (char *path)
@@ -2622,6 +2674,8 @@ int cli_read_rc (void)
 		strncat(cpaths.rlibpath, val, MAXLEN - 1);
 	    } else if (!strcmp(key, "ox")) {
 		strncat(cpaths.oxlpath, val, MAXLEN - 1);
+	    } else if (!strcmp(key, "octave")) {
+		strncat(cpaths.octpath, val, MAXLEN - 1);
 	    } else if (!strcmp(key, "Png_font")) {
 		strncat(cpaths.pngfont, val, 128 - 1);
 	    } else if (!strcmp(key, "Gp_colors")) {
