@@ -160,15 +160,11 @@ static int augment_vecm_restriction (gretl_restriction *rset,
     if (err) {
 	gretl_matrix_free(R2);
     } else if (letter == 'b') {
-	gretl_matrix_free(rset->R);
-	rset->R = R2;
-	gretl_matrix_free(rset->q);
-	rset->q = q2;
+	gretl_matrix_replace(&rset->R, R2);
+	gretl_matrix_replace(&rset->q, q2);
     } else {
-	gretl_matrix_free(rset->Ra);
-	rset->Ra = R2;
-	gretl_matrix_free(rset->qa);
-	rset->qa = q2;
+	gretl_matrix_replace(&rset->Ra, R2);
+	gretl_matrix_replace(&rset->qa, q2);
     }	
 
     return err;
@@ -589,6 +585,10 @@ static int vecm_form_matrices (gretl_restriction *rset)
 	err = check_R_matrix(rset->Ra);
     }
 
+#if RDEBUG
+    fprintf(stderr, "vecm_form_matrices: returning %d\n", err);
+#endif
+
     return err;
 }
 
@@ -751,16 +751,22 @@ bnum_from_name (gretl_restriction *r, const DATAINFO *pdinfo,
    variable.  This function is actually fed the string in question at
    an offset of 1 beyond the "[".  We skip any white space in the 
    string.
+
+   Such terms may be preceded by a multiplier, but they should not
+   be followed by anything, so we flag an error if the closing "]"
+   is not the end of the string.
 */
 
 static int pick_apart (gretl_restriction *r, const char *s, 
 		       int *eq, int *bnum,
 		       const DATAINFO *pdinfo)
 {
+    const char *junk;
     char s1[16] = {0};
     char s2[16] = {0};
     char *targ = s1;
     int i, j, k;
+    int err = 0;
 
 #if RDEBUG
     fprintf(stderr, "pick_apart: looking at '%s'\n", s);
@@ -772,6 +778,13 @@ static int pick_apart (gretl_restriction *r, const char *s,
     if (k <= 0 || k > 30) {
 	return E_PARSE;
     }
+    
+    junk = s + k + 1;
+    while (isspace(*junk)) junk++;
+    if (*junk != '\0') {
+	fprintf(stderr, "Got trailing junk '%s'\n", junk);
+	return E_PARSE;
+    }
 
     j = 0;
     for (i=0; i<k; i++) {
@@ -794,7 +807,7 @@ static int pick_apart (gretl_restriction *r, const char *s,
 	/* got a comma separator: [eqn,bnum] */
 	*eq = positive_int_from_string(s1);
 	if (*eq <= 0) {
-	    return E_PARSE;
+	    err = E_PARSE;
 	}
 	if (isdigit(*s2)) {
 	    *bnum = positive_int_from_string(s2);
@@ -811,7 +824,7 @@ static int pick_apart (gretl_restriction *r, const char *s,
 	}	    
     }
 
-    return 0;
+    return err;
 }
 
 static int bbit_trailing_garbage (const char *s)
@@ -857,7 +870,7 @@ static int parse_b_bit (gretl_restriction *r, const char *s,
     }
 
     if (*bnum < 1) {
-	if (!gretl_errmsg_is_set()) {
+	if (!gretl_errmsg_is_set() && *bnum >= 0) {
 	    gretl_errmsg_sprintf(_("Coefficient number (%d) is out of range"), 
 				 *bnum);
 	}
@@ -873,7 +886,7 @@ static int parse_b_bit (gretl_restriction *r, const char *s,
 	} else if (r->otype != GRETL_OBJ_VAR) {
 	    err = E_PARSE;
 	}
-    } else if (*eq < 1) {
+    } else if (*eq < 1 && *eq >= 0) {
 	gretl_errmsg_sprintf(_("Equation number (%d) is out of range"), 
 			     *eq);
 	err = 1;
@@ -911,15 +924,41 @@ static int r_get_scalar (const char *s, double *x)
     return 0;
 }
 
+static int get_named_param (const char *s, MODEL *pmod,
+			    int *bnum) 
+{
+    if (NONLIST_MODEL(pmod->ci) && pmod->params != NULL) {
+	char test[VNAMELEN];
+	int i;
+
+	*test = '\0';
+	strncat(test, s, VNAMELEN - 1);
+	tailstrip(test);
+
+	for (i=0; i<pmod->nparams; i++) {
+	    if (!strcmp(test, pmod->params[i])) {
+		*bnum = i;
+		return 1;
+	    }
+	}
+    }
+
+    return 0;
+}
+
 #define b_start(r,s) ((*s == 'b' || (r->vecm && *s == 'a')) && \
 		      (isdigit(*(s+1)) || *(s+1) == '['))
 
+#define could_be_param(r,s) (r->otype == GRETL_OBJ_EQN && \
+	                     !isdigit(*s))
+
 static int 
 parse_coeff_chunk (gretl_restriction *r, const char *s, double *x, 
 		   int *eq, int *bnum, char *letter, 
 		   const DATAINFO *pdinfo)
 {
     const char *s0 = s;
+    int done = 0;
     int err = E_PARSE;
 
     *eq = 1;
@@ -931,6 +970,7 @@ parse_coeff_chunk (gretl_restriction *r, const char *s, double *x,
 #endif
 
     if (b_start(r, s)) {
+	/* something like "b[" or "b2" */
 	*letter = *s;
 	s++;
 #if RDEBUG
@@ -938,7 +978,19 @@ parse_coeff_chunk (gretl_restriction *r, const char *s, double *x,
 #endif
 	err = parse_b_bit(r, s, eq, bnum, pdinfo);
 	*x = 1.0;
-    } else if (r_get_scalar(s, x)) {
+	done = 1;
+    } else if (could_be_param(r, s)) {
+	/* e.g. "beta" for MLE or GMM */
+	done = get_named_param(s, r->obj, bnum);
+	if (done) {
+	    *letter = 'b';
+	    *x = 1.0;
+	    *eq = 0;
+	    err = 0;
+	}
+    }
+
+    if (!done && r_get_scalar(s, x)) {
 	s += strspn(s, " ");
 	s += strcspn(s, " *");
 	s += strspn(s, " *");
@@ -1550,6 +1602,16 @@ static int parse_restriction_row (gretl_restriction *rset,
 	}
     }
 
+#if RDEBUG
+    fprintf(stderr, "restriction row: nterms = %d, rhs = %g\n", 
+	    row->nterms, row->rhs);
+#endif
+
+    if (!err && row->nterms == 0) {
+	gretl_errmsg_set(_("Invalid restriction"));
+	err = E_PARSE;
+    }
+
     return err;
 }
 
@@ -1823,8 +1885,7 @@ static int rset_expand_R (gretl_restriction *rset, int k)
 	}
     }
 
-    gretl_matrix_free(rset->R);
-    rset->R = R;
+    gretl_matrix_replace(&rset->R, R);
 
     return 0;
 }
@@ -2187,7 +2248,7 @@ gretl_restricted_vecm (gretl_restriction *rset,
     GRETL_VAR *jvar = NULL;
 
 #if RDEBUG
-    fprintf(stderr, "gretl_restricted_vecm()\n");
+    fprintf(stderr, "gretl_restricted_vecm: starting\n");
 #endif
 
     if (rset == NULL || rset->otype != GRETL_OBJ_VAR) {
@@ -2339,7 +2400,7 @@ gretl_restriction_finalize (gretl_restriction *rset,
     int err = 0;
 
 #if RDEBUG
-    fprintf(stderr, "gretl_restriction_finalize()\n");
+    fprintf(stderr, "gretl_restriction_finalize: starting\n");
 #endif
 
     if (rset == NULL) {
