@@ -20,6 +20,12 @@
 #ifndef GRETL_SCALAR_H
 #define GRETL_SCALAR_H
 
+typedef enum {
+    SCALAR_ADD,
+    SCALAR_SET,
+    SCALAR_DELETE
+} ScalarAction;
+
 int n_saved_scalars (void);
 
 int gretl_is_scalar (const char *name);
@@ -53,9 +59,7 @@ void destroy_private_scalars (void);
 
 void destroy_user_scalars (void);
 
-void print_scalar_by_name (const char *name, gretlopt opt, PRN *prn);
-
-void print_all_scalars (gretlopt opt, PRN *prn);
+void print_scalar_by_name (const char *name, PRN *prn);
 
 void set_auxiliary_scalars (void);
 
@@ -63,4 +67,8 @@ void unset_auxiliary_scalars (void);
 
 void write_scalars_to_file (FILE *fp);
 
+void print_scalars (PRN *prn);
+
+void set_scalar_edit_callback (void (*callback));
+
 #endif /* GRETL_SCALAR_H */
