@@ -108,36 +108,29 @@ int ijton (int i, int j, int nrows)
 }
 
 /**
- * ztox:
- * @i: index number of variable to extract.
- * @px: array into which to write the series.
- * @Z: data matrix.
+ * transcribe_array:
+ * @targ: arrat to which to write.
+ * @src: array from which to read.
  * @pdinfo: data information struct.
  * 
- * Pull one series from data matrix and put it into @px.
+ * Copy from @src to @targ, skipping any missing values,
+ * over the sample range defined in @pdinfo.
  *
- * Returns: the number of valid observations put into @px.
+ * Returns: the number of valid observations put into @targ.
  */
 
-int ztox (int i, double *px, const double **Z, const DATAINFO *pdinfo) 
+int transcribe_array (double *targ, const double *src, 
+		      const DATAINFO *pdinfo) 
 {
-    int t, m = 0;
-    double xx;
+    int t, n = 0;
 
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	xx = Z[i][t];
-	if (na(xx)) {
-	    continue;
+	if (!na(src[t])) {
+	    targ[n++] = src[t];
 	}
-	else px[m++] = xx;
     }
 
-    if (m == 0) {
-	fprintf(stderr, "\nztox: No valid observations for variable %s\n", 
-		pdinfo->varname[i]);
-    } 
-
-    return m;
+    return n;
 }
 
 /**
@@ -146,7 +139,7 @@ int ztox (int i, double *px, const double **Z, const DATAINFO *pdinfo)
  * @t2: ending observation. 
  * @x: data series to examine.
  * 
- * Check whether variable @x has only 0 or 1 values over the
+ * Check whether series @x has only 0 or 1 values over the
  * given sample range (or possibly missing values).
  *
  * Returns: 0 if the variable is not a 0/1 dummy, otherwise the
@@ -183,7 +176,7 @@ int gretl_isdummy (int t1, int t2, const double *x)
  * @t1: starting observation.
  * @t2: ending observation. 
  * 
- * Check whether variable @x has only zero values over the
+ * Check whether series @x has only zero values over the
  * given sample range (or possibly missing values).
  *
  * Returns: 1 if the variable is all zeros, otherwise 0.
@@ -209,7 +202,7 @@ int gretl_iszero (int t1, int t2, const double *x)
  * @t1: starting observation.
  * @t2: ending observation. 
  * 
- * Check whether variable @x is constant over the
+ * Check whether series @x is constant over the
  * given sample range (aside from any missing values).
  *
  * Returns: 1 if the variable is constant, otherwise 0.
@@ -242,7 +235,7 @@ int gretl_isconst (int t1, int t2, const double *x)
  * @t1: starting observation.
  * @t2: ending observation. 
  * 
- * Check whether variable @x equals 1 over the
+ * Check whether series @x equals 1 over the
  * given sample range (aside from any missing values).
  *
  * Returns: 1 if so, otherwise 0.
@@ -268,8 +261,8 @@ int gretl_isunits (int t1, int t2, const double *x)
  * @t1: starting observation.
  * @t2: ending observation. 
  * 
- * Check whether variable @x contains only integer values over the
- * given sample range (aside from any missing values).
+ * Check whether series @x contains only integer values over
+ * the given sample range (aside from any missing values).
  *
  * Returns: 1 if so, otherwise 0.
  */
@@ -288,6 +281,50 @@ int gretl_isint (int t1, int t2, const double *x)
     return ret;
 }
 
+/**
+ * gretl_iscount:
+ * @x: data series to examine.
+ * @t1: starting observation.
+ * @t2: ending observation. 
+ * 
+ * Check whether series @x contains nothing but non-negative
+ * integer values (some of which are > 1) over the 
+ * given sample range.
+ *
+ * Returns: 1 if so, otherwise 0.
+ */
+
+int gretl_iscount (int t1, int t2, const double *x)
+{
+    int t, xi;
+    int g1 = 0;
+    int ret = 1;
+
+    for (t=t1; t<=t2; t++) {
+	if (na(x[t])) {
+	    continue;
+	}
+	if (x[t] < 0.0) {
+	    ret = 0;
+	    break;
+	}
+	xi = x[t];
+	if (x[t] != (double) xi) {
+	    ret = 0;
+	    break;
+	}
+	if (x[t] > 1.0) {
+	    g1 = 1;
+	}
+    }
+
+    if (g1 == 0) {
+	ret = 0;
+    }
+
+    return ret;
+}
+
 #define FEWVALS 32
 
 static int few_vals (int t1, int t2, const double *x, double *ratio)
@@ -684,37 +721,6 @@ int ls_criteria (MODEL *pmod)
     return err;
 }
 
-/**
- * gretl_print_criteria:
- * @ess: error sum of squares.
- * @nobs: number of observations.
- * @ncoeff: number of parameters estimated.
- * @prn: printing struct.
- *
- * Prints the values of the model selection criteria AIC, 
- * BIC and HQC for the given parameters.
- *
- * Returns: 0 on success, non-zero on error.
- */
-
-int gretl_print_criteria (double ess, int nobs, int ncoeff, PRN *prn)
-{
-    double ll, aic, bic, hqc;
-    int err;
-
-    err = gretl_calculate_criteria(ess, nobs, ncoeff, &ll, &aic, &bic, &hqc);
-
-    if (err) {
-	pputs(prn, _("Error calculating model selection criteria\n"));
-    } else {
-	pprintf(prn, _("Using ess = %g, %d observations, %d coefficients\n"), 
-		ess, nobs, ncoeff);
-	pprintf(prn, "\nAIC = %g\nBIC = %g\nHQC = %g\n\n", aic, bic, hqc);
-    }
-
-    return err;
-}
-
 static char *
 real_format_obs (char *obs, int maj, int min, int pd, char sep)
 {
@@ -1234,9 +1240,9 @@ double *copyvec (const double *src, int n)
 
 void doubles_array_free (double **X, int m)
 {
-    int i;
-
     if (X != NULL) {
+	int i;
+
 	for (i=0; i<m; i++) {
 	    free(X[i]);
 	}
@@ -1250,7 +1256,8 @@ void doubles_array_free (double **X, int m)
  * @n: length of each sub-array.
  *
  * Allocates a 2-dimensional array of doubles, that is,
- * @m arrays each containing @n elements.
+ * @m arrays each containing @n elements.  If @n is
+ * zero the sub-arrays are just set to %NULL.
  * 
  * Returns: the allocated array, or %NULL on failure.
  */
@@ -1287,6 +1294,75 @@ double **doubles_array_new (int m, int n)
     return X;
 }
 
+/**
+ * doubles_array_new0:
+ * @m: number of sub-arrays.
+ * @n: length of each sub-array.
+ *
+ * Works just as doubles_array_new(), except that on
+ * successful allocation all values in the arrays are
+ * set to zero.
+ * 
+ * Returns: the allocated array, or %NULL on failure.
+ */
+
+double **doubles_array_new0 (int m, int n)
+{
+    double **X = doubles_array_new(m, n);
+
+    if (X != NULL && n > 0) {
+	int i, j;
+
+	for (i=0; i<m; i++) {
+	    for (j=0; j<n; j++) {
+		X[i][j] = 0.0;
+	    }
+	}
+    }
+
+    return X;
+}
+
+/**
+ * doubles_array_adjust_length:
+ * @X: original two-dimensional array.
+ * @m: number of sub-arrays in @X.
+ * @new_n: new length of each sub-array.
+ * 
+ * For each of the @m sub-arrays in @X, reallocate to
+ * a length of @new_n.
+ * 
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int doubles_array_adjust_length (double **X, int m, int new_n)
+{
+    int err = 0;
+
+    if (X == NULL || m == 0) {
+	; /* no-op */
+    } else {
+	double *xi;
+	int i;
+
+	for (i=0; i<m && !err; i++) {
+	    if (new_n == 0) {
+		free(X[i]);
+		X[i] = NULL;
+	    } else {
+		xi = realloc(X[i], new_n * sizeof *xi);
+		if (xi == NULL) {
+		    err = E_ALLOC;
+		} else {
+		    X[i] = xi;
+		}
+	    }
+	}
+    }
+
+    return err;
+}
+
 /**
  * data_array_from_model:
  * @pmod: reference model.
@@ -1484,11 +1560,33 @@ int gretl_delete_var_by_name (const char *s, PRN *prn)
     return err;
 }
 
-/* internal execution timer: on Windows use use GetTickCount,
-   else use times() if available, otherwise fall back on clock() 
+/* internal execution timer: on OpenMP use omp_get_wtime(); else
+   on Windows use use GetTickCount; else use times() if available,
+   otherwise fall back on clock()
 */ 
 
-#ifdef WIN32
+#if defined(_OPENMP)
+
+#include <omp.h>
+
+static double tim0;
+
+static void gretl_stopwatch_init (void)
+{
+    tim0 = omp_get_wtime();
+}
+
+double gretl_stopwatch (void)
+{
+    double tim1 = omp_get_wtime();
+    double x = tim1 - tim0;
+
+    tim0 = tim1;
+
+    return x;
+} 
+
+#elif defined(WIN32)
 
 #include <windows.h>
 
@@ -1510,7 +1608,7 @@ double gretl_stopwatch (void)
     return x;
 } 
 
-#else /* !WIN32 */
+#else /* !OPENMP, !WIN32 */
 
 static clock_t tim0;
 
@@ -1554,7 +1652,7 @@ double gretl_stopwatch (void)
     return x;
 } 
 
-#endif /* WIN32 or not */
+#endif /* stopwatch variants */
 
 /* library init and cleanup functions */
 
@@ -1580,8 +1678,6 @@ void libgretl_session_cleanup (int mode)
 
 void libgretl_cleanup (void)
 {
-    const char *p;
-
     libgretl_session_cleanup(SESSION_CLEAR_FULL);
 
     gretl_rand_free();
@@ -1594,18 +1690,11 @@ void libgretl_cleanup (void)
     forecast_matrix_cleanup();
     option_flags_cleanup();
     kalman_cleanup();
+    gnuplot_cleanup();
+
 #ifdef USE_RLIB
     gretl_R_cleanup();
 #endif
-
-    p = strstr(gretl_plotfile(), "gpttmp");
-    if (p != NULL) {
-	int pnum;
-
-	if (!sscanf(p, "gpttmp%d.plt", &pnum)) {
-	    gretl_remove(gretl_plotfile());
-	}
-    }
 }
 
 /* record and retrieve hypothesis test results */
