@@ -826,6 +826,31 @@ static int get_contents (int fd, FILE *fp, char **getbuf, long *len,
     return res;
 }
 
+const char *partsep = "AaB03x";
+
+static int write_multipart_trailer (int sock)
+{
+    char buf[16];
+
+    sprintf(buf, "--%s--\r\n", partsep);
+    return iwrite(sock, buf, strlen(buf));
+}
+
+static char *make_posthead (urlinfo *u)
+{
+    char *head = malloc(96);
+
+    if (head != NULL) {
+	int traillen = strlen(partsep) + 6;
+
+	sprintf(head, "Content-Type: multipart/form-data, boundary=%s\r\n"
+		"Content-Length: %d\r\n", 
+		partsep, strlen(u->params) + u->upsize + traillen);
+    }
+
+    return head;
+}
+
 static uerr_t real_get_http (urlinfo *u, struct http_stat *hs, int *dt)
 {
     char *request, *type, *cmd;
@@ -897,14 +922,11 @@ static uerr_t real_get_http (urlinfo *u, struct http_stat *hs, int *dt)
 #endif
 
     if (u->upload != NULL) {
-	posthead = malloc(96);
+	posthead = make_posthead(u);
 	if (posthead == NULL) {
 	    close(sock);
 	    return NOTENOUGHMEM;
 	} else {
-	    sprintf(posthead, "Content-Type: "
-		    "application/x-www-form-urlencoded\r\n"
-		    "Content-Length: %d\r\n", strlen(u->params) + u->upsize);
 	    rlen += strlen(posthead);
 	}
     } else if (u->params != NULL) {
@@ -940,7 +962,7 @@ static uerr_t real_get_http (urlinfo *u, struct http_stat *hs, int *dt)
     strcat(request, "\r\n");
 
 #if WDEBUG > 1
-    fprintf(stderr, "---request begin---\n%s---request end---", request);
+    fprintf(stderr, "---request begin---\n%s---request end---\n", request);
 #endif
 
     /* Send the request to server */
@@ -961,11 +983,17 @@ static uerr_t real_get_http (urlinfo *u, struct http_stat *hs, int *dt)
     }
 
     if (u->upload != NULL) {
-	/* use POST for params and file content */
+	/* we're using POST */
+#if WDEBUG > 1
+	fprintf(stderr, "params:\n'%s'\n", u->params);
+#endif
 	num_written = iwrite(sock, u->params, strlen(u->params));
 	if (num_written > 0) {
 	    num_written = iwrite(sock, u->upload, u->upsize);
 	}
+	if (num_written > 0) {
+	    num_written = write_multipart_trailer(sock);
+	}
 	if (num_written < 0) {
 	    close(sock);
 	    return WRITEFAILED;
@@ -1188,7 +1216,6 @@ static uerr_t try_http (urlinfo *u)
 #endif
 
     for (i=0; i<MAXTRY; i++) {
-
 	err = real_get_http(u, &hstat, &dt);
 
 #if WDEBUG
@@ -1844,23 +1871,29 @@ static int
 urlinfo_set_upload_params (urlinfo *u, const char *login, const char *pass,
 			   const char *fname)
 {
-    int len = strlen(login) + strlen(pass) + strlen(fname) + 40;
+    int len = strlen(login) + strlen(pass) + strlen(fname);
 
-    u->params = malloc(len);
+    u->params = malloc(len + 256);
     if (u->params == NULL) {
 	return 1;
-    }
+    }    
 
-    sprintf(u->params, "opt=UPLOAD&login=%s&pass=%s"
-	    "&fname=%s&content=", login, pass, fname);
+    sprintf(u->params, "--%s\r\ncontent-disposition: form-data; name=\"login\"\r\n"
+	    "\r\n%s\r\n" /* username */
+	    "--%s\r\ncontent-disposition: form-data; name=\"pass\"\r\n"
+	    "\r\n%s\r\n" /* password */
+	    "--%s\r\ncontent-disposition: form-data; name=\"pkg\"; "
+	    "filename=\"%s\"\r\n" /* filename */
+	    "Content-Type: text/plain; charset=utf-8\r\n\r\n",
+	    partsep, login, partsep, pass, partsep, fname);
 
     return 0;
 }
 
-/* The content of the function package to be uploaded is URL-encoded
-   in 'buf'; the (short, pathless) filename for this package is in
-   'fname'.  If 'retbuf' is non-NULL it gets a copy of the response
-   from the server.
+/* The content of the function package to be uploaded is in 'buf';
+   the (short, pathless) filename for this package is in 'fname'.
+   If 'retbuf' is non-NULL it gets a copy of the response from the
+   server.
 */
 
 int upload_function_package (const char *login, const char *pass, 
@@ -1886,7 +1919,7 @@ int upload_function_package (const char *login, const char *pass,
     }
 
     u->upload = buf;
-    u->upsize = strlen(buf) + 1;
+    u->upsize = strlen(buf);
 
     err = get_host_ip(u, gretlhost);
     if (err) {
