@@ -33,7 +33,9 @@ typedef enum {
     GRAB_FUNC,
     GRAB_PDF,
     CHECK_DB,
-    UPLOAD
+    UPLOAD,
+    LIST_PKGS,
+    GRAB_PKG
 } CGIOpt;
 
 typedef enum {
@@ -47,6 +49,8 @@ int list_remote_dbs (char **getbuf);
 
 int list_remote_function_packages (char **getbuf);
 
+int list_remote_data_packages (char **getbuf);
+
 int retrieve_remote_db_index (const char *dbname, char **getbuf);
 
 int retrieve_remote_db (const char *dbname, 
@@ -58,6 +62,9 @@ int check_remote_db (const char *dbname);
 int retrieve_remote_function_package (const char *pkgname, 
 				      const char *localname);
 
+int retrieve_remote_datafiles_package (const char *pkgname, 
+				       const char *localname);
+
 int retrieve_remote_db_data (const char *dbname,
 			     const char *varname,
 			     char **getbuf,
