@@ -881,22 +881,23 @@ static void *gretl_xml_get_array (xmlNodePtr node, xmlDocPtr doc,
 	    for (i=0; i<n && !*err && *s; i++) {
 		while (isspace(*s)) s++;
 		x = strtod(s, &test);
-		if (errno) {
-		    fprintf(stderr, "strtod failed on '%s'\n", s);
-		    perror(NULL);
-		    *err = E_DATA;
-		} else if (!strncmp(test, "NA", 2)) {
+		if (!strncmp(test, "NA", 2)) {
 		    x = NADBL;
 		    s = test + 2;
-		} else if (*test != '\0' && !isspace(*test)) {
-		    *err = E_DATA;
 		} else {
 		    s = test;
+		    if (*s != '\0' && !isspace(*s)) {
+			*err = E_DATA;
+		    } else if (errno) {
+			perror(NULL);
+			x = (x == 0.0)? 0.0 : NADBL;
+			errno = 0;
+		    }
 		}
 		xvals[i] = x;
 		nread++;
 	    }
-	} if (type == GRETL_TYPE_INT_ARRAY) {
+	} else if (type == GRETL_TYPE_INT_ARRAY) {
 	    long kl;
 
 	    for (i=0; i<n && !*err && *s; i++) {
@@ -1400,6 +1401,7 @@ int gretl_write_matrix_as_gdt (const char *fname,
     long sz = 0L;
     int T = X->rows;
     int k = X->cols;
+    int in_c_locale = 0;
     int i, t, err = 0;
 
     fz = gretl_gzopen(fname, "wb");
@@ -1441,6 +1443,7 @@ int gretl_write_matrix_as_gdt (const char *fname,
     free(xmlbuf);
 
     gretl_push_c_numeric_locale();
+    in_c_locale = 1;
 
     gzprintf(fz, "<variables count=\"%d\">\n", k);
 
@@ -1471,7 +1474,9 @@ int gretl_write_matrix_as_gdt (const char *fname,
 
  cleanup: 
 
-    gretl_pop_c_numeric_locale();
+    if (in_c_locale) {
+	gretl_pop_c_numeric_locale();
+    }
 
     if (sz) {
 	(*show_progress)(0, T, SP_FINISH);
@@ -1517,6 +1522,7 @@ int gretl_write_gdt (const char *fname, const int *list,
     int (*show_progress) (long, long, int) = NULL;
     long sz = 0L;
     int i, t, v, nvars;
+    int have_markers, in_c_locale = 0;
     int uerr = 0;
     int err = 0;
 
@@ -1624,6 +1630,7 @@ int gretl_write_gdt (const char *fname, const int *list,
     }
 
     gretl_push_c_numeric_locale();
+    in_c_locale = 1;
 
     /* then listing of variable names and labels */
     if (gz) {
@@ -1713,15 +1720,16 @@ int gretl_write_gdt (const char *fname, const int *list,
     alt_puts("</variables>\n", fp, fz);
 
     panelobs = query_print_panel_obs(pdinfo);
+    have_markers = dataset_has_markers(pdinfo);
 
     /* then listing of observations */
     alt_puts("<observations ", fp, fz);
     if (gz) {
 	gzprintf(fz, "count=\"%d\" labels=\"%s\"",
-		tsamp, (pdinfo->markers && pdinfo->S != NULL)? "true" : "false");
+		 tsamp, (have_markers)? "true" : "false");
     } else {
 	fprintf(fp, "count=\"%d\" labels=\"%s\"",
-		tsamp, (pdinfo->markers && pdinfo->S != NULL)? "true" : "false");
+		tsamp, (have_markers)? "true" : "false");
     }
     if (panelobs) {
 	alt_puts(" panel-info=\"true\"", fp, fz);
@@ -1730,7 +1738,7 @@ int gretl_write_gdt (const char *fname, const int *list,
 
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	alt_puts("<obs", fp, fz);
-	if (pdinfo->markers && pdinfo->S != NULL) {
+	if (have_markers) {
 	    uerr = gretl_xml_encode_to_buf(xmlbuf, pdinfo->S[t], sizeof xmlbuf);
 	    if (!uerr) {
 		if (gz) {
@@ -1775,7 +1783,9 @@ int gretl_write_gdt (const char *fname, const int *list,
 
  cleanup: 
 
-    gretl_pop_c_numeric_locale();
+    if (in_c_locale) {
+	gretl_pop_c_numeric_locale();
+    }
 
     if (sz) {
 	(*show_progress)(0, pdinfo->t2 - pdinfo->t1 + 1, SP_FINISH);
@@ -1800,13 +1810,11 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 {
     xmlNodePtr cur;
     xmlChar *tmp = xmlGetProp(node, (XUC) "count");
-    int i, err = 0;
+    int i, nv = 0, err = 0;
 
     if (tmp != NULL) {
-	int v;
-
-	if (sscanf((char *) tmp, "%d", &v) == 1) {
-	    pdinfo->v = v + 1;
+	if (sscanf((char *) tmp, "%d", &nv) == 1) {
+	    pdinfo->v = nv + 1;
 	} else {
 	    gretl_errmsg_set(_("Failed to parse count of variables"));
 	    err = 1;
@@ -1826,7 +1834,12 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 	err = 1;
     }
 
-    if (err) return 1;
+    if (err) {
+	return 1;
+    } else if (nv == 0) {
+	fprintf(stderr, "Empty dataset!\n");
+	return 0;
+    }
 
     /* now get individual variable info: names and labels */
     cur = node->xmlChildrenNode;
@@ -1834,7 +1847,7 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 	cur = cur->next;
     }
 
-    if (cur == 0) {
+    if (cur == NULL) {
 	gretl_errmsg_set(_("Got no variables"));
 	return 1;
     }
@@ -2098,10 +2111,12 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
 		}
 		free(tmp);
 		t++;
-	    } else {
+	    } else if (pdinfo->v > 1) {
 		gretl_errmsg_sprintf(_("Values missing at observation %d"), t+1);
 		err = E_DATA;
 		goto bailout;
+	    } else {
+		t++;
 	    }
 	}	   
  
@@ -2337,7 +2352,7 @@ static void record_transform_info (double **Z, DATAINFO *pdinfo, double version)
 static void data_read_message (const char *fname, DATAINFO *pdinfo, PRN *prn)
 {
     pprintf(prn, M_("\nRead datafile %s\n"), fname);
-    pprintf(prn, M_("periodicity: %d, maxobs: %d,\n"
+    pprintf(prn, M_("periodicity: %d, maxobs: %d\n"
 		    "observations range: %s-%s\n"), 
 	    (custom_time_series(pdinfo))? 1 : pdinfo->pd, 
 	    pdinfo->n, pdinfo->stobs, pdinfo->endobs);
@@ -2354,6 +2369,23 @@ static long get_filesize (const char *fname)
     return (err)? -1 : buf.st_size;
 }
 
+static int remedy_empty_data (double ***pZ, DATAINFO *pdinfo)
+{
+    int err = dataset_add_series(1, pZ, pdinfo);
+
+    if (!err) {
+	int t;
+
+	strcpy(pdinfo->varname[1], "index");
+	strcpy(VARLABEL(pdinfo, 1), _("index variable"));
+	for (t=0; t<pdinfo->n; t++) {
+	    (*pZ)[1][t] = (double) (t + 1);
+	}
+    }
+
+    return err;
+}
+
 /**
  * gretl_read_gdt:
  * @fname: name of file to try.
@@ -2380,6 +2412,7 @@ int gretl_read_gdt (char *fname, double ***pZ, DATAINFO *pdinfo,
     int caldata = 0;
     double gdtversion = 1.0;
     long fsz, progress = 0L;
+    int in_c_locale = 0;
 
     gretl_error_clear();
 
@@ -2443,13 +2476,13 @@ int gretl_read_gdt (char *fname, double ***pZ, DATAINFO *pdinfo,
     }   
 
     gretl_push_c_numeric_locale();
+    in_c_locale = 1;
 
     strcpy(tmpdinfo->stobs, "1");
     caldata = dataset_is_daily(tmpdinfo) || dataset_is_weekly(tmpdinfo);
 
     err = xml_get_startobs(cur, &tmpdinfo->sd0, tmpdinfo->stobs, caldata);
     if (err) {
-	gretl_pop_c_numeric_locale();
 	goto bailout;
     }     
 
@@ -2458,7 +2491,6 @@ int gretl_read_gdt (char *fname, double ***pZ, DATAINFO *pdinfo,
 
     err = xml_get_endobs(cur, tmpdinfo->endobs, caldata);
     if (err) {
-	gretl_pop_c_numeric_locale();
 	goto bailout;
     }     
 
@@ -2498,30 +2530,27 @@ int gretl_read_gdt (char *fname, double ***pZ, DATAINFO *pdinfo,
     fprintf(stderr, "done walking XML tree...\n");
 #endif
 
-    gretl_pop_c_numeric_locale();
-
-    if (err) {
-	goto bailout;
-    }
-
-    if (!gotvars) {
+    if (!err && !gotvars) {
 	gretl_errmsg_set(_("Variables information is missing"));
 	err = 1;
-	goto bailout;
     }
 
-    if (!gotobs) {
+    if (!err && !gotobs) {
 	gretl_errmsg_set(_("No observations were found"));
 	err = 1;
-	goto bailout;
     }
 
-    data_read_message(fname, tmpdinfo, prn);
-
-    err = merge_or_replace_data(pZ, pdinfo, &tmpZ, &tmpdinfo, opt, prn);
+    if (!err) {
+	data_read_message(fname, tmpdinfo, prn);
+	err = merge_or_replace_data(pZ, pdinfo, &tmpZ, &tmpdinfo, opt, prn);
+    }
 
  bailout:
 
+    if (in_c_locale) {
+	gretl_pop_c_numeric_locale();
+    }
+
     if (doc != NULL) {
 	xmlFreeDoc(doc);
 	xmlCleanupParser();
@@ -2542,6 +2571,10 @@ int gretl_read_gdt (char *fname, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
+    if (!err && pdinfo->v == 1) {
+	err = remedy_empty_data(pZ, pdinfo);
+    }
+
     if (!err && gdtversion < 1.2) {
 	record_transform_info(*pZ, pdinfo, gdtversion);
     }
