@@ -75,12 +75,14 @@ void gretl_cmd_set_context (CMD *cmd, int ci);
 
 void gretl_cmd_destroy_context (CMD *cmd);
 
-char *gretl_cmd_get_savename (char *sname);
+const char *gretl_cmd_get_savename (CMD *cmd);
 
 gretlopt gretl_cmd_get_opt (const CMD *cmd);
 
 void gretl_cmd_set_opt (CMD *cmd, gretlopt opt);
 
+int filter_comments (char *s, CMD *cmd);
+
 int parse_command_line (char *line, CMD *cmd, double ***pZ, 
 			DATAINFO *pdinfo); 
 
