@@ -130,6 +130,8 @@ struct kalman_ {
     gretl_matrix *Tmprr_2a;
     gretl_matrix *Tmprr_2b;
     gretl_matrix *Tmpr1;
+
+    PRN *prn; /* verbose printer */
 };
 
 #define NMATCALLS 5  /* max number of time-varying matrices */
@@ -279,6 +281,7 @@ static kalman *kalman_new_empty (int flags)
 	K->flags = flags;
 	K->fnlevel = 0;
 	K->t = 0;
+	K->prn = NULL;
     }
 
     return K;
@@ -2158,7 +2161,8 @@ static int user_kalman_recheck_matrices (user_kalman *u)
 
 /* Try to find a user-defined Kalman filter at the current level of
    function execution.  Failing that, if @level is given as -1, we
-   also try looking at the parent level.
+   also try looking for a filter available at parent levels of 
+   execution. 
 */
 
 static user_kalman *get_user_kalman (int level)
@@ -2179,11 +2183,12 @@ static user_kalman *get_user_kalman (int level)
     }
 
     if (go_up) {
-	level--;
-	for (i=0; i<n_uK; i++) {
-	    if (uK[i] != NULL && uK[i]->fnlevel == level) {
-		return uK[i];
-	    } 
+	while (--level >= 0) {
+	    for (i=0; i<n_uK; i++) {
+		if (uK[i] != NULL && uK[i]->fnlevel == level) {
+		    return uK[i];
+		} 
+	    }
 	}
     }
 
@@ -2199,7 +2204,7 @@ static int real_destroy_user_kalman (PRN *prn)
 	    kalman_free(uK[i]->K);
 	    free(uK[i]);
 	    uK[i] = NULL;
-	    if (gretl_messages_on()) {
+	    if (prn != NULL && gretl_messages_on()) {
 		pputs(prn, "Deleted kalman filter\n");
 	    }
 	    return 0;
@@ -2234,6 +2239,18 @@ int delete_kalman (PRN *prn)
     return real_destroy_user_kalman(prn);
 }
 
+void kalman_attach_printer (kalman *K, PRN *prn)
+{
+    if (K != NULL) {
+	K->prn = prn;
+    }
+}
+
+PRN *kalman_get_printer (const kalman *K)
+{
+    return (K != NULL)? K->prn : NULL;
+}
+
 static int add_user_kalman (void)
 {
     user_kalman *u = NULL;
@@ -2334,8 +2351,8 @@ static const char *kalman_matrix_name (int sym)
  * @Z: data array (may be %NULL).
  * @pdinfo: dataset information (may be %NULL).
  * @opt: may contain %OPT_D for diffuse initialization of the
- * Kalman filter; also may contain %OPT_C to specify that the
- * disturbances are correlated across the two equations.
+ * Kalman filter, %OPT_C to specify that the disturbances are 
+ * correlated across the two equations.
  *
  * Parses @line and either (a) starts a filter definition or
  * (b) adds a matrix specification to the filter or (c)
