@@ -56,7 +56,10 @@ enum {
     STATE_VERBOSE_INCLUDE = 1 << 14, /* verbose include */
     STATE_SKIP_MISSING    = 1 << 15, /* skip NAs when building matrix from series */
     STATE_LOOPING         = 1 << 16, /* loop is in progress at this level */
-    STATE_LOOP_QUIET      = 1 << 17  /* loop commands should be quiet */
+    STATE_LOOP_QUIET      = 1 << 17, /* loop commands should be quiet */
+    STATE_LOOP_PROG       = 1 << 18, /* progressive loop is in progress */
+    STATE_BFGS_RSTEP      = 1 << 19  /* use Ricardson method in BFGS numerical
+					gradient */
 };    
 
 /* for values that really want a non-negative integer */
@@ -85,6 +88,7 @@ struct set_vars_ {
     int vecm_norm;              /* VECM beta normalization */
     int bfgs_maxiter;           /* max iterations, BFGS */         
     double bfgs_toler;          /* convergence tolerance, BFGS */
+    double bfgs_maxgrad;        /* max acceptable gradient norm, BFGS */
     int bfgs_verbskip;          /* BFGS: show one in n iterations  */
     int bhhh_maxiter;           /* max iterations, BHHH */          
     double bhhh_toler;          /* convergence tolerance, BHHH */
@@ -125,9 +129,11 @@ struct set_vars_ {
                            !strcmp(s, VERBOSE_INCLUDE) || \
                            !strcmp(s, SKIP_MISSING) || \
 			   !strcmp(s, R_FUNCTIONS) || \
-			   !strcmp(s, R_LIB))
+			   !strcmp(s, R_LIB) || \
+			   !strcmp(s, BFGS_RSTEP))
 
 #define libset_double(s) (!strcmp(s, BFGS_TOLER) || \
+			  !strcmp(s, BFGS_MAXGRAD) || \
 			  !strcmp(s, BHHH_TOLER) || \
 			  !strcmp(s, HP_LAMBDA) || \
 			  !strcmp(s, NLS_TOLER) || \
@@ -222,6 +228,12 @@ static const char *vecm_norm_strs[] = {
     NULL
 };
 
+static const char *normal_rand_strs[] = {
+    "ziggurat",
+    "box-muller",
+    NULL
+};
+
 static const char **libset_option_strings (const char *s)
 {
     if (!strcmp(s, GARCH_VCV)) {
@@ -234,6 +246,8 @@ static const char **libset_option_strings (const char *s)
 	return hc_version_strs;
     } else if (!strcmp(s, VECM_NORM)) {
 	return vecm_norm_strs;
+    } else if (!strcmp(s, NORMAL_RAND)) {
+	return normal_rand_strs;
     } else if (!strcmp(s, "csv_delim")) {
 	return csv_delim_args;
     } else {
@@ -280,6 +294,8 @@ static const char *libset_option_string (const char *s)
 	return hc_version_strs[state->ropts.hc_version];
     } else if (!strcmp(s, VECM_NORM)) {
 	return vecm_norm_strs[state->vecm_norm];
+    } else if (!strcmp(s, NORMAL_RAND)) {
+	return normal_rand_strs[gretl_rand_get_box_muller()];
     } else {
 	return "?";
     }
@@ -343,6 +359,7 @@ static void state_vars_copy (set_vars *sv)
     sv->vecm_norm = state->vecm_norm;
     sv->bfgs_maxiter = state->bfgs_maxiter;
     sv->bfgs_toler = state->bfgs_toler;
+    sv->bfgs_maxgrad = state->bfgs_maxgrad;
     sv->bfgs_verbskip = state->bfgs_verbskip;
     sv->bhhh_maxiter = state->bhhh_maxiter;
     sv->bhhh_toler = state->bhhh_toler;
@@ -380,6 +397,7 @@ static void state_vars_init (set_vars *sv)
 
     sv->bfgs_maxiter = -1;
     sv->bfgs_toler = NADBL;
+    sv->bfgs_maxgrad = 5.0;
     sv->bfgs_verbskip = 1;
     sv->bhhh_maxiter = 500;
     sv->bhhh_toler = NADBL;
@@ -837,6 +855,13 @@ static int parse_libset_int_code (const char *key,
 		break;
 	    }
 	}
+    } else if (!strcmp(key, NORMAL_RAND)) {
+	for (i=0; normal_rand_strs[i] != NULL; i++) {
+	    if (!strcmp(val, normal_rand_strs[i])) {
+		gretl_rand_set_box_muller(i);
+		err = 0;
+	    }
+	}
     }
 
     if (err) {
@@ -1177,7 +1202,8 @@ static void libset_print_bool (const char *s, PRN *prn,
 			 !strcmp(s, HAC_LAG) || \
 			 !strcmp(s, HAC_KERNEL) || \
                          !strcmp(s, HC_VERSION) || \
-			 !strcmp(s, VECM_NORM))
+			 !strcmp(s, VECM_NORM) || \
+			 !strcmp(s, NORMAL_RAND))
 
 const char *intvar_code_string (const char *s)
 {
@@ -1228,30 +1254,6 @@ static void libset_print_double (const char *s, PRN *prn,
     }
 }
 
-static const char *normal_rand_string (void)
-{
-    if (gretl_rand_get_box_muller()) {
-	return "box-muller";
-    } else {
-	return "ziggurat";
-    }
-}
-
-static int set_normal_rand (const char *s)
-{
-    int err = E_PARSE;
-
-    if (!strcmp(s, "box-muller")) {
-	gretl_rand_set_box_muller(1);
-	err = 0;
-    } else if (!strcmp(s, "ziggurat")) {
-	gretl_rand_set_box_muller(0);
-	err = 0;
-    }    
-
-    return err;
-}
-
 static void libset_header (char *s, PRN *prn, gretlopt opt) 
 {
     if (opt & OPT_D) {
@@ -1330,10 +1332,12 @@ static int print_settings (PRN *prn, gretlopt opt)
 
     libset_print_int(BFGS_MAXITER, prn, opt);
     libset_print_double(BFGS_TOLER, prn, opt);
+    libset_print_double(BFGS_MAXGRAD, prn, opt);
     libset_print_int(BHHH_MAXITER, prn, opt);
     libset_print_double(BHHH_TOLER, prn, opt);
     libset_print_int(RQ_MAXITER, prn, opt);
     print_initvals(state->initvals, prn, opt);
+    libset_print_bool(BFGS_RSTEP, prn, opt);
     libset_print_bool(USE_LBFGS, prn, opt);
     libset_print_int(LBFGS_MEM, prn, opt);
     libset_print_double(NLS_TOLER, prn, opt);
@@ -1344,10 +1348,10 @@ static int print_settings (PRN *prn, gretlopt opt)
 
     if (opt & OPT_D) {
 	pprintf(prn, " seed = %u\n", gretl_rand_get_seed());
-	pprintf(prn, " normal_rand = %s\n", normal_rand_string());
+	pprintf(prn, " normal_rand = %s\n", libset_option_string(NORMAL_RAND));
     } else {
 	pprintf(prn, "set seed %u\n", gretl_rand_get_seed());
-	pprintf(prn, "set normal_rand %s\n", normal_rand_string());
+	pprintf(prn, "set normal_rand %s\n", libset_option_string(NORMAL_RAND));
     }
 
     libset_header(N_("Robust estimation"), prn, opt);
@@ -1393,8 +1397,7 @@ static int print_settings (PRN *prn, gretlopt opt)
     return 0;
 }
 
-static int 
-libset_query_settings (const char *s, PRN *prn)
+static int libset_query_settings (const char *s, PRN *prn)
 {
     int err = 0;
 
@@ -1439,6 +1442,19 @@ libset_query_settings (const char *s, PRN *prn)
     } else if (!strcmp(s, "shelldir")) {
 	pprintf(prn, "%s: string, currently \"%s\"\n", s,
 		state->shelldir);
+    } else if (!strcmp(s, "workdir")) {
+	pprintf(prn, "%s: string, currently \"%s\"\n", s,
+		gretl_workdir());
+    } else if (!strcmp(s, "include_path")) {
+	pprintf(prn, "%s: string, currently \"%s\"\n", s,
+		include_path);
+    } else if (!strcmp(s, "csv_na")) {
+	pprintf(prn, "%s: string, currently \"%s\"\n", s,
+		state->csv_na);
+    } else if (!strcmp(s, "stopwatch")) {
+	err = 0;
+    } else if (!strcmp(s, "bkbp_limits")) {
+	err = 0;
     } else {
 	err = 1;
     }
@@ -1446,6 +1462,13 @@ libset_query_settings (const char *s, PRN *prn)
     return err;
 }
 
+int is_libset_var (const char *s)
+{
+    int err = libset_query_settings(s, NULL);
+
+    return (err == 0);
+}
+
 #define default_ok(s) (!strcmp(s, BFGS_TOLER) || \
                        !strcmp(s, BHHH_TOLER) || \
                        !strcmp(s, HP_LAMBDA))
@@ -1568,8 +1591,6 @@ int execute_set_line (const char *line, DATAINFO *pdinfo,
 		}
 		state->seed = k;
 	    }
-	} else if (!strcmp(setobj, "normal_rand")) {
-	    err = set_normal_rand(setarg);
 	} else if (!strcmp(setobj, HORIZON)) {
 	    /* horizon for VAR impulse responses */
 	    if (!strcmp(setarg, "auto")) {
@@ -1637,6 +1658,8 @@ double libset_get_double (const char *key)
 	} else {
 	    return state->bfgs_toler;
 	}
+    } else if (!strcmp(key, BFGS_MAXGRAD)) {
+	return state->bfgs_maxgrad;
     } else if (!strcmp(key, HP_LAMBDA)) {
 	return state->hp_lambda;
     } else {
@@ -1654,6 +1677,8 @@ double libset_get_user_tolerance (const char *key)
 	return state->bhhh_toler;
     } else if (!strcmp(key, BFGS_TOLER)) {
 	return state->bfgs_toler;
+    } else if (!strcmp(key, BFGS_MAXGRAD)) {
+	return state->bfgs_maxgrad;
     } else {
 	return NADBL;
     }
@@ -1680,6 +1705,8 @@ int libset_set_double (const char *key, double val)
 	state->bhhh_toler = val;
     } else if (!strcmp(key, BFGS_TOLER)) {
 	state->bfgs_toler = val;
+    } else if (!strcmp(key, BFGS_MAXGRAD)) {
+	state->bfgs_maxgrad = val;
     } else if (!strcmp(key, HP_LAMBDA)) {
 	state->hp_lambda = val;
     } else {
@@ -1857,6 +1884,8 @@ static int boolvar_get_flag (const char *s)
 	return STATE_VERBOSE_INCLUDE;
     } else if (!strcmp(s, SKIP_MISSING)) {
 	return STATE_SKIP_MISSING;
+    } else if (!strcmp(s, BFGS_RSTEP)) {
+	return STATE_BFGS_RSTEP;
     } else {
 	fprintf(stderr, "libset_get_bool: unrecognized "
 		"variable '%s'\n", s);	
@@ -2145,12 +2174,15 @@ void libset_cleanup (void)
 static int batch_mode;
 static int gui_mode;
 
-void set_loop_on (int quiet)
+void set_loop_on (int quiet, int progressive)
 {
     state->flags |= STATE_LOOPING;
     if (quiet) {
 	state->flags |= STATE_LOOP_QUIET;
     }
+    if (progressive) {
+	state->flags |= STATE_LOOP_PROG;
+    }    
 }
 
 void set_loop_off (void)
@@ -2167,6 +2199,15 @@ void set_loop_off (void)
 	    state->flags ^= STATE_LOOP_QUIET;
 	}
     }
+    
+    /* and similarly for progressiveness */
+    if (state->flags & STATE_LOOP_PROG) {
+	int i = n_states - 1;
+
+	if (i <= 0 || !(state_stack[i-1]->flags & STATE_LOOP_PROG)) {
+	    state->flags ^= STATE_LOOP_PROG;
+	}
+    }    
 }
 
 /* returns 1 if there's a loop going on anywhere in the "caller
@@ -2200,6 +2241,11 @@ int gretl_looping_quietly (void)
     return (state->flags & STATE_LOOP_QUIET)? 1 : 0;
 }
 
+int gretl_looping_progressive (void)
+{
+    return (state->flags & STATE_LOOP_PROG)? 1 : 0;
+}
+
 void gretl_set_batch_mode (int b)
 {
     batch_mode = b;
