@@ -32,7 +32,9 @@ typedef enum {
 
 #define BFGS_MAXITER     "bfgs_maxiter"
 #define BFGS_TOLER       "bfgs_toler"
+#define BFGS_MAXGRAD     "bfgs_maxgrad"
 #define BFGS_VERBSKIP    "bfgs_verbskip"
+#define BFGS_RSTEP       "bfgs_richardson"
 #define BHHH_MAXITER     "bhhh_maxiter"
 #define BHHH_TOLER       "bhhh_toler"
 #define LBFGS_MEM        "lbfgs_mem"
@@ -66,6 +68,7 @@ typedef enum {
 #define SKIP_MISSING     "skip_missing"
 #define R_FUNCTIONS      "R_functions"
 #define R_LIB            "R_lib"
+#define NORMAL_RAND      "normal_rand"
 
 typedef int (*ITER_PRINT_FUNC) (int, PRN *);
 typedef int (*DEBUG_READLINE) (void *);
@@ -91,6 +94,8 @@ double libset_get_user_tolerance (const char *key);
 int libset_get_int (const char *key);
 int libset_set_int (const char *key, int val);
 
+int is_libset_var (const char *s);
+
 /* GUI setter functions */
 void set_xsect_hccme (const char *s);
 void set_tseries_hccme (const char *s);
@@ -114,11 +119,12 @@ const gretl_matrix *get_init_vals (void);
 int n_init_vals (void);
 void free_init_vals (void);
 
-void set_loop_on (int quiet);
+void set_loop_on (int quiet, int progressive);
 void set_loop_off (void);
 int gretl_looping (void);
 int gretl_looping_currently (void);
 int gretl_looping_quietly (void);
+int gretl_looping_progressive (void);
 
 void gretl_set_batch_mode (int b);
 int gretl_in_batch_mode (void);
