@@ -24,13 +24,8 @@
 
 #define NEW_NA 0
 
-#if NEW_NA /* not yet! */
-# define NADBL (0.0/0.0)
-# define na(x) isnan(x)
-#else
-# define NADBL DBL_MAX
-# define na(x) ((x) == NADBL)
-#endif
+#define NADBL DBL_MAX
+#define na(x) ((x) == NADBL)
 
 /* xna = "extended NA", including regular NA for missing data
    as well as NaNs and infinities */
