@@ -22,6 +22,8 @@
 #ifndef MODELPRINT_H
 #define MODELPRINT_H
 
+#define MC_NAMELEN 32
+
 typedef struct model_coeff_ model_coeff;
 
 struct model_coeff_ {
@@ -34,12 +36,8 @@ struct model_coeff_ {
     double hi;
     int show_pval;
     int df_pval;
-    char name[32];
-};
-
-enum {
-    COEFF_HEADING_VARNAME,
-    COEFF_HEADING_PARAM
+    int multi;
+    char name[MC_NAMELEN];
 };
 
 int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
@@ -51,13 +49,12 @@ void print_model_vcv_info (const MODEL *pmod, PRN *prn);
 
 int ols_print_anova (const MODEL *pmod, PRN *prn);
 
-void print_coeff_heading (int mode, PRN *prn);
+int print_coeffs (const double *b,
+		  const double *se,
+		  const char **names,
+		  int nc, int df, int ci,
+		  PRN *prn);
 
-void model_coeff_init (model_coeff *mc);
-
-void print_coeff (const model_coeff *mc, PRN *prn);
-
-void print_arch_coeffs (const double *a, const double *se,
-			int T, int order, PRN *prn, int aux);
+int do_modprint (const char *line, gretlopt opt, PRN *prn);
 
 #endif /* MODELPRINT_H */
