@@ -164,6 +164,20 @@ static int nls_dynamic_check (nlspec *s, char *formula)
     return err;
 }
 
+static int scalar_acceptable (nlspec *s, int i, const char *dername)
+{
+    if (i < s->naux) {
+	/* auxiliary variable */
+	return 1;
+    } else if (i == s->naux) {
+	/* criterion value */
+	return s->ci == MLE;
+    } else {
+	/* i > s->naux: derivative */
+	return dername != NULL && gretl_is_scalar(dername);
+    }
+}
+
 /* we "compile" the required equations first, so we can subsequently
    execute the compiled versions for maximum efficiency 
 */
@@ -201,6 +215,7 @@ static int nls_genr_setup (nlspec *s)
 
     for (i=0; i<ngen && !err; i++) {
 	char *dname = NULL;
+	int gentype = 0;
 
 	if (i < s->naux) {
 	    /* auxiliary variables */
@@ -259,15 +274,20 @@ static int nls_genr_setup (nlspec *s)
 	m = genr_get_output_matrix(genrs[i]);
 
 	if (v == 0 && m == NULL) {
-	    /* not a series, not a matrix: should be scalar */
 	    if (genr_is_print(genrs[i])) {
 		continue;
-	    } else if (i >= s->naux && (dname == NULL || !gretl_is_scalar(dname))) {
+	    }
+	    /* not a series, not a matrix: should be scalar */
+	    gentype = genr_get_output_type(genrs[i]);
+	    if (gentype != GRETL_TYPE_DOUBLE) {
+		err = E_TYPES;
+		break;
+	    } else if (!scalar_acceptable(s, i, dname)) {
 		gretl_errmsg_sprintf(_("The formula '%s'\n produced a scalar result"), 
 				     formula);
 		err = E_TYPES;
 		break;
-	    }
+	    } 
 	}
 
 	if (i == s->naux) {
@@ -278,6 +298,8 @@ static int nls_genr_setup (nlspec *s)
 	    } else if (v > 0) {
 		s->lhv = v;
 		s->lhtype = GRETL_TYPE_SERIES;
+	    } else if (gentype == GRETL_TYPE_DOUBLE) {
+		s->lhtype = GRETL_TYPE_DOUBLE;
 	    } else {
 		err = E_TYPES;
 	    }
@@ -749,8 +771,8 @@ static int nl_missval_check (nlspec *s, const DATAINFO *pdinfo)
 	return err;
     }
 
-    if (s->lvec != NULL) {
-	/* vector result */
+    if (s->lvec != NULL || s->lhtype == GRETL_TYPE_DOUBLE) {
+	/* not a series result */
 	goto nl_miss_exit;
     }
 
@@ -830,6 +852,11 @@ static double get_mle_ll (const double *b, void *p)
 	return NADBL;
     }
 
+    if (s->lhtype == GRETL_TYPE_DOUBLE) {
+	s->crit = gretl_scalar_get_value(s->lhname);
+	return s->crit;
+    }
+
     s->crit = 0.0;
 
     if (s->lhtype == GRETL_TYPE_MATRIX) {
@@ -1158,42 +1185,28 @@ static void add_stats_to_model (MODEL *pmod, nlspec *spec,
 
 static int QML_vcv (nlspec *spec, gretl_matrix *V)
 {
-    gretl_matrix *Hinv = NULL;
     gretl_matrix *tmp = NULL;
     int k = V->rows;
-    double x;
-    int i, j;
     
-    Hinv = gretl_matrix_alloc(k, k);
     tmp = gretl_matrix_alloc(k, k);
-
-    if (Hinv == NULL || tmp == NULL) {
-	free(Hinv);
-	free(tmp);
+    if (tmp == NULL) {
 	return E_ALLOC;
     }
 
-    /* expand Hinv */
-    for (i=0; i<k; i++) {
-	for (j=0; j<=i; j++) {
-	    x = spec->hessvec[ijton(i, j, k)];
-	    gretl_matrix_set(Hinv, i, j, x);
-	    gretl_matrix_set(Hinv, j, i, x);
-	}
-    }
-	    
     /* form sandwich: V <- H^{-1} V H^{-1} */
     gretl_matrix_copy_values(tmp, V);
-    gretl_matrix_qform(Hinv, GRETL_MOD_NONE, tmp,
+    gretl_matrix_qform(spec->Hinv, GRETL_MOD_NONE, tmp,
 		       V, GRETL_MOD_NONE);
 
-    gretl_matrix_free(Hinv);
     gretl_matrix_free(tmp);
 
     return 0;
 }
 
-static double *mle_score_callback (const double *b, int i, void *p)
+/* returns the per-observation contributions to the log
+   likelihood */
+
+static const double *mle_llt_callback (const double *b, int i, void *p)
 {
     nlspec *s = (nlspec *) p;
     int err;
@@ -1206,7 +1219,7 @@ static double *mle_score_callback (const double *b, int i, void *p)
     } else if (s->lhtype == GRETL_TYPE_MATRIX) {
 	s->lvec = get_matrix_by_name(s->lhname);
 	if (s->lvec == NULL) {
-	    fprintf(stderr, "mle_score_callback: s->lvec is gone!\n");
+	    fprintf(stderr, "mle_llt_callback: s->lvec is gone!\n");
 	    return NULL;
 	} else {
 	    return s->lvec->val;
@@ -1216,78 +1229,6 @@ static double *mle_score_callback (const double *b, int i, void *p)
     }
 }
 
-#define ALT_OPG 0
-
-/* build the G matrix, given a final set of coefficient
-   estimates, b, and a function for calculating the score
-   vector, scorefun
-*/
-
-gretl_matrix *build_OPG_matrix (double *b, int k, int T,
-				BFGS_SCORE_FUNC scorefun,
-				void *data, int *err)
-{
-    double h = 1e-8;
-#if ALT_OPG
-    double d = 1.0e-4;
-#endif
-    gretl_matrix *G;
-    const double *x;
-    double bi0, x0;
-    int i, t;
-
-    G = gretl_matrix_alloc(k, T);
-    if (G == NULL) {
-	*err = E_ALLOC;
-	return NULL;
-    }
-
-    gretl_matrix_zero(G);
-
-    for (i=0; i<k; i++) {
-	bi0 = b[i];
-#if ALT_OPG
-	h = d * bi0 + d * (b[i] == 0.0);
-#endif
-	b[i] = bi0 - h;
-	x = scorefun(b, i, data);
-	if (x == NULL) {
-	    *err = E_NAN;
-	    goto bailout;
-	}
-	for (t=0; t<T; t++) {
-	    gretl_matrix_set(G, i, t, x[t]);
-	}
-	b[i] = bi0 + h;
-	x = scorefun(b, i, data);
-	if (x == NULL) {
-	    *err = E_NAN;
-	    goto bailout;
-	}
-	for (t=0; t<T; t++) {
-	    x0 = gretl_matrix_get(G, i, t);
-	    gretl_matrix_set(G, i, t, (x[t] - x0) / (2.0 * h));
-	}
-	b[i] = bi0;
-#if NLS_DEBUG
-	fprintf(stderr, "b[%d]: using %#.12g and %#.12g\n", i, bi0 - h, bi0 + h);
-#endif
-    }
-
-#if NLS_DEBUG
-    gretl_matrix_print(G, "Numerically estimated score");
-#endif
-
- bailout:
-
-    if (*err) {
-	gretl_matrix_free(G);
-	G = NULL;
-    }
-
-    return G;
-}
-
 /* add variance matrix based on OPG, (GG')^{-1}, or on QML sandwich,
    H^{-1} GG' H^{-1}
 */
@@ -1309,8 +1250,8 @@ static int mle_build_vcv (MODEL *pmod, nlspec *spec, int *vcvopt)
     }
 
     if (numeric_mode(spec)) {
-	G = build_OPG_matrix(spec->coeff, k, T, mle_score_callback, 
-			     (void *) spec, &err);
+	G = build_score_matrix(spec->coeff, k, T, mle_llt_callback, 
+			       (void *) spec, &err);
 	if (err) {
 	    gretl_matrix_free(V);
 	    return err;
@@ -1356,7 +1297,7 @@ static int mle_build_vcv (MODEL *pmod, nlspec *spec, int *vcvopt)
 			      G, GRETL_MOD_TRANSPOSE,
 			      V, GRETL_MOD_NONE);
 
-    if ((spec->opt & OPT_R) && spec->hessvec != NULL) {
+    if ((spec->opt & OPT_R) && spec->Hinv != NULL) {
 	/* robust option -> QML */
 	err = QML_vcv(spec, V);
 	*vcvopt = VCV_QML;
@@ -1381,29 +1322,24 @@ static int mle_build_vcv (MODEL *pmod, nlspec *spec, int *vcvopt)
 
 static int mle_add_vcv (MODEL *pmod, nlspec *spec)
 {
-    int i, k = spec->ncoeff;
     int vcvopt = VCV_OP;
-    double x;
     int err = 0;
 
-    if ((spec->opt & OPT_H) && spec->hessvec != NULL) {
-	/* vcv based on Hessian */
-	int n = (k * (k + 1)) / 2;
+    if (spec->opt & OPT_A) {
+	int i;
 
-	for (i=0; i<n; i++) {
-	    pmod->vcv[i] = spec->hessvec[i];
-	}
-	for (i=0; i<k; i++) {
-	    x = pmod->vcv[ijton(i, i, k)];
-	    pmod->sderr[i] = sqrt(x);
+	for (i=0; i<pmod->ncoeff; i++) {
+	    pmod->sderr[i] = NADBL;
 	}
+    } else if (spec->opt & OPT_H) {
 	vcvopt = VCV_HESSIAN;
+	err = gretl_model_write_vcv(pmod, spec->Hinv);
     } else {
 	/* either OPG or QML */
 	err = mle_build_vcv(pmod, spec, &vcvopt);
     }
 
-    if (!err) {
+    if (!err && pmod->vcv != NULL) {
 	gretl_model_set_vcv_info(pmod, VCV_ML, vcvopt);
     }    
 
@@ -1830,13 +1766,25 @@ static int nl_model_allocate (MODEL *pmod, nlspec *spec)
     int k = spec->ncoeff;
     int nvc = (k * k + k) / 2;
 
+    pmod->vcv = NULL;
+
     pmod->coeff = malloc(k * sizeof *pmod->coeff);
     pmod->sderr = malloc(k * sizeof *pmod->sderr);
-    pmod->vcv = malloc(nvc * sizeof *pmod->vcv);
 
-    if (pmod->coeff == NULL || pmod->sderr == NULL || pmod->vcv == NULL) {
+    if (pmod->coeff == NULL || pmod->sderr == NULL) {
 	pmod->errcode = E_ALLOC;
-    } else {
+	return pmod->errcode;
+    }
+
+    if (!(spec->opt & OPT_A)) {
+	/* not an auxiliary model */
+	pmod->vcv = malloc(nvc * sizeof *pmod->vcv);
+	if (pmod->vcv == NULL) {
+	    pmod->errcode = E_ALLOC;
+	}
+    }
+
+    if (pmod->errcode == 0) {
 	pmod->ncoeff = k;
     }
 
@@ -1850,8 +1798,7 @@ static int nl_model_allocate (MODEL *pmod, nlspec *spec)
 */
 
 static int make_nl_model (MODEL *pmod, nlspec *spec, 
-			  const DATAINFO *pdinfo,
-			  gretlopt opt)
+			  const DATAINFO *pdinfo)
 {
     nl_model_allocate(pmod, spec);
     if (pmod->errcode) {
@@ -1879,8 +1826,9 @@ static int make_nl_model (MODEL *pmod, nlspec *spec,
 
     add_coeffs_to_model(pmod, spec->coeff);
 
-    if (!(opt & OPT_G)) {
-	/* not IVREG via GMM */
+    if (spec->ci == GMM && (spec->opt & OPT_G)) {
+	; /* IVREG via GMM */
+    } else {
 	pmod->errcode = add_param_names_to_model(pmod, spec, pdinfo);
     }
 
@@ -1979,8 +1927,8 @@ static void clear_nlspec (nlspec *spec)
     free(spec->nlfunc);
     spec->nlfunc = NULL;
 
-    free(spec->hessvec);
-    spec->hessvec = NULL;
+    gretl_matrix_free(spec->Hinv);
+    spec->Hinv = NULL;
 
     spec->flags = 0;
     spec->opt = OPT_NONE;
@@ -2070,30 +2018,28 @@ static int nls_calc (integer *m, integer *n, double *x, double *fvec,
 /* in case the user supplied analytical derivatives for the
    parameters, check them for sanity */
 
-static int check_derivatives (integer m, integer n, double *x,
-			      double *fvec, double *jac,
-			      integer ldjac, PRN *prn,
-			      nlspec *spec)
+static int check_derivatives (nlspec *spec, PRN *prn)
 {
 #if NLS_DEBUG > 1
     int T = spec->nobs * spec->ncoeff;
 #endif
+    double *x = spec->coeff;
+    double *fvec = spec->fvec;
+    double *jac = spec->jac;
+    integer m = spec->nobs;
+    integer n = spec->ncoeff;
+    integer ldjac = m;
     integer mode, iflag;
-    doublereal *xp = NULL;
-    doublereal *err = NULL;
-    doublereal *fvecp = NULL;
+    doublereal *xp, *xerr, *fvecp;
     int i, badcount = 0, zerocount = 0;
 
-    xp = malloc(n * sizeof *xp);
-    err = malloc(m * sizeof *err);
-    fvecp = malloc(m * sizeof *fvecp);
-
-    if (xp == NULL || err == NULL || fvecp == NULL) {
-	free(err);
-	free(xp);
-	free(fvecp);
-	return 1;
+    xp = malloc((n + m + m) * sizeof *xp);
+    if (xp == NULL) {
+	return E_ALLOC;
     }
+    
+    xerr = xp + n;
+    fvecp = xerr + m;
 
 #if NLS_DEBUG > 1
     fprintf(stderr, "\nchkder, starting: m=%d, n=%d, ldjac=%d\n",
@@ -2109,7 +2055,7 @@ static int check_derivatives (integer m, integer n, double *x,
     /* mode 1: x contains the point of evaluation of the function; on
        output xp is set to a neighboring point. */
     mode = 1;
-    chkder_(&m, &n, x, fvec, jac, &ldjac, xp, fvecp, &mode, err);
+    chkder_(&m, &n, x, fvec, jac, &ldjac, xp, fvecp, &mode, xerr);
 
     /* calculate gradient */
     iflag = 2;
@@ -2130,25 +2076,25 @@ static int check_derivatives (integer m, integer n, double *x,
 
     /* mode 2: on input, fvec must contain the functions, the rows of
        fjac must contain the gradients evaluated at x, and fvecp must
-       contain the functions evaluated at xp.  On output, err contains
+       contain the functions evaluated at xp.  On output, xerr contains
        measures of correctness of the respective gradients.
     */
     mode = 2;
-    chkder_(&m, &n, x, fvec, jac, &ldjac, xp, fvecp, &mode, err);
+    chkder_(&m, &n, x, fvec, jac, &ldjac, xp, fvecp, &mode, xerr);
 
 #if NLS_DEBUG > 1
     fprintf(stderr, "\nchkder, done mode 2:\n");
     for (i=0; i<m; i++) {
-	fprintf(stderr, "%d: fvec = %.14g, fvecp = %.14g, err = %g\n", i, 
-		fvec[i], fvecp[i], err[i]);
+	fprintf(stderr, "%d: fvec = %.14g, fvecp = %.14g, xerr = %g\n", i, 
+		fvec[i], fvecp[i], xerr[i]);
     }
 #endif
 
-    /* examine "err" vector */
+    /* examine "xerr" vector */
     for (i=0; i<m; i++) {
-	if (err[i] == 0.0) {
+	if (xerr[i] == 0.0) {
 	    zerocount++;
-	} else if (err[i] < 0.35) {
+	} else if (xerr[i] < 0.35) {
 	    badcount++;
 	}
     }
@@ -2164,9 +2110,8 @@ static int check_derivatives (integer m, integer n, double *x,
     }
 
  chkderiv_abort:
+
     free(xp);
-    free(err);
-    free(fvecp);
 
     return (zerocount > m / 4);
 }
@@ -2178,13 +2123,8 @@ static int mle_calculate (nlspec *s, PRN *prn)
     int err = 0;
 
     if (analytic_mode(s)) {
-	integer m = s->nobs;
-	integer n = s->ncoeff;
-	integer ldjac = m; 
-
 	if (!(s->opt & OPT_G)) {
-	    err = check_derivatives(m, n, s->coeff, s->fvec, 
-				    s->jac, ldjac, prn, s);
+	    err = check_derivatives(s, prn);
 	}
     }
 
@@ -2196,11 +2136,11 @@ static int mle_calculate (nlspec *s, PRN *prn)
 	err = BFGS_max(s->coeff, s->ncoeff, maxit, s->tol, 
 		       &s->fncount, &s->grcount, 
 		       get_mle_ll, C_LOGLIK, gradfun, s,
-		       s->opt, s->prn);
+		       NULL, s->opt, s->prn);
 	if (!err && (s->opt & (OPT_H | OPT_R))) {
 	    /* doing Hessian or QML covariance matrix */
-	    s->hessvec = numerical_hessian(s->coeff, s->ncoeff, 
-					   get_mle_ll, s, &err);
+	    s->Hinv = numerical_hessian(s->coeff, s->ncoeff, 
+					get_mle_ll, s, &err);
 	}
     }
 
@@ -2212,17 +2152,15 @@ static int mle_calculate (nlspec *s, PRN *prn)
 
 static int lm_calculate (nlspec *spec, PRN *prn)
 {
-    integer info, lwa;
-    integer m, n, ldjac;
+    integer m = spec->nobs;
+    integer n = spec->ncoeff;
+    integer lwa = 5 * n + m; /* work array size */
+    integer ldjac = m;       /* leading dimension of jac array */
+    integer info = 0;
     integer *ipvt;
     doublereal *wa;
     int err = 0;
 
-    m = spec->nobs;    /* number of observations */
-    n = spec->ncoeff;  /* number of coefficients */
-    lwa = 5 * n + m;   /* work array size */
-    ldjac = m;         /* leading dimension of jac array */
-
     wa = malloc(lwa * sizeof *wa);
     ipvt = malloc(n * sizeof *ipvt);
 
@@ -2231,8 +2169,7 @@ static int lm_calculate (nlspec *spec, PRN *prn)
 	goto nls_cleanup;
     }
 
-    err = check_derivatives(m, n, spec->coeff, spec->fvec, spec->jac, 
-			    ldjac, prn, spec);
+    err = check_derivatives(spec, prn);
     if (err) {
 	goto nls_cleanup; 
     }
@@ -2307,8 +2244,12 @@ nls_calc_approx (integer *m, integer *n, double *x, double *fvec,
 
 static int lm_approximate (nlspec *spec, PRN *prn)
 {
-    integer info, m, n, ldjac;
-    integer maxfev, mode = 1, nprint = 0, nfev = 0;
+    integer m = spec->nobs;
+    integer n = spec->ncoeff;
+    integer ldjac = m;
+    integer info = 0;
+    integer maxfev = 200 * (n + 1); /* max iterations */
+    integer mode = 1, nprint = 0, nfev = 0;
     integer iflag = 0;
     integer *ipvt;
     doublereal gtol = 0.0;
@@ -2317,12 +2258,6 @@ static int lm_approximate (nlspec *spec, PRN *prn)
     doublereal *wa1, *wa2, *wa3, *wa4;
     int err = 0;
     
-    m = spec->nobs;              /* number of observations */
-    n = spec->ncoeff;            /* number of parameters */
-    ldjac = m;                   /* leading dimension of jac array */
-
-    maxfev = 200 * (n + 1);      /* max iterations */
-
     dspace = malloc((5 * n + m) * sizeof *dspace);
     ipvt = malloc(n * sizeof *ipvt);
 
@@ -2732,16 +2667,39 @@ double get_default_nls_toler (void)
 
 static int check_spec_requirements (nlspec *spec)
 {
+    int err = 0;
+
     if (spec->nparam < 1) {
 	gretl_errmsg_set(_("No parameters have been specified"));
-	return 1;
+	err = 1;
+    } if (spec->ci == GMM) {
+	err = check_gmm_requirements(spec);
+    } 
+
+    return err;
+}
+
+static int mle_scalar_check (nlspec *spec)
+{
+    int err = 0;
+
+    if (spec->lhtype == GRETL_TYPE_DOUBLE) {
+	if (numeric_mode(spec)) {
+	    /* can't do OPG, so can't do QMLE for variance */
+	    if (spec->opt & OPT_R) {
+		gretl_errmsg_set("Scalar loglikelihood: can't do QML");
+		err = E_BADOPT;
+	    } else if (!(spec->opt & OPT_A)) {
+		/* ensure that we use the Hessian */
+		spec->opt |= OPT_H;
+	    }
+	} else {
+	    /* analytic mode: don't try to check gradient */
+	    spec->opt |= OPT_G;
+	}
     }
 
-    if (spec->ci == GMM) {
-	return check_gmm_requirements(spec);
-    }
-
-    return 0;
+    return err;
 }
 
 /* remedial treatment for an NLS model estimated using
@@ -2841,7 +2799,7 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
     spec->dinfo = pdinfo;
     spec->prn = prn;
 
-    if (opt & OPT_N) {
+    if (spec->opt & OPT_N) {
 	/* ignore any analytical derivs */
 	set_numeric_mode(spec);
     }
@@ -2864,28 +2822,37 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 	err = nl_missval_check(spec, pdinfo);
     }
 
+    if (!err && spec->ci == MLE) {
+	err = mle_scalar_check(spec);
+    }
+
     if (err) {
 	nlmod.errcode = err;
 	goto bailout;
     }
 
-    /* allocate arrays to be passed to minpack */
-    spec->fvec = malloc(spec->nobs * sizeof *spec->fvec);
-    spec->jac = malloc(spec->nobs * spec->ncoeff * sizeof *spec->jac);
-
-    if (spec->fvec == NULL || spec->jac == NULL) {
-	nlmod.errcode = E_ALLOC;
-	goto bailout;
-    }
-
-    if (spec->lvec != NULL) {
-	for (t=0; t<spec->nobs; t++) {
-	    spec->fvec[t] = spec->lvec->val[t];
-	}
+    if (spec->ci == GRETL_TYPE_DOUBLE) {
+	spec->fvec = NULL;
+	spec->jac = NULL;
     } else {
-	i = 0;
-	for (t=spec->t1; t<=spec->t2; t++) {
-	    spec->fvec[i++] = (*spec->Z)[spec->lhv][t];
+	/* allocate auxiliary arrays */
+	spec->fvec = malloc(spec->nobs * sizeof *spec->fvec);
+	spec->jac = malloc(spec->nobs * spec->ncoeff * sizeof *spec->jac);
+
+	if (spec->fvec == NULL || spec->jac == NULL) {
+	    nlmod.errcode = E_ALLOC;
+	    goto bailout;
+	}
+
+	if (spec->lvec != NULL) {
+	    for (t=0; t<spec->nobs; t++) {
+		spec->fvec[t] = spec->lvec->val[t];
+	    }
+	} else {
+	    i = 0;
+	    for (t=spec->t1; t<=spec->t2; t++) {
+		spec->fvec[i++] = (*spec->Z)[spec->lhv][t];
+	    }
 	}
     }
 
@@ -2896,7 +2863,7 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 	spec->tol = libset_get_double(NLS_TOLER);
     }
 
-    if (!(opt & OPT_Q)) {
+    if (spec->ci != GMM && !(spec->opt & OPT_Q)) {
 	pputs(prn, (numeric_mode(spec))?
 	      _("Using numerical derivatives\n") :
 	      _("Using analytical derivatives\n"));
@@ -2920,7 +2887,7 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    if (!(opt & OPT_Q)) {
+    if (!(spec->opt & OPT_Q)) {
 	pprintf(prn, _("Tolerance = %g\n"), spec->tol);
     }
 
@@ -2937,7 +2904,7 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 	    }
 	} else {
 	    /* MLE, GMM */
-	    make_nl_model(&nlmod, spec, pdinfo, opt);
+	    make_nl_model(&nlmod, spec, pdinfo);
 	}
     } else if (nlmod.errcode == 0) { 
 	/* error code missing: supply one */
@@ -2977,14 +2944,8 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 
     dataset_drop_last_variables(pdinfo->v - origv, pZ, pdinfo);
 
-    if (!(opt & OPT_A)) {
-	if (nlmod.errcode) {
-	    if (opt & OPT_U) {
-		nlmod.opt |= OPT_U; /* continue on error */
-	    }
-	} else {
-	    set_model_id(&nlmod);
-	}
+    if (!(opt & OPT_A) && !nlmod.errcode) {
+	set_model_id(&nlmod);
     }
 
     return nlmod;
@@ -3077,7 +3038,7 @@ nlspec *nlspec_new (int ci, const DATAINFO *pdinfo)
     spec->ncoeff = 0;
     spec->nvec = 0;
 
-    spec->hessvec = NULL;
+    spec->Hinv = NULL;
 
     spec->ci = ci;
     spec->flags = 0;
