@@ -18,11 +18,42 @@
  */
 
 #include "libgretl.h"
+#include "gretl_scalar.h"
+
 #include <errno.h>
 
-/* model commands plus ADD and OMIT */
+#define OPTDEBUG 0
+
 #define vcv_opt_ok(c) (MODEL_COMMAND(c) || c == ADD || c == OMIT)
 
+#define quiet_opt_ok(c) (MODEL_COMMAND(c) ||	\
+			 c == ADD ||		\
+			 c == ADF ||		\
+			 c == ANOVA ||		\
+			 c == APPEND ||		\
+			 c == CHOW ||		\
+			 c == COINT2 ||		\
+			 c == CORRGM ||		\
+			 c == CUSUM ||		\
+			 c == DATA ||		\
+			 c == ESTIMATE ||	\
+			 c == FCAST ||		\
+			 c == FOREIGN ||	\
+			 c == FREQ ||		\
+			 c == KPSS ||		\
+			 c == MODTEST ||	\
+			 c == LOOP ||		\
+			 c == NORMTEST ||	\
+			 c == OLS ||		\
+			 c == OMIT ||		\
+			 c == OPEN ||		\
+			 c == RESET ||		\
+			 c == RESTRICT ||	\
+			 c == RMPLOT ||		\
+			 c == VAR ||		\
+			 c == VECM ||		\
+			 c == XCORRGM)
+
 struct gretl_option {
     int ci;              /* command index (gives context) */
     gretlopt o;          /* index of integer type */
@@ -48,7 +79,6 @@ struct flag_match {
 struct gretl_option gretl_opts[] = {
     { ADD,      OPT_B, "both", 0 },
     { ADD,      OPT_I, "silent", 0 },
-    { ADD,      OPT_Q, "quiet", 0 },
     { ADD,      OPT_T, "inst", 0 },
     { ADF,      OPT_N, "nc", 0 }, 
     { ADF,      OPT_C, "c", 0 }, 
@@ -56,17 +86,13 @@ struct gretl_option gretl_opts[] = {
     { ADF,      OPT_R, "ctt", 0 },     
     { ADF,      OPT_T, "ct", 0 }, 
     { ADF,      OPT_V, "verbose", 0 },
-    { ADF,      OPT_Q, "quiet", 0 },
     { ADF,      OPT_F, "difference", 0 },
     { ADF,      OPT_E, "test-down", 0 },
     { ADF,      OPT_G, "gls", 0 },
-    { ANOVA,    OPT_Q, "quiet", 0 },
     { AR1,      OPT_B, "no-corc", 0 },
     { AR1,      OPT_H, "hilu", 0 },
     { AR1,      OPT_P, "pwe", 0 },
-    { AR1,      OPT_Q, "quiet", 0},
     { APPEND,   OPT_T, "time-series", 0 },
-    { APPEND,   OPT_Q, "quiet", 0 },
     { ARBOND,   OPT_A, "asymptotic", 0 },
     { ARBOND,   OPT_D, "time-dummies", 0 },
     { ARBOND,   OPT_H, "orthdev", 0 },
@@ -76,7 +102,6 @@ struct gretl_option gretl_opts[] = {
     { ARMA,     OPT_H, "hessian", 0 },
     { ARMA,     OPT_L, "lbfgs", 0 },
     { ARMA,     OPT_N, "nc", 0 },    
-    { ARMA,     OPT_Q, "quiet", 0 },
     { ARMA,     OPT_U, "continue", 0 },
     { ARMA,     OPT_V, "verbose", 0 },
     { ARMA,     OPT_X, "x-12-arima", 0 },
@@ -84,7 +109,6 @@ struct gretl_option gretl_opts[] = {
     { BXPLOT,   OPT_O, "notches", 0 },
     { BXPLOT,   OPT_U, "output", 2 },
     { CHOW,     OPT_D, "dummy", 0 },
-    { CHOW,     OPT_Q, "quiet", 0 },
     { COINT,    OPT_E, "test-down", 0 },
     { COINT,    OPT_N, "nc", 0 },
     { COINT,    OPT_R, "ctt", 0 },     
@@ -93,7 +117,6 @@ struct gretl_option gretl_opts[] = {
     { COINT2,   OPT_A, "crt", 0 },
     { COINT2,   OPT_D, "seasonals", 0 },
     { COINT2,   OPT_N, "nc", 0 },
-    { COINT2,   OPT_Q, "quiet", 0 },
     { COINT2,   OPT_R, "rc", 0 },
     { COINT2,   OPT_T, "ct", 0 },
     { COINT2,   OPT_V, "verbose", 0 },
@@ -101,13 +124,11 @@ struct gretl_option gretl_opts[] = {
     { CORR,     OPT_S, "spearman", 0 },
     { CORR,     OPT_U, "uniform", 0 },
     { CORR,     OPT_V, "verbose", 0 },
-    { CORRGM,   OPT_Q, "quiet", 0 },
-    { CUSUM,    OPT_Q, "quiet", 0 },
     { CUSUM,    OPT_R, "squares", 0 },
     { DATA,     OPT_O, "odbc", 0 },
-    { DATA,     OPT_Q, "quiet", 0 },
     { DATAMOD,  OPT_P, "preserve", 0 },
     { DELEET,   OPT_D, "db", 0 },
+    { DELEET,   OPT_L, "list", 0 },
     { DIFFTEST, OPT_G, "sign", 0 },
     { DIFFTEST, OPT_R, "rank-sum", 0 },
     { DIFFTEST, OPT_I, "signed-rank", 0 },
@@ -115,46 +136,56 @@ struct gretl_option gretl_opts[] = {
     { DISCRETE, OPT_R, "reverse", 0 },
     { DUMMIFY,  OPT_F, "drop-first", 0 },
     { DUMMIFY,  OPT_L, "drop-last", 0 },
+    { DURATION, OPT_W, "weibull", 0 },
+    { DURATION, OPT_E, "exponential", 0 },
+    { DURATION, OPT_L, "loglogistic", 0 },
+    { DURATION, OPT_Z, "lognormal", 0 },
+    { DURATION, OPT_M, "medians", 0 },    
+    { DURATION, OPT_G, "opg", 0 },
+    { DURATION, OPT_R, "robust", 0 },
+    { DURATION, OPT_V, "verbose", 0 },
+#if 0 /* not yet */
+    { EQNPRINT, OPT_F, "filename", 0 }, /* backward compatibility */
+#endif
     { EQNPRINT, OPT_O, "complete", 0 },
     { EQNPRINT, OPT_T, "t-ratios", 0 },
     { TABPRINT, OPT_O, "complete", 0 },
     { TABPRINT, OPT_R, "rtf", 0 },
+#if 0 /* not yet */
+    { TABPRINT, OPT_F, "filename", 0 }, /* backward compatibility */
+#endif
     { ESTIMATE, OPT_I, "iterate", 0 },
     { ESTIMATE, OPT_M, "geomean", 0 },
     { ESTIMATE, OPT_N, "no-df-corr", 0 },
-    { ESTIMATE, OPT_Q, "quiet", 0 },
     { ESTIMATE, OPT_V, "verbose", 0 },
     { FCAST,    OPT_D, "dynamic", 0 },
-    { FCAST,    OPT_G, "plot", 1 },
     { FCAST,    OPT_N, "no-stats", 0 },
     { FCAST,    OPT_S, "static", 0 },
-    { FCAST,    OPT_Q, "quiet", 0 },
     { FCAST,    OPT_R, "rolling", 0 },
     { FCAST,    OPT_O, "out-of-sample", 0 },
     { FCAST,    OPT_I, "integrate", 0 },
+    { FCAST,    OPT_U, "plot", 1 },
     { FOREIGN,  OPT_D, "send-data", 0 },
-    { FOREIGN,  OPT_Q, "quiet", 0 },
     { FOREIGN,  OPT_V, "verbose", 0 },
     { FREQ,     OPT_O, "gamma", 0 },
-    { FREQ,     OPT_Q, "quiet", 0 },
     { FREQ,     OPT_S, "silent", 0 },
     { FREQ,     OPT_Z, "normal", 0 },
     { FUNDEBUG, OPT_C, "continue", 0 },
     { FUNDEBUG, OPT_N, "next", 0 },
     { FUNDEBUG, OPT_Q, "quit", 0 },
     { GARCH,    OPT_A, "arma-init", 0 },
-    { GARCH,    OPT_E, "stdresid", 0 },
     { GARCH,    OPT_F, "fcp", 0 }, 
     { GARCH,    OPT_N, "nc", 0 }, 
     { GARCH,    OPT_R, "robust", 0 },
     { GARCH,    OPT_U, "continue", 0 },
     { GARCH,    OPT_V, "verbose", 0 },
+    { GARCH,    OPT_Z, "stdresid", 0 },
     { GMM,      OPT_I, "iterate", 0 },
     { GMM,      OPT_L, "lbfgs", 0 },
-    { GMM,      OPT_Q, "quiet", 0 },
     { GMM,      OPT_T, "two-step", 0 },
     { GMM,      OPT_U, "continue", 0 },
     { GMM,      OPT_V, "verbose", 0 },
+    { GNUPLOT,  OPT_D, "input", 2 },
     { GNUPLOT,  OPT_O, "with-lines", 0 },
     { GNUPLOT,  OPT_I, "inverse-fit", 0 },
     { GNUPLOT,  OPT_L, "loess-fit", 0 },
@@ -168,19 +199,15 @@ struct gretl_option gretl_opts[] = {
     { GNUPLOT,  OPT_U, "output", 2 },
     { GNUPLOT,  OPT_Y, "single-yaxis", 0 },
     { GNUPLOT,  OPT_X, "matrix", 2 },
-    { GRAPH,    OPT_O, "tall", 0 },
     { HECKIT,   OPT_M, "ml", 0 },
     { HECKIT,   OPT_T, "two-step", 0 },
     { HECKIT,   OPT_V, "verbose", 0 },
     { HELP,     OPT_F, "func", 0 },
-    { HSK,      OPT_Q, "quiet", 0 },
-    { INTREG,   OPT_Q, "quiet", 0 },
     { INTREG,   OPT_R, "robust", 0 },
     { INTREG,   OPT_V, "verbose", 0 },
     { IVREG,    OPT_G, "gmm", 0 },
     { IVREG,    OPT_I, "iterate", 0 },
     { IVREG,    OPT_L, "liml", 0 },
-    { IVREG,    OPT_Q, "quiet", 0 },
     { IVREG,    OPT_R, "robust", 0 },  
     { IVREG,    OPT_S, "save", 0 },
     { IVREG,    OPT_T, "two-step", 0 },
@@ -189,7 +216,6 @@ struct gretl_option gretl_opts[] = {
     { KALMAN,   OPT_D, "diffuse", 0 },
     { KPSS,     OPT_T, "trend", 0 },
     { KPSS,     OPT_V, "verbose", 0 },
-    { KPSS,     OPT_Q, "quiet", 0 },
     { KPSS,     OPT_F, "difference", 0 },
     { LEVERAGE, OPT_S, "save", 0 },
     { MODTEST,  OPT_A, "autocorr", 0 },
@@ -201,38 +227,40 @@ struct gretl_option gretl_opts[] = {
     { MODTEST,  OPT_S, "squares", 0 }, 
     { MODTEST,  OPT_P, "panel", 0 },
     { MODTEST,  OPT_R, "robust", 0 },
-    { MODTEST,  OPT_Q, "quiet", 0 },
     { MODTEST,  OPT_W, "white", 0 },
     { MODTEST,  OPT_X, "white-nocross", 0 },
+    { LABELS,   OPT_F, "from-file", 2 },
+    { LABELS,   OPT_T, "to_file", 2 },
     { LOGIT,    OPT_M, "multinomial", 0 },
     { LOGIT,    OPT_P, "p-values", 0 },
-    { LOGIT,    OPT_Q, "quiet", 0 },
     { LOGIT,    OPT_R, "robust", 0 },
     { LOGIT,    OPT_V, "verbose", 0 },
     { LOOP,     OPT_P, "progressive", 0 },
-    { LOOP,     OPT_Q, "quiet", 0 },
     { LOOP,     OPT_V, "verbose", 0 },
     { MAHAL,    OPT_S, "save", 0 },
     { MAHAL,    OPT_V, "vcv", 0 },
     { MEANTEST, OPT_O, "unequal-vars", 0 },
+    { MLE,      OPT_A, "auxiliary", 0 },
     { MLE,      OPT_H, "hessian", 0 },
     { MLE,      OPT_G, "no-gradient-check", 0 },
     { MLE,      OPT_L, "lbfgs", 0 },
     { MLE,      OPT_N, "numerical", 0 },
-    { MLE,      OPT_Q, "quiet", 0 },
     { MLE,      OPT_R, "robust", 0 },
     { MLE,      OPT_U, "continue", 0 },
     { MLE,      OPT_V, "verbose", 0 },
+    { MODPRINT, OPT_A, "addstats", 2 },
     { MODPRINT, OPT_C, "csv", 0 },
     { MODPRINT, OPT_O, "complete", 0 },
     { MODPRINT, OPT_R, "rtf", 0 },
     { MODPRINT, OPT_T, "tex", 0 },
     { MPOLS,    OPT_O, "vcv", 0 },
-    { MPOLS,    OPT_Q, "quiet", 0 },
     { MPOLS,    OPT_S, "simple-print", 0 },
+    { NEGBIN,   OPT_G, "opg", 0 },
+    { NEGBIN,   OPT_M, "model1", 0 },
+    { NEGBIN,   OPT_R, "robust", 0 },
+    { NEGBIN,   OPT_V, "verbose", 0 },
     { NLS,      OPT_N, "numerical", 0 },
     { NLS,      OPT_O, "vcv", 0 },
-    { NLS,      OPT_Q, "quiet", 0 },
     { NLS,      OPT_R, "robust", 0 },
     { NLS,      OPT_U, "continue", 0 },
     { NLS,      OPT_V, "verbose", 0 },
@@ -241,7 +269,6 @@ struct gretl_option gretl_opts[] = {
     { NORMTEST, OPT_W, "swilk", 0 },
     { NORMTEST, OPT_J, "jbera", 0 },
     { NORMTEST, OPT_L, "lillie", 0 },
-    { NORMTEST, OPT_Q, "quiet", 0 },
     { NULLDATA, OPT_P, "preserve", 0 },
     { OLS,      OPT_F, "print-final", 0 },
     { OLS,      OPT_J, "jackknife", 0 },
@@ -254,7 +281,6 @@ struct gretl_option gretl_opts[] = {
     { OMIT,     OPT_A, "auto", 1 },
     { OMIT,     OPT_B, "both", 0 },
     { OMIT,     OPT_I, "silent", 0 },
-    { OMIT,     OPT_Q, "quiet", 0 },
     { OMIT,     OPT_T, "inst", 0 },
     { OMIT,     OPT_W, "wald", 0 },
     { OPEN,     OPT_B, "progress-bar", 0 },
@@ -264,7 +290,6 @@ struct gretl_option gretl_opts[] = {
     { OPEN,     OPT_O, "odbc", 0 },
     { OPEN,     OPT_P, "preserve", 0 },
     { OPEN,     OPT_W, "www", 0 },
-    { OPEN,     OPT_Q, "quiet", 0 },
     { OUTFILE,  OPT_A, "append", 0 },
     { OUTFILE,  OPT_C, "close", 0 },
     { OUTFILE,  OPT_W, "write", 0 },
@@ -276,12 +301,12 @@ struct gretl_option gretl_opts[] = {
     { PANEL,    OPT_M, "matrix-diff", 0 },
     { PANEL,    OPT_O, "vcv", 0 },
     { PANEL,    OPT_P, "pooled", 0 },
-    { PANEL,    OPT_Q, "quiet", 0 },
     { PANEL,    OPT_R, "robust", 0 },
     { PANEL,    OPT_S, "silent", 0 },
     { PANEL,    OPT_U, "random-effects", 0 },
     { PANEL,    OPT_V, "verbose", 0 },
     { PANEL,    OPT_W, "unit-weights", 0 },
+    { POISSON,  OPT_R, "robust", 0 },    
     { POISSON,  OPT_V, "verbose", 0 },
     { PCA,      OPT_C, "covariance", 0 },
     { PCA,      OPT_A, "save-all", 0 },
@@ -289,31 +314,28 @@ struct gretl_option gretl_opts[] = {
     { PERGM,    OPT_O, "bartlett", 0 },
     { PERGM,    OPT_L, "log", 0 },
     { PLOT,     OPT_O, "one-scale", 0 },
+    { PLOT,     OPT_S, "time-series", 0 },
+    { PLOT,     OPT_T, "tall", 0 },
     { PRINT,    OPT_O, "byobs", 0 },
     { PRINT,    OPT_N, "no-dates", 0 },
     { PROBIT,   OPT_P, "p-values", 0 },
-    { PROBIT,   OPT_Q, "quiet", 0 },
     { PROBIT,   OPT_R, "robust", 0 },
     { PROBIT,   OPT_V, "verbose", 0 },
     { QQPLOT,   OPT_R, "raw", 0 },
     { QQPLOT,   OPT_Z, "z-scores", 0 },      
     { QUANTREG, OPT_I, "intervals", 1 },
     { QUANTREG, OPT_N, "no-df-corr", 0 },
-    { QUANTREG, OPT_Q, "quiet", 0 },
     { QUANTREG, OPT_R, "robust", 0 },
     { QUIT,     OPT_X, "exit", 0 },
     { RESET,    OPT_C, "cubes-only", 0 },
-    { RESET,    OPT_Q, "quiet", 0 },
     { RESET,    OPT_R, "squares-only", 0 },
     { RESTRICT, OPT_B, "bootstrap", 0 },
     { RESTRICT, OPT_F, "full", 0 },
     { RESTRICT, OPT_J, "jitter", 0 },
-    { RESTRICT, OPT_Q, "quiet", 0 },
     { RESTRICT, OPT_V, "verbose", 0 },
     { RESTRICT, OPT_L, "lbfgs", 0 },
     { RESTRICT, OPT_N, "no-scaling", 0 },
     { RESTRICT, OPT_S, "silent", 0 },
-    { RMPLOT,   OPT_Q, "quiet", 0 },
     { RUNS,     OPT_D, "difference", 0 },
     { RUNS,     OPT_E, "equal", 0 },
     { SCATTERS, OPT_L, "with-lines", 0 },
@@ -330,6 +352,7 @@ struct gretl_option gretl_opts[] = {
     { SETOBS,   OPT_X, "cross-section", 0 },
     { SETOBS,   OPT_N, "special-time-series", 0 },
     { SMPL,     OPT_B, "balanced", 0 },
+    { SMPL,     OPT_C, "contiguous", 0 },
     { SMPL,     OPT_F, "full", 0 },
     { SMPL,     OPT_O, "dummy", 0 },
     { SMPL,     OPT_M, "no-missing", 0 },
@@ -353,13 +376,15 @@ struct gretl_option gretl_opts[] = {
     { SUMMARY,  OPT_S, "simple", 0 },
     { SYSTEM,   OPT_I, "iterate", 0 },
     { SYSTEM,   OPT_V, "verbose", 0 },
+#if 0
+    { TABPRINT, OPT_F, "filename", 0 }, /* backward compatibility */
+#endif
     { TOBIT,    OPT_V, "verbose", 0 },
     { VAR,      OPT_D, "seasonals", 0 },
     { VAR,      OPT_F, "variance-decomp", 0 },
     { VAR,      OPT_I, "impulse-responses", 0 },
     { VAR,      OPT_L, "lagselect", 0 },
     { VAR,      OPT_N, "nc", 0 },
-    { VAR,      OPT_Q, "quiet", 0 }, 
     { VAR,      OPT_R, "robust", 0 }, 
     { VAR,      OPT_T, "trend", 0 }, 
     { VAR,      OPT_S, "lags", 2 },
@@ -369,13 +394,10 @@ struct gretl_option gretl_opts[] = {
     { VECM,     OPT_F, "variance-decomp", 0 },
     { VECM,     OPT_I, "impulse-responses", 0 },
     { VECM,     OPT_N, "nc", 0 },
-    { VECM,     OPT_Q, "quiet", 0 },
     { VECM,     OPT_R, "rc", 0 },
     { VECM,     OPT_T, "ct", 0 },
     { VECM,     OPT_V, "verbose", 0 },
     { WLS,      OPT_R, "robust", 0 },    
-    { WLS,      OPT_Q, "quiet", 0 },
-    { XCORRGM,  OPT_Q, "quiet", 0 },
     { XTAB,     OPT_C, "column", 0 },
     { XTAB,     OPT_M, "matrix", 2 },
     { XTAB,     OPT_R, "row", 0 },
@@ -444,12 +466,14 @@ const char **get_opts_for_command (int ci, int *nopt)
     int i, j, n = 0;
     const char **ret = NULL;
 
-    if (vcv_opt_ok(ci) && ci != OLS) {
-	n++; /* vcv */
+    if (ci != OLS) {
+	/* widely applicable options which are "attached" to OLS */
+	n += vcv_opt_ok(ci);
+	n += quiet_opt_ok(ci);
     }
 
     for (i=0; gretl_opts[i].ci != 0; i++) {
-	if (gretl_opts[i].ci == ci) n++;
+	n += (gretl_opts[i].ci == ci);
     }
 
     if (n == 0) {
@@ -467,8 +491,13 @@ const char **get_opts_for_command (int ci, int *nopt)
 	}
     }
 
-    if (vcv_opt_ok(ci) && ci != OLS) {
-	ret[j++] = "vcv";
+    if (ci != OLS) {
+	if (vcv_opt_ok(ci)) {
+	    ret[j++] = "vcv";
+	}
+	if (quiet_opt_ok(ci)) {
+	    ret[j++] = "quiet";
+	}
     }
 
     *nopt = n;
@@ -539,6 +568,10 @@ static int opt_is_valid (gretlopt opt, int ci, char c)
 	return 1;
     }
 
+    if (opt == OPT_Q && quiet_opt_ok(ci)) {
+	return 1;
+    }    
+
     for (i=0; gretl_opts[i].ci != 0; i++) {
 	if (ci == gretl_opts[i].ci && opt == gretl_opts[i].o) {
 	    return 1;
@@ -552,6 +585,28 @@ static int opt_is_valid (gretlopt opt, int ci, char c)
     return 0;
 }
 
+gretlopt valid_short_opt (int ci, char c)
+{
+    gretlopt opt = 0;
+    int i, ok;
+
+    for (i=0; flag_matches[i].c != '\0'; i++) {
+	if (c == flag_matches[i].c) {
+	    opt = flag_matches[i].o;
+	    break;
+	}
+    }
+
+    if (opt) {
+	ok = opt_is_valid(opt, ci, c);
+	if (!ok) {
+	    opt = 0;
+	}
+    }
+	
+    return opt;
+}
+
 /* See if at point @p (at which we've found '-') in string @s we
    might be at the start of an option flag: the previous character
    must be a space, and we must not be inside a quoted string.
@@ -614,49 +669,10 @@ static gretlopt get_short_opts (char *line, int ci, int *err)
     return ret;
 }
 
-static int valid_long_opt (int ci, const char *lopt)
-{
-    int opt = OPT_NONE;
-    int i;
-
-    if (vcv_opt_ok(ci) && !strcmp(lopt, "vcv")) {
-	return OPT_O;
-    }
-
-    /* start by looking for an exact match */
-    for (i=0; gretl_opts[i].o != 0; i++) {
-	if (ci == gretl_opts[i].ci && 
-	    !strcmp(lopt, gretl_opts[i].longopt)) {
-	    opt = gretl_opts[i].o;
-	    break;
-	}
-    }
-
-    /* if this failed, try for an abbreviation or extension */
-    if (opt == OPT_NONE) {
-	int len, len1, len2;
-
-	len1 = strlen(lopt);
-	for (i=0; gretl_opts[i].o != 0; i++) {
-	    len2 = strlen(gretl_opts[i].longopt);
-	    len = (len2 > len1)? len1 : len2;
-	    if (ci == gretl_opts[i].ci && 
-		!strncmp(lopt, gretl_opts[i].longopt, len)) {
-		opt = gretl_opts[i].o;
-		break;
-	    }
-	} 
-    }   
-
-    return opt;
-}
-
 /* Apparatus for setting and retrieving parameters associated
    with command options, as in --opt=val.  
 */
 
-#define OPDEBUG 0
-
 typedef struct optparm_ optparm;
 
 struct optparm_ {
@@ -684,7 +700,7 @@ void clear_option_params (void)
 {
     int i;
 
-#if OPDEBUG 
+#if OPTDEBUG > 1
     fprintf(stderr, "clearing option params\n");
 #endif
 
@@ -709,44 +725,6 @@ static optparm *matching_optparm (int ci, gretlopt opt)
     return NULL;
 }
 
-static int push_optparm (int ci, gretlopt opt, char *val)
-{
-    optparm *op;
-    int n = n_parms + 1;
-
-    op = matching_optparm(ci, opt);
-    if (op != NULL) {
-	/* got a match for the ci, opt pair already */
-	free(op->val);
-	op->val = val;
-	return 0;
-    }
-
-    op = realloc(optparms, n * sizeof *op);
-    if (op == NULL) {
-	return E_ALLOC;
-    }
- 
-    optparms = op;
-    op = &optparms[n-1];
-    op->ci = ci;
-    op->opt = opt;
-    op->val = val;
-    n_parms = n;
-
-#if OPDEBUG 
-    fprintf(stderr, "push_optparm: val='%s', n_parms=%d\n",
-	    op->val, n_parms);
-#endif
-
-    return 0;
-}
-
-enum {
-    ACCEPTS_PARM = 1,
-    NEEDS_PARM
-};
-
 /* for a given @ci, @opt pair, determine its status with regard
    to a parameter value (not allowed, allowed, or required)
 */
@@ -764,32 +742,62 @@ static int option_parm_status (int ci, gretlopt opt)
     return 0;
 }
 
-/* We got an "=val" parameter value in connection with a given option:
-   check this for validity.  Note that (at present), for almost all
-   options that accept but do not mandate a parameter value, the
-   parameter must be a numerical value.
-*/
-
-static int check_optval (int ci, gretlopt opt, int status, char *val)
+static int real_push_option_param (int ci, gretlopt opt, char *val,
+				   int checked)
 {
-    int err = 0;
+    optparm *op;
+    int n, err = 0;
 
-    if (status == NEEDS_PARM) {
-	err = push_optparm(ci, opt, val);
-    } else if (status == ACCEPTS_PARM) {
-	if (ci == FCAST && opt == OPT_G) {
-	    /* --plot=filename */
-	    err = push_optparm(ci, opt, val);
-	} else if (numeric_string(val)) {
-	    err = push_optparm(ci, opt, val);
-	} else {
-	    err = 1;
-	}
-    } else {
-	err = 1;
+    if (!checked && option_parm_status(ci, opt) == OPT_NO_PARM)  {
+	return E_DATA;
     }
 
-    return err;
+    n = n_parms + 1;
+
+    op = matching_optparm(ci, opt);
+    if (op != NULL) {
+	/* got a match for the ci, opt pair already */
+	free(op->val);
+	op->val = val;
+	return 0;
+    }
+
+    op = realloc(optparms, n * sizeof *op);
+
+    if (op == NULL) {
+	err = E_ALLOC;
+    } else {
+	optparms = op;
+	op = &optparms[n-1];
+	op->ci = ci;
+	op->opt = opt;
+	op->val = val;
+	n_parms = n;
+    }
+
+#if OPTDEBUG 
+    fprintf(stderr, "push_option_param: val='%s', n_parms=%d, "
+	    "err = %d\n", op->val, n_parms, err);
+#endif
+
+    return 0;
+}
+
+/**
+ * push_option_param:
+ * @ci: gretl command index.
+ * @opt: gretl option flag.
+ * @val: parameter value as string.
+ *
+ * Pushes onto an internal stack a record of the @val
+ * to be associated with @opt for the current @ci.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ */
+
+int push_option_param (int ci, gretlopt opt, char *val)
+{
+    return real_push_option_param(ci, opt, val, 0);
 }
 
 /**
@@ -822,9 +830,17 @@ const char *get_optval_string (int ci, gretlopt opt)
 double get_optval_double (int ci, gretlopt opt)
 {
     optparm *op = matching_optparm(ci, opt);
+    double ret = NADBL;
 
-    return (op != NULL && op->val != NULL)? 
-	dot_atof(op->val) : NADBL;
+    if (op != NULL && op->val != NULL) {
+	if (numeric_string(op->val)) {
+	    ret = dot_atof(op->val);
+	} else {
+	    ret = gretl_scalar_get_value(op->val);
+	}
+    }
+
+    return ret;
 }
 
 /* called via GUI */
@@ -837,16 +853,68 @@ double get_optval_double (int ci, gretlopt opt)
  *
  * Sets a double-precision ancillary value to be associated
  * with option @opt for command @ci.
+ *
+ * Returns: 0 on success, non-zero on failure.
  */
 
-void set_optval_double (int ci, gretlopt opt, double x)
+int set_optval_double (int ci, gretlopt opt, double x)
 {
     char s[32];
 
     gretl_push_c_numeric_locale();
     sprintf(s, "%g", x);
     gretl_pop_c_numeric_locale();
-    push_optparm(ci, opt, gretl_strdup(s));
+
+    return real_push_option_param(ci, opt, gretl_strdup(s), 1);
+}
+
+gretlopt valid_long_opt (int ci, const char *lopt, OptStatus *status)
+{
+    int opt = OPT_NONE;
+    int i;
+
+    *status = 0;
+
+    if (*lopt == '\0') {
+	return 0;
+    }
+
+    if (vcv_opt_ok(ci) && !strcmp(lopt, "vcv")) {
+	return OPT_O;
+    }
+
+    if (quiet_opt_ok(ci) && !strcmp(lopt, "quiet")) {
+	return OPT_Q;
+    }
+
+    /* start by looking for an exact match */
+    for (i=0; gretl_opts[i].o != 0; i++) {
+	if (ci == gretl_opts[i].ci && 
+	    !strcmp(lopt, gretl_opts[i].longopt)) {
+	    opt = gretl_opts[i].o;
+	    *status = gretl_opts[i].parminfo;
+	    break;
+	}
+    }
+
+    /* if this failed, try for an abbreviation or extension */
+    if (opt == OPT_NONE) {
+	int len, len1, len2;
+
+	len1 = strlen(lopt);
+	for (i=0; gretl_opts[i].o != 0; i++) {
+	    len2 = strlen(gretl_opts[i].longopt);
+	    len = (len2 > len1)? len1 : len2;
+	    if (ci == gretl_opts[i].ci && 
+		!strncmp(lopt, gretl_opts[i].longopt, len)) {
+		opt = gretl_opts[i].o;
+		*status = gretl_opts[i].parminfo;
+		break;
+	    }
+	} 
+    }  
+
+    return opt;
 }
 
 #define data_open_special(s) (!strcmp(s, "sheet") || \
@@ -890,7 +958,11 @@ static int handle_optval (char *s, int ci, gretlopt opt, int status)
     } 
 
     if (!err) {
-	err = check_optval(ci, opt, status, val);
+	if (status == OPT_NEEDS_PARM || status == OPT_ACCEPTS_PARM) {
+	    err = real_push_option_param(ci, opt, val, 1);
+	} else {
+	    err = E_DATA;
+	}
     }
 
     if (err) {
@@ -911,15 +983,20 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
 {
     char *s = line;
     char longopt[32];
+    OptStatus status = 0;
     gretlopt match, ret = 0L;
 
     while ((s = strstr(s, "--")) != NULL) {
 	match = 0;
 	*longopt = '\0';
 
+#if OPTDEBUG
+	fprintf(stderr, "get_long_opts: s = '%s'\n", s);
+#endif
+
 	if (maybe_opt_start(line, s)) {
 	    sscanf(s + 2, "%31[^ =]", longopt);
-	    match = valid_long_opt(ci, longopt);
+	    match = valid_long_opt(ci, longopt, &status);
 	    if (match > 0) {
 		/* recognized an acceptable option flag */
 		ret |= match;
@@ -936,18 +1013,15 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
 	}
 
 	if (match > 0) {
-	    int status = option_parm_status(ci, match);
-
 	    gretl_delete(s, 0, 2 + strlen(longopt));
 	    if (*s == '=') {
 		/* got a param value: is it OK? */
-		if (status == 0) {
-		    /* this option does not accept a param */
+		if (status == OPT_NO_PARM) {
 		    *err = E_PARSE;
 		} else {
 		    *err = handle_optval(s, ci, match, status);
 		}
-	    } else if (status == NEEDS_PARM) {
+	    } else if (status == OPT_NEEDS_PARM) {
 		/* we need a param value but there's none */
 		gretl_errmsg_sprintf(_("The option '--%s' requires a parameter"), 
 				     longopt);
@@ -1048,24 +1122,23 @@ static int end_block_ci (const char *s)
 gretlopt get_gretl_options (char *line, int *err)
 {
     gretlopt oflags = 0L;
-    int n = strlen(line);
     gretlopt opt;
     char cmdword[9] = {0};
+    int endblock = 0;
     int ci, myerr = 0;
 
-    gretl_error_clear();
-
     if (err != NULL) {
 	*err = 0;
     }
 
-    if (n < 2 || *line == '#') {
+    if (strlen(line) < 2 || *line == '#') {
 	return oflags;
     }
 
     get_cmdword(line, cmdword);
 
     if (!strcmp(cmdword, "end")) {
+	endblock = 1;
 	ci = end_block_ci(line);
     } else if (ar1_alias(cmdword)) {
 	ci = AR1;
@@ -1079,10 +1152,16 @@ gretlopt get_gretl_options (char *line, int *err)
 	ci = gretl_command_number(cmdword);
     }
 
+#if OPTDEBUG
+    fprintf(stderr, "get gretl_options: '%s' -> ci = %d\n", cmdword, ci);
+#endif
+
     /* some commands do not take a "flag", and "-%c" may have
        some other meaning */
     if (ci == 0 || ci == GENR || ci == PRINTF) {
 	return oflags;
+    } else if (!endblock && (ci == GMM || ci == MLE || ci == NLS)) {
+	return oflags;
     }
 
     /* smpl: in some contexts options don't make sense */
@@ -1162,6 +1241,11 @@ const char *print_flags (gretlopt oflags, int ci)
 	oflags &= ~OPT_O; /* handled */
     }
 
+    if ((oflags & OPT_Q) && quiet_opt_ok(ci)) {
+	pputs(flagprn, " --quiet");
+	oflags &= ~OPT_Q; /* handled */
+    }
+
     for (i=0; gretl_opts[i].ci != 0; i++) {
 	opt = gretl_opts[i].o;
 	if (ci == gretl_opts[i].ci && (oflags & opt)) {
