@@ -64,7 +64,8 @@ enum {
     P_ODBC,
     P_QUANTREG,
     P_INTREG,
-    P_ANOVA
+    P_ANOVA,
+    P_DURATION
 } plugin_codes;
 
 struct plugin_info {
@@ -115,7 +116,8 @@ struct plugin_info plugins[] = {
     { P_ODBC,            "odbc_import" },
     { P_QUANTREG,        "quantreg" },
     { P_INTREG,          "interval" },
-    { P_ANOVA,           "anova" }
+    { P_ANOVA,           "anova" },
+    { P_DURATION,        "duration" }
 };  
 
 struct plugin_function plugin_functions[] = { 
@@ -182,7 +184,7 @@ struct plugin_function plugin_functions[] = {
     { "arma_x12_model",    P_ARMA_X12 },
     { "tobit_estimate",    P_TOBIT },
     { "garch_model",       P_GARCH },
-    { "poisson_estimate",  P_POISSON },
+    { "count_data_estimate", P_POISSON },
     { "heckit_estimate",   P_HECKIT },
     { "interval_estimate", P_INTREG },
 
@@ -196,6 +198,7 @@ struct plugin_function plugin_functions[] = {
     /* kernel density estimation */
     { "kernel_density",        P_KERNEL },
     { "array_kernel_density",  P_KERNEL },
+    { "kernel_density_matrix", P_KERNEL },
 
     /* Hurst exponent estimation */
     { "hurst_exponent",    P_FRACTAL },
@@ -221,6 +224,9 @@ struct plugin_function plugin_functions[] = {
     /* analysis of variance */
     { "gretl_anova", P_ANOVA},
 
+    /* duration models */
+    { "duration_estimate", P_DURATION},
+
     /* sentinel */
     { NULL, 0 }
 };
@@ -289,7 +295,6 @@ void *gretl_dlsym (void *handle, const char *name)
 static void *get_plugin_handle (const char *plugin)
 {
     char pluginpath[MAXLEN];
-    void *handle = NULL;
 
     strcpy(pluginpath, gretl_lib_path());
 
@@ -302,16 +307,7 @@ static void *get_plugin_handle (const char *plugin)
     strcat(pluginpath, ".so");
 #endif 
 
-    handle = gretl_dlopen(pluginpath, 0);
-
-    if (handle == NULL) {
-        gretl_errmsg_sprintf(_("Failed to load plugin: %s"), pluginpath);
-#if !defined(WIN32) && !defined(OSX_NATIVE)
-	fprintf(stderr, "%s\n", dlerror());
-#endif
-    }     
-
-    return handle;
+    return gretl_dlopen(pluginpath, 0);
 }
 
 void *get_plugin_function (const char *funcname, void **handle)
