@@ -233,7 +233,7 @@ int max_obs_label_length (const DATAINFO *pdinfo)
 	/* we have specific observation strings */
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	    get_obs_string(s, t, pdinfo);
-	    n = strlen(s);
+	    n = g_utf8_strlen(s, -1);
 	    if (n > nmax) {
 		nmax = n;
 	    }
@@ -1280,7 +1280,7 @@ static void print_series_by_var (const double *z, const DATAINFO *pdinfo,
     char format[12];
     int t, ls = 0;
     int anyneg = 0;
-    double xx;
+    double x;
 
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	if (z[t] < 0) {
@@ -1299,12 +1299,17 @@ static void print_series_by_var (const double *z, const DATAINFO *pdinfo,
 	char str[32];
 	int n;
 
-	xx = z[t];
+	x = z[t];
 
-	if (na(xx)) {
+	if (na(x)) {
 	    sprintf(str, "%*s  ", GRETL_DIGITS + 1 + anyneg, "NA");
+	} else if (isnan(x)) {
+	    sprintf(str, "%*s  ", GRETL_DIGITS + 1 + anyneg, "NaN");
+	} else if (isinf(x)) {
+	    sprintf(str, "%*s  ", GRETL_DIGITS + 1 + anyneg, 
+		    (x < 0)? "-inf" : "inf");
 	} else {
-	    sprintf(str, format, xx);
+	    sprintf(str, format, x);
 	}
 
 	n = strlen(str);
@@ -1466,6 +1471,14 @@ static char *bufprintnum (char *buf, double x, int signif,
 
     *buf = '\0';
 
+    if (isnan(x)) {
+	strcpy(numstr, "NaN");
+	goto finish;
+    } else if (isinf(x)) {
+	strcpy(numstr, (x < 0)? "-inf" : "inf");
+	goto finish;
+    }
+
     /* guard against monster numbers that will smash the stack */
     if (fabs(x) > 1.0e20 || signif == PMAX_NOT_AVAILABLE) {
 	sprintf(numstr, "%.*g", gprec, x);
@@ -1883,6 +1896,7 @@ static int print_by_obs (int *list, const double **Z,
 	varheading(blist, obslen, colwidth, pdinfo, 0, prn);
 	
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	    int thislen = obslen;
 
 	    if (screenvar && Z[screenvar][t] == 0.0) {
 		/* screened out by boolean */
@@ -1893,11 +1907,14 @@ static int print_by_obs (int *list, const double **Z,
 		strcpy(obslabel, SORTED_MARKER(pdinfo, sortvar, t));
 	    } else if (opt & OPT_N) {
 		sprintf(obslabel, "%d", t + 1);
+	    } else if (dataset_has_markers(pdinfo)) {
+		strcpy(obslabel, pdinfo->S[t]);
+		thislen = get_utf_width(obslabel, obslen);
 	    } else {
-		get_obs_string(obslabel, t, pdinfo);
+		ntodate(obslabel, t, pdinfo);
 	    }
 
-	    pprintf(prn, "%*s", obslen, obslabel);
+	    pprintf(prn, "%*s", thislen, obslabel);
 
 	    for (i=1, j=j0; i<=blist[0]; i++, j++) {
 		x = Z[blist[i]][t];
@@ -2595,8 +2612,9 @@ int text_print_forecast (const FITRESID *fr, DATAINFO *pdinfo,
 
     if ((opt & OPT_P) && fr->nobs > 0) {
 	err = plot_fcast_errs(fr, maxerr, pdinfo, opt);
-	if (!err && (opt & OPT_G)) {
-	    pprintf(prn, _("wrote %s\n"), gretl_plotfile());
+	if (!err && (opt & OPT_U)) {
+	    /* specified output file for graph */
+	    report_plot_written(prn);
 	}
     }
 
@@ -2668,11 +2686,12 @@ print_iter_info (int iter, double crit, int type, int k,
 		 double sl, PRN *prn)
 {
     const char *cstrs[] = {
-	N_("Log-likelihood"),
+	N_("loglikelihood"),
 	N_("GMM criterion"),
 	N_("Criterion"),
     };
     const char *cstr = cstrs[type];
+    double x;
     int i;
 
     if (type == C_GMM) {
@@ -2704,10 +2723,12 @@ print_iter_info (int iter, double crit, int type, int k,
     pputc(prn, '\n');
 
     pputs(prn, _("Gradients:  "));
+    x = 0.0;
     for (i=0; i<k; i++) {
+	x += fabs(b[i] * g[i]);
 	print_iter_val(g[i], i, k, prn);
     }
-    pputs(prn, "\n\n");
+    pprintf(prn, " (norm %.2e)\n\n", sqrt(x/k));
 
     if (iter < 0 || (iter % 20 == 0)) {
 	/* experimental */
