@@ -38,7 +38,6 @@ double gamma_function (double x)
     double ret = cephes_gamma(x);
 
     if (get_cephes_errno()) {
-	fprintf(stderr, "gamma_function: arg = %g\n", x);
 	ret = NADBL;
     }
 
@@ -46,18 +45,35 @@ double gamma_function (double x)
 }
 
 /**
- * log_gamma_function:
+ * ln_gamma:
  * @x: argument.
  *
  * Returns: the log gamma function of @x, or #NADBL on failure.
  */
 
-double log_gamma_function (double x)
+double ln_gamma (double x)
 {
     double ret = cephes_lgamma(x);
 
     if (get_cephes_errno()) {
-	fprintf(stderr, "log_gamma_function: arg = %g\n", x);
+	ret = NADBL;
+    }
+
+    return ret;
+}
+
+/**
+ * digamma:
+ * @x: argument.
+ *
+ * Returns: the digamma (or Psi) function of @x, or #NADBL on failure.
+ */
+
+double digamma (double x)
+{
+    double ret = psi(x);
+
+    if (get_cephes_errno()) {
 	ret = NADBL;
     }
 
@@ -117,15 +133,15 @@ double binomial_cdf_comp (double p, int n, int k)
 /*  
     gives the event probability p such that the sum of the terms 0
     through k of the Binomial probability density, for n trials, is
-    equal to the given cumulative probability y.
+    equal to the given cumulative probability a.
 */
 
-static double binomial_cdf_inverse (double y, int n, int k)
+static double binomial_cdf_inverse (int n, int k, double a)
 {
     double p = NADBL;
 
-    if (y >= 0 && n >= 0 && k >= 0) {
-	p = bdtri(k, n, y);
+    if (a >= 0 && n >= 0 && k >= 0) {
+	p = bdtri(k, n, a);
 	if (get_cephes_errno()) {
 	    p = NADBL;
 	}
@@ -705,9 +721,9 @@ static double Binv (double p, double q)
     errno = 0;
 
     if (p > 0 && q > 0) {
-	double x1 = log_gamma_function(p + q);
-	double x2 = log_gamma_function(p);
-	double x3 = log_gamma_function(q);
+	double x1 = ln_gamma(p + q);
+	double x2 = ln_gamma(p);
+	double x3 = ln_gamma(q);
 
 	if (!na(x1) && !na(x2) && !na(x3)) {
 	    f = exp(x1 - x2 - x3);
@@ -765,7 +781,6 @@ static int snedecor_pdf_array (int v1, int v2, double *x, int n)
     return err;
 }
 
-
 static double snedecor_pdf (int m, int n, double x)
 {
     snedecor_pdf_array(m, n, &x, 1);
@@ -1034,32 +1049,31 @@ double normal_pdf (double x)
 
 double log_normal_pdf (double x)
 {
-    return (x * x) / 2 - LN_SQRT_2_PI;
+    return -0.5 * x * x - LN_SQRT_2_PI;
 }
 
 /**
  * bvnorm_cdf:
- * @a: input value.
- * @b: input value.
- * @rho: input value, correlation coefficient.
+ * @rho: correlation coefficient.
+ * @a: abscissa value, first Gaussian r.v.
+ * @b: abscissa value, second Gaussian r.v.
  *
  * Ripped and adapted from Gnumeric, with a bug corrected for the case
  * (a * b < 0) && (rho < 0).
  *
- * Returns: for (x,y) a bivariate standard Normal rv with correlation
- * coefficient rho, the joint probability that (x < a) && (y < b), or
+ * Returns: for (x, y) a bivariate standard Normal rv with correlation
+ * coefficient @rho, the joint probability that (x < @a) and (y < @b), or
  * #NADBL on failure.
  */
 
-double bvnorm_cdf (double a, double b, double rho)
+double bvnorm_cdf (double rho, double a, double b)
 {
     static const double x[] = {0.24840615, 0.39233107, 0.21141819, 
 			       0.03324666, 0.00082485334};
     static const double y[] = {0.10024215, 0.48281397, 1.0609498, 
 			       1.7797294, 2.6697604};
-
-    double den, a1, b1;
     double ret = NADBL;
+    double a1, b1, den;
     int i, j;
 
     if (fabs(rho) > 1) {
@@ -1070,8 +1084,22 @@ double bvnorm_cdf (double a, double b, double rho)
 	/* joint prob is just the product of the marginals */
 	return normal_cdf(a) * normal_cdf(b);
     }
-	
+
+    if (rho == 1.0) {
+	/* the two variables are in fact the same */
+	return normal_cdf(a<b ? a : b);
+    }
+    
+    if (rho == -1.0) {
+	/* the two variables are perfectly negatively correlated: 
+	   P(x<a, y<b) = P((x<a) && (x>b)) = P(x \in (b,a))
+	*/
+	ret = (a<=b) ? 0 : normal_cdf(a)-normal_cdf(b);
+	return ret;
+    }
+    
     den = sqrt(2.0 * (1 - rho * rho));
+
     a1 = a / den;
     b1 = b / den;
 
@@ -1089,11 +1117,11 @@ double bvnorm_cdf (double a, double b, double rho)
 	}
 	ret = (sqrt(1 - (rho * rho)) / M_PI * sum);
     } else if (a <= 0 && b >= 0 && rho > 0) {
-	ret = normal_cdf(a) - bvnorm_cdf(a, -b, -rho);
+	ret = normal_cdf(a) - bvnorm_cdf(-rho, a, -b);
     } else if (a >= 0 && b <= 0 && rho > 0) {
-	ret = normal_cdf(b) - bvnorm_cdf(-a, b, -rho);
+	ret = normal_cdf(b) - bvnorm_cdf(-rho, -a, b);
     } else if (a >= 0 && b >= 0 && rho < 0) {
-	ret = normal_cdf(a) + normal_cdf(b) - 1 + bvnorm_cdf(-a, -b, rho);
+	ret = normal_cdf(a) + normal_cdf(b) - 1 + bvnorm_cdf(rho, -a, -b);
     } else if ((a * b * rho) > 0) {
 	int sgna = (a < 0)? -1 : 1;
 	int sgnb = (b < 0)? -1 : 1;
@@ -1104,7 +1132,7 @@ double bvnorm_cdf (double a, double b, double rho)
 	rho2 = (rho * b - a) * sgnb / tmp;
 	delta = (sgna * sgnb && (rho > 0))? 0 : 0.5;
 
-	ret = (bvnorm_cdf(a, 0, rho1) + bvnorm_cdf(b, 0, rho2) - delta);
+	ret = (bvnorm_cdf(rho1, a, 0) + bvnorm_cdf(rho2, b, 0) - delta);
     }    
 
     return ret;
@@ -1404,7 +1432,7 @@ weibull_critval (double shape, double scale, double rtail)
  * the Weibull distribution with parameters @shape and @scale.
  */
 
-static double weibull_cdf (double shape, double scale, double x)
+double weibull_cdf (double shape, double scale, double x)
 {
     double ret = NADBL;
 
@@ -1436,43 +1464,32 @@ static double weibull_cdf_comp (double shape, double scale, double x)
 
 /* order in x: [params], alpha, critval */
 
-void print_critval (char st, double *x, double c, PRN *prn)
+void print_critval (char st, const double *parm, double a, double c, PRN *prn)
 {
-    int apos = 0;
-    double a;
-
     switch (st) {
     case 'z':
 	pprintf(prn, "%s", _("Standard normal distribution"));
 	break;
     case 't':
-	pprintf(prn, "t(%g)", x[0]);
-	apos = 1;
+	pprintf(prn, "t(%g)", parm[0]);
 	break;
     case 'X':
-	pprintf(prn, _("Chi-square(%g)"), x[0]);
-	apos = 1;
+	pprintf(prn, _("Chi-square(%g)"), parm[0]);
 	break;
     case 'F':
-	pprintf(prn, "F(%g, %g)", x[0], x[1]);
-	apos = 2;
+	pprintf(prn, "F(%g, %g)", parm[0], parm[1]);
 	break;
     case 'B':
-	pprintf(prn, "Binomial (P = %g, %g trials)", x[0], x[1]);
-	apos = 2;
+	pprintf(prn, "Binomial (P = %g, %g trials)", parm[0], parm[1]);
 	break;
     case 'P':
-	pprintf(prn, "Poisson (mean = %g)", x[0]);
-	apos = 1;
+	pprintf(prn, "Poisson (mean = %g)", parm[0]);
 	break;
     case 'W':
-	pprintf(prn, "Weibull (shape = %g, scale = %g)", x[0], x[1]);
-	apos = 2;
+	pprintf(prn, "Weibull (shape = %g, scale = %g)", parm[0], parm[1]);
 	break;
     }
 
-    a = x[apos];
-
     pputs(prn, "\n ");
     pprintf(prn, _("right-tail probability = %g"), a);
     pputs(prn, "\n ");
@@ -1486,162 +1503,320 @@ void print_critval (char st, double *x, double c, PRN *prn)
     pputc(prn, '\n');    
 }
 
-static double dparm[3];
+/* This apparatus is for use with the "batch p-value" 
+   routine: it remembers the parameters (p) and
+   argument (x) from the last internal p-value 
+   assessment.
+*/
 
-static void dparm_set (const double *p)
+static double pvargs[3];
+
+static void remember_pvalue_args (const double *p, double x)
 {
-    int i;
-
-    for (i=0; i<3; i++) {
-	dparm[i] = p[i];
-    }
+    pvargs[0] = p[0];
+    pvargs[1] = p[1];
+    pvargs[2] = x;
 }
 
-double gretl_get_cdf_inverse (char st, double *p)
+/* end remember parameters */
+
+static int pdist_check_input (char st, const double *parm,
+			      double x)
 {
-    double x = NADBL;
+    int i, np = 1;
+
+    if (na(x)) {
+	return E_MISSDATA;
+    }
 
     if (st == 'z') {
-	x = normal_cdf_inverse(p[0]);
+	np = 0;
+    } else if (st == 'F' || st == 'G' ||
+	       st == 'B' || st == 'W') {
+	np = 2;
+    }
+
+    for (i=0; i<np; i++) {
+	if (na(parm[i])) {
+	    return E_MISSDATA;
+	}
+    }
+
+    return 0;
+}
+
+/**
+ * gretl_get_cdf_inverse:
+ * @st: distribution code.
+ * @parm: array holding from zero to two parameter values, 
+ * depending on the distribution.
+ * @a: probability value.
+ *
+ * Returns: the argument, y, for which the area under the PDF
+ * specified by @st and @parm, integrated from its minimum to y,
+ * is equal to @a, or #NADBL on failure.
+ */
+
+double gretl_get_cdf_inverse (char st, const double *parm, 
+			      double a)
+{
+    double y = NADBL;
+
+    if (pdist_check_input(st, parm, a) == E_MISSDATA) {
+	return y;
+    }
+
+    if (st == 'z') {
+	y = normal_cdf_inverse(a);
     } else if (st == 't') {
-	x = student_cdf_inverse(p[0], p[1]);
+	y = student_cdf_inverse(parm[0], a);
     } else if (st == 'X') {
-	x = chisq_cdf_inverse((int) p[0], p[1]);
+	y = chisq_cdf_inverse((int) parm[0], a);
     } else if (st == 'F') {
-	x = snedecor_cdf_inverse((int) p[0], (int) p[1], p[2]);
+	y = snedecor_cdf_inverse((int) parm[0], (int) parm[1], a);
     } else if (st == 'B') {
-	x = binomial_cdf_inverse((int) p[0], (int) p[1], p[2]);
+	y = binomial_cdf_inverse((int) parm[0], (int) parm[1], a);
     } else if (st == 'P') {
-	x = poisson_cdf_inverse((int) p[0], p[1]);
+	y = poisson_cdf_inverse((int) parm[0], a);
     } 
 
-    return x;
+    return y;
 }
 
-double gretl_get_critval (char st, double *p)
+/**
+ * gretl_get_critval:
+ * @st: distribution code.
+ * @parm: array holding from zero to two parameter values, 
+ * depending on the distribution.
+ * @a: right-tail probability.
+ *
+ * Returns: the abcsissa value x for the distribution specified
+ * by @st and @parm, such that P(X >= x) = @a, or #NADBL on 
+ * failure.
+ */
+
+double gretl_get_critval (char st, const double *parm, double a)
 {
     double x = NADBL;
 
+    if (pdist_check_input(st, parm, a) == E_MISSDATA) {
+	return x;
+    }
+
     if (st == 'z') {
-	x = normal_critval(p[0]);
+	x = normal_critval(a);
     } else if (st == 't') {
-	x = student_critval(p[0], p[1]);
+	x = student_critval(parm[0], a);
     } else if (st == 'X') {	
-	x = chisq_critval((int) p[0], p[1]);
+	x = chisq_critval((int) parm[0], a);
     } else if (st == 'F') {
-	x = snedecor_critval((int) p[0], (int) p[1], p[2]);
+	x = snedecor_critval((int) parm[0], (int) parm[1], a);
     } else if (st == 'B') {
-	x = binomial_critval(p[0], (int) p[1], p[2]);
+	x = binomial_critval(parm[0], (int) parm[1], a);
     } else if (st == 'P') {
-	x = poisson_critval(p[0], p[1]);
+	x = poisson_critval(parm[0], a);
     } else if (st == 'W') {
-	x = weibull_critval(p[0], p[1], p[2]);
+	x = weibull_critval(parm[0], parm[1], a);
     } 
 
     return x;
 }
 
-double gretl_get_cdf (char st, double *p)
+/**
+ * gretl_get_cdf:
+ * @st: distribution code.
+ * @parm: array holding from zero to two parameter values, 
+ * depending on the distribution.
+ * @x: abscissa value.
+ *
+ * Evaluates the CDF for the distribution specified by
+ * @st and @parm applicable at @x. 
+ *
+ * Returns: the CDF value, or #NADBL on error.
+ */
+
+double gretl_get_cdf (char st, const double *parm, double x)
 {
-    double x = NADBL;
+    double y = NADBL;
+
+    if (pdist_check_input(st, parm, x) == E_MISSDATA) {
+	return y;
+    }    
 
     if (st == 'z') {
-	x = normal_cdf(p[0]);
+	y = normal_cdf(x);
     } else if (st == 't') {
-	x = student_cdf((int) p[0], p[1]);
+	y = student_cdf((int) parm[0], x);
     } else if (st == 'X') {
-	x = chisq_cdf((int) p[0], p[1]);
+	y = chisq_cdf((int) parm[0], x);
     } else if (st == 'F') {
-	x = snedecor_cdf((int) p[0], (int) p[1], p[2]);
+	y = snedecor_cdf((int) parm[0], (int) parm[1], x);
     } else if (st == 'G') {
-	x = gamma_cdf(p[0], p[1], p[2], 1);
+	y = gamma_cdf(parm[0], parm[1], x, 1);
     } else if (st == 'B') {
-	x = binomial_cdf(p[0], (int) p[1], (int) p[2]);
-    } else if (st == 'D') {
-	x = bvnorm_cdf(p[1], p[2], p[0]);
+	y = binomial_cdf(parm[0], (int) parm[1], (int) x);
     } else if (st == 'P') {
-	x = poisson_cdf(p[0], (int) p[1]);
+	y = poisson_cdf(parm[0], (int) x);
     } else if (st == 'W') {
-	x = weibull_cdf(p[0], p[1], p[2]);
+	y = weibull_cdf(parm[0], parm[1], x);
     }
 
-    return x;
+    return y;
 }
 
-double gretl_get_pdf (char st, double *p)
+/**
+ * gretl_get_pdf:
+ * @st: distribution code.
+ * @parm: array holding from zero to two parameter values, 
+ * depending on the distribution.
+ * @x: abscissa value.
+ *
+ * Evaluates the PDF for the distribution specified by
+ * @st and @parm at @x. 
+ *
+ * Returns: the PDF value, or #NADBL on error.
+ */
+
+double gretl_get_pdf (char st, const double *parm, double x)
 {
-    double x = NADBL;
+    double y = NADBL;
+
+    if (pdist_check_input(st, parm, x) == E_MISSDATA) {
+	return y;
+    } 
 
     if (st == 'z') {
-	x = normal_pdf(p[0]);
+	y = normal_pdf(x);
     } else if (st == 't') {
-	x = student_pdf(p[0], p[1]);
+	y = student_pdf(parm[0], x);
     } else if (st == 'X') {
-	x = chisq_pdf((int) p[0], p[1]);
+	y = chisq_pdf((int) parm[0], x);
     } else if (st == 'F') {
-	x = snedecor_pdf((int) p[0], (int) p[1], p[2]);
+	y = snedecor_pdf((int) parm[0], (int) parm[1], x);
     } else if (st == 'G') {
-	x = gamma_pdf(p[0], p[1], p[2]);
+	y = gamma_pdf(parm[0], parm[1], x);
     } else if (st == 'W') {
-	x = weibull_pdf(p[0], p[1], p[2]);
+	y = weibull_pdf(parm[0], parm[1], x);
     }
 
-    return x;
+    return y;
 }
 
-int gretl_fill_pdf_array (char st, double *p, double *x, int n)
+/**
+ * gretl_fill_pdf_array:
+ * @st: distribution code.
+ * @parm: array holding from zero to two parameter values,
+ * depending on the distribution.
+ * @x: see below.
+ * @n: number of elements in @x.
+ *
+ * On input, @x contains an array of abscissae at which the
+ * PDF specified by @st and @parm should be evaluated.  On
+ * output it contains the corresponding PDF values.
+ * 
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_fill_pdf_array (char st, const double *parm, 
+			  double *x, int n)
 {
     int err = E_DATA;
 
+    if (pdist_check_input(st, parm, 0) == E_MISSDATA) {
+	return E_MISSDATA;
+    } 
+
     if (st == 'z') {
 	err = normal_pdf_array(x, n);
     } else if (st == 't') {
-	err = student_pdf_array(p[0], x, n);
+	err = student_pdf_array(parm[0], x, n);
     } else if (st == 'X') {
-	err = chisq_pdf_array(p[0], x, n);
+	err = chisq_pdf_array(parm[0], x, n);
     } else if (st == 'F') {
-	err = snedecor_pdf_array((int) p[0], (int) p[1], x, n);
+	err = snedecor_pdf_array((int) parm[0], (int) parm[1], x, n);
     } else if (st == 'G') {
-	err = gamma_pdf_array(p[0], p[1], x, n); 
+	err = gamma_pdf_array(parm[0], parm[1], x, n); 
     } else if (st == 'W') {
-	err = weibull_pdf_array(p[0], p[1], x, n);
+	err = weibull_pdf_array(parm[0], parm[1], x, n);
     }
 
     return err;
 }
 
-double gretl_get_pvalue (char st, const double *p)
+/**
+ * gretl_get_pvalue:
+ * @st: distribution code.
+ * @parm: array holding from zero to two parameter values, 
+ * depending on the distribution.
+ * @x: abscissa value.
+ *
+ * Returns: the integral of the PDF specified by @st and
+ * @parm from @x to infinity, or #NADBL on error.
+ */
+
+double gretl_get_pvalue (char st, const double *parm, double x)
 {
-    double x = NADBL;
+    double y = NADBL;
+
+    if (pdist_check_input(st, parm, x) == E_MISSDATA) {
+	return y;
+    } 
 
     if (st == 'z') {
-	x = normal_cdf_comp(p[0]);
+	y = normal_cdf_comp(x);
     } else if (st == 't') {
-	x = student_cdf_comp((int) p[0], p[1]);
+	y = student_cdf_comp((int) parm[0], x);
     } else if (st == 'X') {
-	x = chisq_cdf_comp((int) p[0], p[1]);
+	y = chisq_cdf_comp((int) parm[0], x);
     } else if (st == 'F') {
-	x = snedecor_cdf_comp((int) p[0], (int) p[1], p[2]);
+	y = snedecor_cdf_comp((int) parm[0], (int) parm[1], x);
     } else if (st == 'G') {
-	x = gamma_cdf_comp(p[0], p[1], p[2], 1);
+	y = gamma_cdf_comp(parm[0], parm[1], x, 1);
     } else if (st == 'B') {
-	x = binomial_cdf_comp(p[0], (int) p[1], (int) p[2]);
+	y = binomial_cdf_comp(parm[0], (int) parm[1], x);
     } else if (st == 'P') {
-	x = poisson_cdf_comp(p[0], (int) p[1]);
+	y = poisson_cdf_comp(parm[0], x);
     } else if (st == 'W') {
-	x = weibull_cdf_comp(p[0], p[1], p[2]);
+	y = weibull_cdf_comp(parm[0], parm[1], x);
     }
 
-    dparm_set(p);
+    remember_pvalue_args(parm, x);
 
-    return x;
+    return y;
 }
 
+/**
+ * gretl_get_random_series:
+ * @st: distribution code.
+ * @parm: array holding either one or two scalar 
+ * parameter values, depending on the distribution.
+ * @serp1: series containing values for first param,
+ * or %NULL.
+ * @serp2: series containing values for second param,
+ * or %NULL.
+ * @pdinfo: dataset information.
+ * @err: location to receive error code.
+ *
+ * Produces a random series conforming to the distribution
+ * given by @st, which may require specification of either
+ * one or two parameters. These parameters are either
+ * given as (scalar) elements of @parm, or they may vary
+ * by observation, in which case they are given as the
+ * elements of @serp1 or @serp2.
+ *
+ * Returns: the array of pseudo-random values, or %NULL
+ * on error.
+ */
+
 double *gretl_get_random_series (char st, const double *p,
+				 const double *serp1, 
+				 const double *serp2,
 				 const DATAINFO *pdinfo,
 				 int *err)
 {
     double *x = malloc(pdinfo->n * sizeof *x);
+    int t1 = pdinfo->t1;
+    int t2 = pdinfo->t2;
     int t;
 
     if (x == NULL) {
@@ -1654,42 +1829,120 @@ double *gretl_get_random_series (char st, const double *p,
     }
 
     if (st == 'u') {
+	/* uniform */
 	double min = p[0], max = p[1];
 
-	*err = gretl_rand_uniform_minmax(x, pdinfo->t1, pdinfo->t2, min, max);
+	if (serp1 != NULL || serp2 != NULL) {
+	    for (t=t1; t<=t2 && !*err; t++) {
+		if (serp1 != NULL) min = serp1[t];
+		if (serp2 != NULL) max = serp2[t];
+		*err = gretl_rand_uniform_minmax(x, t, t, min, max);
+	    }
+	} else {
+	    *err = gretl_rand_uniform_minmax(x, t1, t2, min, max);
+	}
     } else if (st == 'z') {
+	/* normal */
 	double mu = p[0], sd = p[1];
 
-	*err = gretl_rand_normal_full(x, pdinfo->t1, pdinfo->t2, mu, sd);
+	if (serp1 != NULL || serp2 != NULL) {
+	    for (t=t1; t<=t2 && !*err; t++) {
+		if (serp1 != NULL) mu = serp1[t];
+		if (serp2 != NULL) sd = serp2[t];
+		*err = gretl_rand_normal_full(x, t, t, mu, sd);
+	    }
+	} else {
+	    *err = gretl_rand_normal_full(x, t1, t2, mu, sd);
+	}
     } else if (st == 't') {
+	/* Student's t */
 	int v = p[0];
 
-	*err = gretl_rand_student(x, pdinfo->t1, pdinfo->t2, v);
+	if (serp1 != NULL) {
+	    for (t=t1; t<=t2 && !*err; t++) {
+		v = serp1[t];
+		*err = gretl_rand_student(x, t, t, v);
+	    }
+	} else {	
+	    *err = gretl_rand_student(x, t1, t2, v);
+	}
     } else if (st == 'X') {
+	/* chi-square */
 	int v = p[0];
 
-	*err = gretl_rand_chisq(x, pdinfo->t1, pdinfo->t2, v);
+	if (serp1 != NULL) {
+	    for (t=t1; t<=t2 && !*err; t++) {
+		v = serp1[t];
+		*err = gretl_rand_chisq(x, t, t, v);
+	    }
+	} else {
+	    *err = gretl_rand_chisq(x, t1, t2, v);
+	}
     } else if (st == 'F') {
+	/* Snedecor F */
 	int v1 = p[0], v2 = p[1];
 
-	*err = gretl_rand_F(x, pdinfo->t1, pdinfo->t2, v1, v2);
+	if (serp1 != NULL || serp2 != NULL) {
+	    for (t=t1; t<=t2 && !*err; t++) {
+		if (serp1 != NULL) v1 = serp1[t];
+		if (serp2 != NULL) v2 = serp2[t];
+		*err = gretl_rand_F(x, t, t, v1, v2);
+	    }
+	} else {
+	    *err = gretl_rand_F(x, t1, t2, v1, v2);
+	}
     } else if (st == 'G') {
+	/* gamma */
 	double shape = p[0], scale = p[1];
 
-	*err = gretl_rand_gamma(x, pdinfo->t1, pdinfo->t2, shape, scale);
+	if (serp1 != NULL || serp2 != NULL) {
+	    for (t=t1; t<=t2 && !*err; t++) {
+		if (serp1 != NULL) shape = serp1[t];
+		if (serp2 != NULL) scale = serp2[t];
+		*err = gretl_rand_gamma(x, t, t, shape, scale);
+	    }
+	} else {
+	    *err = gretl_rand_gamma(x, t1, t2, shape, scale);
+	}
     } else if (st == 'B') {
+	/* binomial */
 	double pr = p[0];
 	int n = p[1];
 
-	*err = gretl_rand_binomial(x, pdinfo->t1, pdinfo->t2, n, pr);
+	if (serp1 != NULL || serp2 != NULL) {
+	    for (t=t1; t<=t2 && !*err; t++) {
+		if (serp1 != NULL) pr = serp1[t];
+		if (serp2 != NULL) n = serp2[1];
+		*err = gretl_rand_binomial(x, t, t, n, pr);
+	    }
+	} else {
+	    *err = gretl_rand_binomial(x, t1, t2, n, pr);
+	}
     } else if (st == 'P') {
-	double m = p[0]; /* FIXME? */
+	/* Poisson */
+	double m = p[0];
 
-	gretl_rand_poisson(x, pdinfo->t1, pdinfo->t2, &m, 0);
+	if (serp1 != NULL) {
+	    for (t=t1; t<=t2 && !*err; t++) {
+		m = serp1[t];
+		*err = gretl_rand_poisson(x, t, t, &m, 0);
+	    }
+	} else {
+	    *err = gretl_rand_poisson(x, t1, t2, &m, 0);
+	}
     } else if (st == 'W') {
+	/* Weibull */ 
 	double shape = p[0], scale = p[1];
 
-	*err = gretl_rand_weibull(x, pdinfo->t1, pdinfo->t2, shape, scale);
+	if (serp1 != NULL || serp2 != NULL) {
+	    for (t=t1; t<=t2 && !*err; t++) {
+		if (serp1 != NULL) shape = serp1[t];
+		if (serp2 != NULL) scale = serp2[t];
+		*err = gretl_rand_weibull(x, t, t, shape, scale);
+	    }
+	} else {
+	    *err = gretl_rand_weibull(x, t1, t2, shape, scale);
+	}
     }	
 
     return x;
@@ -1716,7 +1969,8 @@ print_pv_string (double x, double p, PRN *prn)
     return 0;
 }
 
-void print_pvalue (char st, double *p, double pv, PRN *prn)
+void print_pvalue (char st, const double *parm, double x,
+		   double pv, PRN *prn)
 {
     double pc;
     int err;
@@ -1728,13 +1982,13 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'N':
     case '1':
 	pprintf(prn, "%s: ", _("Standard normal"));
-	err = print_pv_string(p[0], pv, prn);
+	err = print_pv_string(x, pv, prn);
 	if (err) return;
 	if (pv < 0.5) {
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2 * pv, 1 - 2 * pv);
 	} else {
-	    pc = normal_cdf(p[0]);
+	    pc = normal_cdf(x);
 	    pprintf(prn, _("(to the left: %g)\n"), pc);
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2 * pc, 1 - 2 * pc);
@@ -1743,14 +1997,14 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
 
     case 't':
     case '2':
-	pprintf(prn, "t(%d): ", (int) p[0]);
-	err = print_pv_string(p[1], pv, prn);
+	pprintf(prn, "t(%d): ", (int) parm[0]);
+	err = print_pv_string(x, pv, prn);
 	if (err) return;
 	if (pv < 0.5) {
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2 * pv, 1 - 2 * pv);
 	} else {
-	    pc = student_cdf((int) p[0], p[1]);
+	    pc = student_cdf((int) parm[0], x);
 	    pprintf(prn, _("(to the left: %g)\n"), pc);
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2 * pc, 1 - 2 * pc);
@@ -1761,20 +2015,20 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'x':
     case 'c':
     case '3':
-	pprintf(prn, "%s(%d): ", _("Chi-square"), (int) p[0]);
-	err = print_pv_string(p[1], pv, prn);
+	pprintf(prn, "%s(%d): ", _("Chi-square"), (int) parm[0]);
+	err = print_pv_string(x, pv, prn);
 	if (err) return;
-	pc = chisq_cdf(p[0], p[1]);
+	pc = chisq_cdf(parm[0], x);
 	pprintf(prn, _("(to the left: %g)\n"), pc);
 	break;
 
     case 'F':
     case 'f':
     case '4':
-	pprintf(prn, "F(%d, %d): ", (int) p[0], (int) p[1]);
-	err = print_pv_string(p[2], pv, prn);
+	pprintf(prn, "F(%d, %d): ", (int) parm[0], (int) parm[1]);
+	err = print_pv_string(x, pv, prn);
 	if (err) return;
-	pc = snedecor_cdf((int) p[0], (int) p[1], p[2]);
+	pc = snedecor_cdf((int) parm[0], (int) parm[1], x);
 	pprintf(prn, _("(to the left: %g)\n"), pc);
 	break;
 
@@ -1783,8 +2037,9 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case '5':
 	pprintf(prn, _("Gamma (shape %g, scale %g, mean %g, variance %g):"
 		       "\n area to the right of %g = %g\n"), 
-		p[0], p[1], p[0] * p[1], p[0] * p[1] * p[1],
-		p[2], pv);
+		parm[0], parm[1], parm[0] * parm[1], 
+		parm[0] * parm[1] * parm[1],
+		x, pv);
 	break;
 
     case 'B':
@@ -1792,30 +2047,30 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case '6':
 	pprintf(prn, _("Binomial (p = %g, n = %d):"
 		       "\n Prob(x > %d) = %g\n"), 
-		p[0], (int) p[1], (int) p[2], pv);
-	pc = binomial_cdf(p[0], p[1], p[2]);
-	if (p[2] > 0) {
-	    pprintf(prn, _(" Prob(x <= %d) = %g\n"), (int) p[2], pc);
-	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) p[2],
-		    pc - binomial_cdf(p[0], p[1], p[2] - 1));
+		parm[0], (int) parm[1], (int) x, pv);
+	pc = binomial_cdf(parm[0], parm[1], x);
+	if (x > 0) {
+	    pprintf(prn, _(" Prob(x <= %d) = %g\n"), (int) x, pc);
+	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) x,
+		    pc - binomial_cdf(parm[0], parm[1], x - 1));
 	} else {
-	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) p[2], pc);
+	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) x, pc);
 	}
 	break;
 
     case 'p':
     case 'P':
     case '8':
-	pprintf(prn, _("Poisson (mean = %g): "), p[0]);
-	err = print_pv_string(p[1], pv, prn);
+	pprintf(prn, _("Poisson (mean = %g): "), parm[0]);
+	err = print_pv_string(x, pv, prn);
 	if (err) return;
-	pc = poisson_cdf(p[0], (int) p[1]);
-	if (p[1] > 0) {
-	    pprintf(prn, _(" Prob(x <= %d) = %g\n"), (int) p[1], pc);
-	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) p[1],
-		    poisson_pmf(p[0], p[1]));
+	pc = poisson_cdf(parm[0], (int) x);
+	if (x > 0) {
+	    pprintf(prn, _(" Prob(x <= %d) = %g\n"), (int) x, pc);
+	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) x,
+		    poisson_pmf(parm[0], x));
 	} else {
-	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) p[1], pc);
+	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) x, pc);
 	}
 	break;	
 
@@ -1823,10 +2078,10 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'W':
     case '9':
 	pprintf(prn, _("Weibull (shape = %g, scale = %g): "), 
-		p[0], p[1]);
-	err = print_pv_string(p[2], pv, prn);
+		parm[0], parm[1]);
+	err = print_pv_string(x, pv, prn);
 	if (err) return;
-	pc = weibull_cdf(p[0], p[1], p[2]);
+	pc = weibull_cdf(parm[0], parm[1], x);
 	pprintf(prn, _("(to the left: %g)\n"), pc);
 	break;
 
@@ -1861,7 +2116,7 @@ int batch_pvalue (const char *str,
 		  double ***pZ, DATAINFO *pdinfo, 
 		  PRN *prn)
 {
-    double x = NADBL;
+    double pv = NADBL;
     char line[MAXLEN];
     char **S;
     char st;
@@ -1894,11 +2149,11 @@ int batch_pvalue (const char *str,
     free_strings_array(S, n);
 
     if (!err) {
-	x = generate_scalar(line, pZ, pdinfo, &err);
+	pv = generate_scalar(line, pZ, pdinfo, &err);
     }
 
     if (!err) {
-	print_pvalue(st, dparm, x, prn);
+	print_pvalue(st, pvargs, pvargs[2], pv, prn);
     }
 
     return err;
