@@ -209,13 +209,13 @@ static int qr_make_vcv (MODEL *pmod, gretl_matrix *v, int flag)
 static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 				gretl_matrix *yhat, int fulln)
 {
-    int t, i = 0;
     int dwt = gretl_model_get_int(pmod, "wt_dummy");
     int qdiff = (pmod->rho != 0.0);
     int pwe = (pmod->opt & OPT_P);
     int yvar = pmod->list[1];
     double *u = pmod->uhat;
     double y;
+    int t, i = 0;
 
     if (dwt) {
 	dwt = pmod->nwt;
@@ -260,6 +260,8 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 	    }
 	}
     } else {
+	double sigma, z;
+
 	for (t=0; t<fulln; t++) {
 	    if (t < pmod->t1 || t > pmod->t2 || model_missing(pmod, t)) {
 		pmod->yhat[t] = u[t] = NADBL;
@@ -270,6 +272,17 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 		i++;
 	    }
 	}
+	
+	sigma = sqrt(pmod->ess / i);
+
+	for (t=0; t<fulln; t++) {
+	    if (na(u[t])) {
+		pmod->llt[t] = NADBL;
+	    } else {
+		z = u[t] / sigma;
+		pmod->llt[t] = -(LN_SQRT_2_PI + log(sigma) + 0.5*z*z);
+	    }
+	}
     }
 
     /* if SSR is small enough, treat it as zero */
@@ -802,14 +815,21 @@ allocate_model_arrays (MODEL *pmod, int k, int T)
     if (pmod->sderr == NULL) {
 	pmod->sderr = malloc(k * sizeof *pmod->sderr);
     }
+
     if (pmod->yhat == NULL) {
 	pmod->yhat = malloc(T * sizeof *pmod->yhat);
     }
+
     if (pmod->uhat == NULL) {
 	pmod->uhat = malloc(T * sizeof *pmod->uhat);
     }
 
-    if (pmod->sderr == NULL || pmod->yhat == NULL || pmod->uhat == NULL) {
+    if (pmod->llt == NULL) {
+	pmod->llt = malloc(T * sizeof *pmod->llt);
+    }
+
+    if (pmod->sderr == NULL || pmod->yhat == NULL || 
+	pmod->uhat == NULL || pmod->llt == NULL) {
 	return 1;
     }
 
