@@ -1481,8 +1481,7 @@ void free_strings_array (char **strs, int nstrs)
 
 char *get_obs_string (char *obs, int t, const DATAINFO *pdinfo)
 {
-    if (pdinfo->markers && pdinfo->S != NULL) { 
-	/* data marker strings present */
+    if (dataset_has_markers(pdinfo)) { 
 	strcpy(obs, pdinfo->S[t]);
     } else {
 	ntodate(obs, t, pdinfo);
@@ -1937,7 +1936,7 @@ const char *path_last_element (const char *path)
 
 #ifdef WIN32
     if (p == NULL) {
-	p = strrchr(path, '\\');
+	p = strrchr(path, '/');
     }
 #endif
 
