@@ -401,53 +401,43 @@ static void system_clear_results (equation_system *sys)
     sys->bdiff = 0.0;
 
     if (sys->b != NULL) {
-	gretl_matrix_free(sys->b);
-	sys->b = NULL;
+	gretl_matrix_replace(&sys->b, NULL);
     }
 
     if (sys->vcv != NULL) {
-	gretl_matrix_free(sys->vcv);
-	sys->vcv = NULL;
+	gretl_matrix_replace(&sys->vcv, NULL);
     }
 
     if (sys->S != NULL) {
-	gretl_matrix_free(sys->S);
-	sys->S = NULL;
+	gretl_matrix_replace(&sys->S, NULL);
     }
 
     if (sys->E != NULL) {
-	gretl_matrix_free(sys->E);
-	sys->E = NULL;
+	gretl_matrix_replace(&sys->E, NULL);
     }
 
     if (sys->yhat != NULL) {
-	gretl_matrix_free(sys->yhat);
-	sys->yhat = NULL;
+	gretl_matrix_replace(&sys->yhat, NULL);
     }
 
     if (sys->Gamma != NULL) {
-	gretl_matrix_free(sys->Gamma);
-	sys->Gamma = NULL;
+	gretl_matrix_replace(&sys->Gamma, NULL);
     }
 
     if (sys->B != NULL) {
-	gretl_matrix_free(sys->B);
-	sys->B = NULL;
+	gretl_matrix_replace(&sys->B, NULL);
     }
 
     if (sys->A != NULL) {
-	gretl_matrix_free(sys->A);
-	sys->A = NULL;
+	gretl_matrix_replace(&sys->A, NULL);
     }
 
     if (sys->Sr != NULL) {
-	gretl_matrix_free(sys->Sr);
-	sys->Sr = NULL;
+	gretl_matrix_replace(&sys->Sr, NULL);
     }    
 
     if (sys->F != NULL) {
-	gretl_matrix_free(sys->F);
-	sys->F = NULL;
+	gretl_matrix_replace(&sys->F, NULL);
     }
 
     if (sys->ldata != NULL) {
@@ -463,6 +453,10 @@ void equation_system_destroy (equation_system *sys)
 {
     int i;
 
+#if SYSDEBUG
+    fprintf(stderr, "equation_system_destroy: %p\n", (void *) sys);
+#endif
+
     if (sys == NULL || sys->lists == NULL) {
 	return;
     }
@@ -495,13 +489,8 @@ void equation_system_destroy (equation_system *sys)
 
     free(sys->name);
 
-    if (sys->R != NULL) {
-	gretl_matrix_free(sys->R);
-    }
-
-    if (sys->q != NULL) {
-	gretl_matrix_free(sys->q);
-    }
+    gretl_matrix_free(sys->R);
+    gretl_matrix_free(sys->q);
 
     system_clear_results(sys);
 
@@ -559,7 +548,7 @@ int equation_system_append (equation_system *sys,
     return 0;
 }
 
-/* retrieve the name -- possible quoted with embedded spaces -- for
+/* retrieve the name -- possibly quoted with embedded spaces -- for
    an equation system */
 
 char *get_system_name_from_line (const char *s, int context)
@@ -573,21 +562,12 @@ char *get_system_name_from_line (const char *s, int context)
     char *name = NULL;
     int pchars = 0;
 
-    if (context < 0 || context > 3) {
+    if (context < 0 || context > 2) {
 	return NULL;
     }
 
     p = strstr(s, tests[context]);
 
-    if (context == SYSNAME_NEW && p == NULL) {
-	char savename[MAXSAVENAME];
-
-	gretl_cmd_get_savename(savename);
-	if (*savename != '\0') {
-	    return gretl_strdup(savename);
-	} 
-    } 
-
     if (p == NULL) {
 	return NULL;
     }
@@ -661,21 +641,32 @@ static int get_estimation_method_from_line (const char *s)
 /**
  * equation_system_start:
  * @line: command line.
- * @opt: may include %OPT_I for iterative estimation, if the
- * estimation method supports this.
+ * @name: On input, name to be given to system, if any (otherwise
+ * may be %NULL or an empty string). On output, see below.
+ * on output, if non-%NULL, name given via "name=foo" mechanism in
+ * @line, if present.
+ * @opt: may include %OPT_I for iterative estimation (will be
+ * ignored if the the estimation method does not supports it).
  * 
- * Start compiling an equation system: @line must specify a "method" 
- * (estimation method) and/or a name for the system.  If a method is
- * given, the system will be estimated as soon as its definition is 
- * complete.  If a name is given, the system definition is saved on a 
- * stack, and it can subsequently be estimated via various methods
- * If both a name and an estimation method are given, the system 
+ * Start compiling an equation system. Either @line must contain
+ * an estimation method or the system must be given a name.
+ * If a method is given, the system will be estimated as soon as its 
+ * definition is complete. If a name is given, the system definition 
+ * is saved on a stack and it can subsequently be estimated via various 
+ * methods. If both a name and an estimation method are given, the system
  * is both estimated and saved.
+ *
+ * The name may be given via the @name argument, or (for backward
+ * compatibility) it may be given via "name=foo" in @line. In the
+ * latter case, if @name is non-%NULL, the name extracted from
+ * @line is written into that variable on output. The variable
+ * must be able to hold up to #MAXSAVENAME bytes.
  * 
  * Returns: pointer to a new equation system, or %NULL on error.
  */
 
 equation_system *equation_system_start (const char *line, 
+					char *name, 
 					gretlopt opt,
 					int *err)
 {
@@ -683,6 +674,10 @@ equation_system *equation_system_start (const char *line,
     char *sysname = NULL;
     int method;
 
+#if SYSDEBUG > 1
+    fprintf(stderr, "equation_system_start: '%s'\n", line);
+#endif
+
     method = get_estimation_method_from_line(line);
 
     if (method == SYS_METHOD_MAX) {
@@ -692,7 +687,13 @@ equation_system *equation_system_start (const char *line,
 	return NULL;
     }
 
-    sysname = get_system_name_from_line(line, SYSNAME_NEW);
+    if (name != NULL && *name != '\0') {
+	/* "foo <- system" */
+	sysname = gretl_strdup(name);
+    } else {
+	/* "system name=foo" (maybe) */
+	sysname = get_system_name_from_line(line, SYSNAME_NEW);
+    }
 
     if (method < 0 && sysname == NULL) {
 	/* neither a method nor a name was specified */
@@ -713,7 +714,14 @@ equation_system *equation_system_start (const char *line,
 	sys->flags |= SYSTEM_ITERATE;
     }
 
+#if SYSDEBUG > 1
+    fprintf(stderr, "new system '%s' at %p\n", sysname, (void *) sys);
+#endif
+
     if (sysname != NULL) {
+	if (name != NULL && *name == '\0') {
+	    strcpy(name, sysname);
+	}
 	free(sysname);
     }
 
@@ -745,19 +753,21 @@ system_print_F_test (const equation_system *sys,
 {
     const gretl_matrix *R = sys->R;
     const gretl_matrix *q = sys->q;
-    int Rrows = gretl_matrix_rows(R);
-    int dfu = system_get_dfu(sys);
-    int dfn = gretl_matrix_rows(R);
+    int Rrows, dfu, dfn;
     gretl_matrix *Rbq = NULL;
     gretl_matrix *RvR = NULL;
     double F = NADBL;
     int err = 0;
 
     if (R == NULL || q == NULL || b == NULL || vcv == NULL) {
-	pputs(prn, "Missing data in F test\n");
-	goto bailout;
+	pputs(prn, "Missing matrix in system F test!\n");
+	return;
     }
 
+    Rrows = gretl_matrix_rows(R);
+    dfu = system_get_dfu(sys);
+    dfn = gretl_matrix_rows(R);
+
     Rbq = gretl_matrix_alloc(Rrows, 1);
     RvR = gretl_matrix_alloc(Rrows, Rrows);
 
@@ -862,6 +872,10 @@ static int shrink_b_and_vcv (const gretl_matrix *b,
     double x;
     int i, j;
 
+    if (sys->vcv == NULL) {
+	return E_DATA;
+    }
+
     if (sys->vcv->rows == nc) {
 	/* no-op (shouldn't happen) */
 	return 0;
@@ -881,8 +895,7 @@ static int shrink_b_and_vcv (const gretl_matrix *b,
 	}
     }
 
-    gretl_matrix_free(sys->vcv);
-    sys->vcv = V;
+    gretl_matrix_replace(&sys->vcv, V);
 
     return 0;
 }
@@ -1095,7 +1108,7 @@ equation_system_estimate (equation_system *sys,
 	close_plugin(handle);
     }
 
-    if (!err && sys->neqns >= 2) {
+    if (!err && sys->neqns > 1) {
 	set_as_last_model(sys, GRETL_OBJ_SYS);
     } 
 
@@ -1458,8 +1471,10 @@ int equation_system_finalize (equation_system *sys,
 
     err = sys_check_lists(sys, (const double **) *pZ, pdinfo);
 
-    if (!err && sys->name != NULL && *sys->name != '\0') {
-	/* save the system for subsequent estimation */
+    if (!err && !(opt & OPT_S) && sys->name != NULL && *sys->name != '\0') {
+	/* save the system for subsequent estimation: note that we
+	   should not do this if given OPT_S, for single-equation
+	   LIML */
 	err = gretl_stack_object_as(sys, GRETL_OBJ_SYS, sys->name);
     }
 
@@ -1638,49 +1653,32 @@ int system_normality_test (const equation_system *sys, PRN *prn)
     return err;
 }
 
-static void
-add_system_var_info (equation_system *sys, int i, 
-		     DATAINFO *pdinfo, int v, int code)
+double *system_get_resid_series (equation_system *sys, int eqnum,
+				 DATAINFO *pdinfo, int *err)
 {
-    char *label = VARLABEL(pdinfo, v);
-
-    if (code == SYS_UHAT) {
-	sprintf(pdinfo->varname[v], "uhat_s%02d", i);
-	sprintf(label, _("system residual, equation %d"), i);
-    } else if (code == SYS_YHAT) {
-	sprintf(pdinfo->varname[v], "yhat_s%02d", i);
-	sprintf(label, _("system fitted value, equation %d"), i);
-    }
-}
-
-int 
-system_add_resids_to_dataset (equation_system *sys, 
-			      int eqnum, double ***pZ, 
-			      DATAINFO *pdinfo)
-{
-    int v, t;
+    double *u = NULL;
+    int t;
 
     if (sys->E == NULL) {
-	return E_DATA;
+	*err = E_DATA;
+	return NULL;
     }
 
-    if (dataset_add_series(1, pZ, pdinfo)) {
-	return E_ALLOC;
+    u = malloc(pdinfo->n * sizeof *u);
+    if (u == NULL) {
+	*err = E_ALLOC;
+	return NULL;
     }
 
-    v = pdinfo->v - 1;
-
     for (t=0; t<pdinfo->n; t++) {
 	if (t < sys->t1 || t > sys->t2) {
-	    (*pZ)[v][t] = NADBL;
+	    u[t] = NADBL;
 	} else {
-	    (*pZ)[v][t] = gretl_matrix_get(sys->E, t - sys->t1, eqnum);
+	    u[t] = gretl_matrix_get(sys->E, t - sys->t1, eqnum);
 	}
     }
 
-    add_system_var_info(sys, eqnum + 1, pdinfo, v, SYS_UHAT);
-
-    return 0;
+    return u;
 }
 
 static const char *system_get_full_string (const equation_system *sys,
@@ -1753,39 +1751,23 @@ int *system_get_instr_vars (const equation_system *sys)
 
 void system_attach_coeffs (equation_system *sys, gretl_matrix *b)
 {
-    if (sys->b != NULL) {
-	gretl_matrix_free(sys->b);
-    }
-
-    sys->b = b;
+    gretl_matrix_replace(&sys->b, b);
 }
 
 void system_attach_vcv (equation_system *sys, gretl_matrix *vcv)
 {
-    if (sys->vcv != NULL) {
-	gretl_matrix_free(sys->vcv);
-    }
-
-    sys->vcv = vcv;
+    gretl_matrix_replace(&sys->vcv, vcv);
     gretl_matrix_xtr_symmetric(sys->vcv);
 }
 
 void system_attach_sigma (equation_system *sys, gretl_matrix *S)
 {
-    if (sys->S != NULL) {
-	gretl_matrix_free(sys->S);
-    }
-
-    sys->S = S;
+    gretl_matrix_replace(&sys->S, S);
 }
 
 void system_attach_uhat (equation_system *sys, gretl_matrix *E)
 {
-    if (sys->E != NULL) {
-	gretl_matrix_free(sys->E);
-    }
-
-    sys->E = E;
+    gretl_matrix_replace(&sys->E, E);
 }
 
 MODEL *system_get_model (const equation_system *sys, int i)
@@ -2191,14 +2173,18 @@ system_parse_line (equation_system *sys, const char *line,
 
     gretl_error_clear();
 
+#if SYSDEBUG > 1
+    fprintf(stderr, "*** system_parse_line: '%s'\n", line);
+#endif
+
     if (strncmp(line, "identity", 8) == 0) {
 	err = add_identity_to_sys(sys, line + 8, pZ, pdinfo);
     } else if (strncmp(line, "endog", 5) == 0) {
 	err = add_aux_list_to_sys(sys, line + 5, pZ, pdinfo, ENDOG_LIST);
     } else if (strncmp(line, "instr", 5) == 0) {
 	err = add_aux_list_to_sys(sys, line + 5, pZ, pdinfo, INSTR_LIST);
-    } else {
-	err = E_PARSE;
+    } else if (strncmp(line, "system", 6) == 0) {
+	err = E_DATA;
     }
 
     if (err) {
@@ -2593,27 +2579,6 @@ int equation_system_serialize (equation_system *sys,
     return err;
 }
 
-void system_set_save_flag (equation_system *sys)
-{
-    sys->flags |= SYSTEM_SAVEIT;
-}
-
-void system_unset_save_flag (equation_system *sys)
-{
-    sys->flags &= ~SYSTEM_SAVEIT;
-}
-
-int system_save_flag_is_set (equation_system *sys)
-{
-    if (sys == NULL) {
-	return 0;
-    } else if (sys->flags & SYSTEM_SAVEIT) {
-	return 1;
-    } else {
-	return 0;
-    }
-}
-
 static int sur_ols_diag (equation_system *sys)
 {
     double s2, ls2sum = 0.0;
@@ -3497,8 +3462,7 @@ system_get_forecast_matrix (equation_system *sys, int t1, int t2,
 			    gretlopt opt, int *err)
 {
     if (sys->F != NULL) {
-	gretl_matrix_free(sys->F);
-	sys->F = NULL;
+	gretl_matrix_replace(&sys->F, NULL);
     }
 	
     *err = sys_add_forecast(sys, t1, t2, Z, pdinfo, opt);
