@@ -481,12 +481,14 @@ static int tsls_loglik (MODEL *pmod,
     if (!err) {
 	double ldet = gretl_matrix_log_determinant(W, &err);
 
-	if (!err) {
+	if (err) {
+	    pmod->lnL = NADBL;
+	} else {
 	    /* Davidson and MacKinnon, ETM, p. 538 */
 	    pmod->lnL = -(T / 2.0) * (LN_2_PI + ldet);
 	} 
     }
-
+    
     mle_criteria(pmod, 0);
 
     gretl_matrix_free(E);
@@ -563,25 +565,26 @@ ivreg_sargan_test (MODEL *pmod, int Orank, int *instlist,
     return err;
 }
 
+#define HTDBG 0
+
+/* Hausman test via the regression method */
+
 static int 
 tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 		   double ***pZ, DATAINFO *pdinfo)
 {
-    double RRSS;
-    int df, err = 0;
-
     MODEL hmod;
+    double URSS;
     int *HT_list = NULL;
+    int t, df, nobs1;
+    int nv = pdinfo->v;
+    int ku = 0;
+    int err = 0;
 
-    hmod = lsq(reglist, pZ, pdinfo, OLS, OPT_A);
-    if (hmod.errcode) {
-	fprintf(stderr, "tsls_hausman_test: hmod.errcode (R) = %d\n", hmod.errcode);
-	err = hmod.errcode;
-	goto bailout;
-    } 
-
-    RRSS = hmod.ess;
-    clear_model(&hmod);
+#if HTDBG
+    PRN *dbgprn = NULL;
+    dbgprn = gretl_print_new(GRETL_PRINT_STDOUT, NULL);
+#endif
 
     HT_list = gretl_list_add(reglist, hatlist, &err);
     if (err) {
@@ -591,24 +594,69 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 	} else {
 	    fprintf(stderr, "tsls_hausman_test: gretl_list_add: err = %d\n", err);
 	}
-	goto bailout;
+	return err;
     } 
 
+    /* estimate the unrestricted model */
     hmod = lsq(HT_list, pZ, pdinfo, OLS, OPT_A);
-
     if (hmod.errcode) {
 	fprintf(stderr, "tsls_hausman_test: hmod.errcode (U) = %d\n", hmod.errcode);
 	err = hmod.errcode;
 	goto bailout;
-    } else if ((df = hmod.list[0] - reglist[0]) > 0) {
+    } 
+
+#if HTDBG
+    pprintf(dbgprn, "Hausman: unrestricted model\n");
+    printmodel(&hmod, pdinfo, OPT_NONE, dbgprn);
+#endif
+
+    URSS = hmod.ess;
+    nobs1 = hmod.nobs;
+    ku = hmod.ncoeff;
+
+    /* add fitted values from unrestricted model to dataset */
+    err = dataset_add_series(1, pZ, pdinfo);
+    if (err) {
+	goto bailout;
+    } 
+
+    for (t=hmod.t1; t<=hmod.t2; t++) {
+	(*pZ)[nv][t] = hmod.yhat[t];
+    }
+
+    clear_model(&hmod);
+
+    free(HT_list);
+    HT_list = gretl_list_copy(reglist);
+    HT_list[1] = nv;
+
+    /* regress the U fitted values on the regressors of the restricted 
+       model (so the sum of squares equals RRSS-URSS)
+    */
+    hmod = lsq(HT_list, pZ, pdinfo, OLS, OPT_A);
+
+#if HTDBG
+    printmodel(&hmod, pdinfo, OPT_NONE, dbgprn);
+    pprintf(dbgprn, "smpl1 = %d, smpl2 = %d\n", nobs1, hmod.nobs);
+    pprintf(dbgprn, "k1 = %d, k2 = %d\n", hmod.ncoeff, ku);
+#endif
+
+    if (hmod.errcode) {
+	fprintf(stderr, "tsls_hausman_test: hmod.errcode (D) = %d\n", hmod.errcode);
+	err = hmod.errcode;
+    } else if ((df = ku - hmod.ncoeff) > 0) {
 	/* Degrees of freedom for the Hausman test need not be equal to the
 	   number of added regressors, since some of them may not really be 
 	   endogenous.
 	*/
-	double URSS = hmod.ess;
-	double HTest = (RRSS/URSS - 1) * hmod.nobs;
+	double DRSS = hmod.ess;
+	double HTest = (DRSS/URSS) * hmod.nobs;
 	ModelTest *test = model_test_new(GRETL_TEST_IV_HAUSMAN);
 
+#if HTDBG
+	pprintf(dbgprn, "(RRSS - URSS) = %g, URSS = %g\n", DRSS, URSS);
+#endif
+
 	if (test != NULL) {
 	    model_test_set_teststat(test, GRETL_STAT_WALD_CHISQ);
 	    model_test_set_dfn(test, df);
@@ -620,6 +668,11 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 
  bailout:
 
+#if HTDBG
+    gretl_print_destroy(dbgprn);
+#endif
+
+    dataset_drop_last_variables(pdinfo->v - nv, pZ, pdinfo);
     clear_model(&hmod);
     free(HT_list);
 
@@ -820,11 +873,11 @@ static void tsls_extra_stats (MODEL *pmod, int overid, const double **Z,
 
     pmod->rsq = gretl_corr_rsq(pmod->t1, pmod->t2, Z[yno], pmod->yhat);
 #else
-    pmod->rsq = 1.0 - pmod->ess / pmod->tss;
+    pmod->rsq = 1 - pmod->ess / pmod->tss;
 #endif
-    
-    r = 1.0 - pmod->rsq;
-    pmod->adjrsq = 1.0 - (r * (pmod->nobs - 1.0) / pmod->dfd);
+
+    r = 1 - pmod->rsq;
+    pmod->adjrsq = 1.0 - (r * (pmod->nobs - 1) / pmod->dfd);
 
     pmod->fstt = pmod->chisq = NADBL;
 
@@ -1059,6 +1112,14 @@ compute_stock_yogo (MODEL *pmod, const int *endolist,
 	}
     }
 
+    if (!err) {
+	double rc = gretl_symmetric_matrix_rcond(S, &err);
+
+	if (!err && (na(rc) || rc < 1.0e-6)) {
+	    err = E_SINGULAR;
+	}
+    }
+
     if (!err) {
 	/* S^{-1/2}: invert S and Cholesky-decompose */
 	err = gretl_invert_symmetric_matrix(S);
@@ -1310,7 +1371,7 @@ int ivreg_process_lists (const int *list, int **reglist, int **instlist)
     rlist = *reglist;
     zlist = *instlist;
 
-    if (rlist[0] < 2 || zlist[0] < 1) {
+    if (rlist[0] < 2 || zlist == NULL || zlist[0] < 1) {
 	err = E_ARGS;
     } else {
 	for (i=1; i<=zlist[0]; i++) {
@@ -1498,6 +1559,8 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
 
 	/* give the fitted series the same name as the original */
 	strcpy(pdinfo->varname[v1], pdinfo->varname[v0]);
+	strcpy(pdinfo->varname[v1], "h_");
+	strncat(pdinfo->varname[v1], pdinfo->varname[v0], VNAMELEN - 3);
 
 	/* substitute v1 into the right place in the second-stage
 	   regression list */
