@@ -64,9 +64,13 @@ user_matrix *get_user_matrix_by_name (const char *name);
 
 user_matrix *get_user_matrix_by_index (int idx);
 
+user_matrix *get_user_matrix_by_data (const gretl_matrix *M);
+
 int user_matrix_add (gretl_matrix *M, const char *name);
 
-int matrix_copy_add (gretl_matrix *M, const char *name);
+int private_matrix_add (gretl_matrix *M, const char *name);
+
+int matrix_add_as_shell (gretl_matrix *M, const char *name);
 
 int user_matrix_destroy_by_name (const char *name, PRN *prn);
 
@@ -130,6 +134,7 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
 gretl_matrix *user_matrix_ols (const gretl_matrix *Y, 
 			       const gretl_matrix *X, 
 			       const char *Uname, 
+			       const char *Vname,
 			       gretlopt opt,
 			       int *err);
 
