@@ -43,6 +43,7 @@ struct GRETL_VAR_ {
     int df;              /* T - average coeffs per equation */
     int ifc;             /* equations include a constant (1) or not (0) */
     int ncoeff;          /* total coefficients per equation */
+    int k;               /* maximum columns of Y (VECM special) */
     int *lags;           /* list of specific lags */
     int *ylist;          /* list of stochastic vars */
     int *xlist;          /* list of exogenous variables */
@@ -59,6 +60,7 @@ struct GRETL_VAR_ {
     gretl_matrix *C;     /* augmented Cholesky-decomposed error matrix */
     gretl_matrix *S;     /* cross-equation variance matrix */
     gretl_matrix *F;     /* optional forecast matrix */
+    gretl_matrix *ord;   /* optional Cholesky-ordering vector */
     MODEL **models;      /* pointers to individual equation estimates */
     double *Fvals;       /* hold results of F-tests */
     double *Ivals;       /* hold results of info criteria comparisons */
@@ -100,7 +102,8 @@ gretl_VAR_get_fcast_decomp (GRETL_VAR *var, int targ, int periods,
 			    int *errp);
 
 int 
-gretl_VAR_do_error_decomp (const gretl_matrix *S, gretl_matrix *C);
+gretl_VAR_do_error_decomp (const gretl_matrix *S, gretl_matrix *C,
+			   const gretl_matrix *ord);
 
 const gretl_matrix *gretl_VAR_get_roots (GRETL_VAR *var, int *err);
 
@@ -131,8 +134,7 @@ int default_VAR_horizon (const DATAINFO *pdinfo);
 gretl_matrix *
 gretl_VAR_get_impulse_response (GRETL_VAR *var, 
 				int targ, int shock,
-				int periods,
-				double alpha,
+				int periods, double alpha,
 				const double **Z,
 				const DATAINFO *pdinfo);
 
@@ -150,8 +152,10 @@ int gretl_VAR_get_t2 (const GRETL_VAR *var);
 
 const MODEL *gretl_VAR_get_model (const GRETL_VAR *var, int i);
 
-int gretl_VAR_add_resids_to_dataset (GRETL_VAR *var, int eqnum,
-				     double ***pZ, DATAINFO *pdinfo);
+double *gretl_VAR_get_resid_series (GRETL_VAR *var, int eqnum,
+				    int *err);
+
+int gretl_VAR_set_ordering (GRETL_VAR *var, gretl_matrix *ord);
 
 int gretl_VAR_do_irf (GRETL_VAR *var, const char *line,
 		      const double **Z, const DATAINFO *pdinfo);
