@@ -1286,6 +1286,8 @@ static int transcribe_data (wbook *book, double **Z, DATAINFO *pdinfo,
 	    sprintf(pdinfo->varname[j], "v%d", j);
 	} else if (rows[roff].cells[i] == NULL) {
 	    sprintf(pdinfo->varname[j], "v%d", j);
+	} else if (i >= rows[roff].end) {
+	    sprintf(pdinfo->varname[j], "v%d", j);
 	} else {
 	    strncat(pdinfo->varname[j], rows[roff].cells[i] + 1, 
 		    VNAMELEN - 1);
@@ -1466,6 +1468,7 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
     int ts_markers = 0;
     char **ts_S = NULL;
     int r0, c0;
+    int merge = (*pZ != NULL);
     int i, t, pd = 0;
     int err = 0;
 
@@ -1653,6 +1656,10 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
 
     err = merge_or_replace_data(pZ, pdinfo, &newZ, &newinfo, opt, prn);
 
+    if (!err && !merge) {
+	dataset_add_import_info(pdinfo, fname, GRETL_XLS);
+    }
+
     if (!err && gui) {
 	wbook_record_params(book, list);
     }
