@@ -1039,8 +1039,34 @@ static int adjust_ml_vcv_hyperbolic (h_container *HC)
     return err;
 }
 
+#define INITH_OPG 1
+
+#if INITH_OPG
+
+/* calculate the OPG matrix at the starting point and use 
+   its inverse (if any) as initial curvature matrix for BFGS
+*/
+
+static gretl_matrix *heckit_init_H (double *theta,
+				    h_container *HC,
+				    int np)
+{
+    gretl_matrix *H = NULL;
+    double ll = h_loglik(theta, HC);
+    int err = 0;
+
+    if (!na(ll)) {
+	H = gretl_matrix_GG_inverse(HC->score, &err);
+    }
+
+    return H;
+}
+
+#endif
+
 int heckit_ml (MODEL *hm, h_container *HC, PRN *prn)
 {
+    gretl_matrix *init_H = NULL;
     int maxit, fncount, grcount;
     double hij, rho, toler;
     double *hess = NULL;
@@ -1073,12 +1099,16 @@ int heckit_ml (MODEL *hm, h_container *HC, PRN *prn)
 
     BFGS_defaults(&maxit, &toler, HECKIT);
 
+#if INITH_OPG
+    init_H = heckit_init_H(theta, HC, np);
+#endif
+
     err = BFGS_max(theta, np, maxit, toler, &fncount, 
 		   &grcount, h_loglik, C_LOGLIK,
-		   heckit_score, HC, 
+		   heckit_score, HC, init_H,
 		   (prn != NULL)? OPT_V : OPT_NONE, prn);
 
-
+    gretl_matrix_free(init_H);
 
     if (!err) {
 	HC->ll = hm->lnL = h_loglik(theta, HC);
@@ -1274,11 +1304,16 @@ static MODEL heckit_init (h_container *HC, double ***pZ, DATAINFO *pdinfo)
 MODEL heckit_estimate (const int *list, double ***pZ, DATAINFO *pdinfo, 
 		       gretlopt opt, PRN *prn) 
 {
+    PRN *vprn = NULL;
     h_container *HC = NULL;
     MODEL hm;
     int oldv = pdinfo->v;
     int err = 0;
 
+    if (opt & OPT_V) {
+	vprn = prn;
+    }
+
     gretl_model_init(&hm);
 
     HC = h_container_new(list);
@@ -1298,7 +1333,7 @@ MODEL heckit_estimate (const int *list, double ***pZ, DATAINFO *pdinfo,
 	err = heckit_2step_vcv(HC, &hm);
     } else {
 	/* use MLE */
-	err = heckit_ml(&hm, HC, prn);
+	err = heckit_ml(&hm, HC, vprn);
 	if (!err) {
 	    err = transcribe_ml_vcv(&hm, HC);
 	}
